@@ -29,29 +29,28 @@
 
 static void fluid_settings_init(fluid_settings_t* settings);
 static void fluid_settings_hash_delete(void* value, int type);
-static int fluid_settings_tokenize(char* s, char *buf, char** ptr);
+static int fluid_settings_tokenize(const char* s, char *buf, char** ptr);
 
 
 typedef struct {
-  char* value;
-  char* def;
-  int hints;
-  fluid_list_t* options;
-  fluid_str_update_t update;
-  void* data;
-} fluid_str_setting_t;
+      char* value;
+      char* def;
+      int hints;
+      fluid_list_t* options;
+      fluid_str_update_t update;
+      void* data;
+      } fluid_str_setting_t;
 
 static fluid_str_setting_t*
-new_fluid_str_setting(char* value, char* def, int hints, fluid_str_update_t fun, void* data)
+new_fluid_str_setting(const char* value, const char* def, int hints, fluid_str_update_t fun, void* data)
       {
-      fluid_str_setting_t* str;
-      str = FLUID_NEW(fluid_str_setting_t);
-      str->value = (char*)(value ? FLUID_STRDUP(value) : 0);
-      str->def = def? FLUID_STRDUP(def) : 0;
-      str->hints = hints;
+      fluid_str_setting_t* str = FLUID_NEW(fluid_str_setting_t);
+      str->value   = (char*)(value ? FLUID_STRDUP(value) : 0);
+      str->def     = def ? FLUID_STRDUP(def) : 0;
+      str->hints   = hints;
       str->options = 0;
-      str->update = fun;
-      str->data = data;
+      str->update  = fun;
+      str->data    = data;
       return str;
       }
 
@@ -192,7 +191,7 @@ void fluid_settings_init(fluid_settings_t* settings)
   fluid_synth_settings(settings);
 }
 
-static int fluid_settings_tokenize(char* s, char *buf, char** ptr)
+static int fluid_settings_tokenize(const char* s, char *buf, char** ptr)
 {
   char *tokstr, *tok;
   int n = 0;
@@ -294,9 +293,11 @@ static int fluid_settings_set(fluid_settings_t* settings,
 
 /** returns 1 if the value has been registered correctly, 0
     otherwise */
-int fluid_settings_register_str(fluid_settings_t* settings, char* name, char* def, int hints,
-			       fluid_str_update_t fun, void* data)
-{
+
+int fluid_settings_register_str(fluid_settings_t* settings,
+   const char* name, const char* def, int hints,
+  fluid_str_update_t fun, void* data)
+      {
   int type;
   void* value;
   char* tokens[MAX_SETTINGS_TOKENS];
@@ -328,7 +329,7 @@ int fluid_settings_register_str(fluid_settings_t* settings, char* name, char* de
 
 /** returns 1 if the value has been register correctly, zero
     otherwise */
-int fluid_settings_register_num(fluid_settings_t* settings, char* name, double def,
+int fluid_settings_register_num(fluid_settings_t* settings, const char* name, double def,
 			       double min, double max, int hints,
 			       fluid_num_update_t fun, void* data)
 {
@@ -368,7 +369,7 @@ int fluid_settings_register_num(fluid_settings_t* settings, char* name, double d
 
 /** returns 1 if the value has been register correctly, zero
     otherwise */
-int fluid_settings_register_int(fluid_settings_t* settings, char* name, int def,
+int fluid_settings_register_int(fluid_settings_t* settings, const char* name, int def,
 			       int min, int max, int hints,
 			       fluid_int_update_t fun, void* data)
 {
@@ -406,7 +407,7 @@ int fluid_settings_register_int(fluid_settings_t* settings, char* name, int def,
   }
 }
 
-int fluid_settings_get_type(fluid_settings_t* settings, char* name)
+int fluid_settings_get_type(fluid_settings_t* settings, const char* name)
 {
   int type;
   void* value;
@@ -419,7 +420,7 @@ int fluid_settings_get_type(fluid_settings_t* settings, char* name)
   return (fluid_settings_get(settings, tokens, ntokens, &value, &type))? type : FLUID_NO_TYPE;
 }
 
-int fluid_settings_get_hints(fluid_settings_t* settings, char* name)
+int fluid_settings_get_hints(fluid_settings_t* settings, const char* name)
 {
   int type;
   void* value;
@@ -444,7 +445,7 @@ int fluid_settings_get_hints(fluid_settings_t* settings, char* name)
   }
 }
 
-int fluid_settings_is_realtime(fluid_settings_t* settings, char* name)
+int fluid_settings_is_realtime(fluid_settings_t* settings, const char* name)
 {
   int type;
   void* value;
@@ -470,7 +471,7 @@ int fluid_settings_is_realtime(fluid_settings_t* settings, char* name)
   }
 }
 
-int fluid_settings_setstr(fluid_settings_t* settings, char* name, char* str)
+int fluid_settings_setstr(fluid_settings_t* settings, const char* name, char* str)
 {
   char* tokens[MAX_SETTINGS_TOKENS];
   char buf[MAX_SETTINGS_LABEL+1];
@@ -508,7 +509,7 @@ int fluid_settings_setstr(fluid_settings_t* settings, char* name, char* str)
   }
 }
 
-int fluid_settings_getstr(fluid_settings_t* settings, char* name, char** str)
+int fluid_settings_getstr(fluid_settings_t* settings, const char* name, char** str)
 {
   int type;
   void* value;
@@ -528,7 +529,7 @@ int fluid_settings_getstr(fluid_settings_t* settings, char* name, char** str)
   return 0;
 }
 
-int fluid_settings_str_equal(fluid_settings_t* settings, char* name, char* s)
+int fluid_settings_str_equal(fluid_settings_t* settings, const char* name, const char* s)
 {
   int type;
   void* value;
@@ -547,7 +548,7 @@ int fluid_settings_str_equal(fluid_settings_t* settings, char* name, char* s)
 }
 
 char*
-fluid_settings_getstr_default(fluid_settings_t* settings, char* name)
+fluid_settings_getstr_default(fluid_settings_t* settings, const char* name)
 {
   int type;
   void* value;
@@ -566,7 +567,7 @@ fluid_settings_getstr_default(fluid_settings_t* settings, char* name)
   }
 }
 
-int fluid_settings_add_option(fluid_settings_t* settings, char* name, char* s)
+int fluid_settings_add_option(fluid_settings_t* settings, const char* name, char* s)
 {
   int type;
   void* value;
@@ -587,7 +588,7 @@ int fluid_settings_add_option(fluid_settings_t* settings, char* name, char* s)
   }
 }
 
-int fluid_settings_remove_option(fluid_settings_t* settings, char* name, char* s)
+int fluid_settings_remove_option(fluid_settings_t* settings, const char* name, char* s)
 {
   int type;
   void* value;
@@ -619,49 +620,45 @@ int fluid_settings_remove_option(fluid_settings_t* settings, char* name, char* s
   }
 }
 
-int fluid_settings_setnum(fluid_settings_t* settings, char* name, double val)
-{
-  int type;
-  void* value;
-  fluid_num_setting_t* setting;
-  char* tokens[MAX_SETTINGS_TOKENS];
-  char buf[MAX_SETTINGS_LABEL+1];
-  int ntokens;
+int fluid_settings_setnum(fluid_settings_t* settings, const char* name, double val)
+      {
+      int type;
+      void* value;
+      fluid_num_setting_t* setting;
+      char* tokens[MAX_SETTINGS_TOKENS];
+      char buf[MAX_SETTINGS_LABEL+1];
 
-  ntokens = fluid_settings_tokenize(name, buf, tokens);
+      int ntokens = fluid_settings_tokenize(name, buf, tokens);
 
-  if (fluid_settings_get(settings, tokens, ntokens, &value, &type)) {
+      if (fluid_settings_get(settings, tokens, ntokens, &value, &type)) {
+            if (type != FLUID_NUM_TYPE)
+                  return 0;
 
-    if (type != FLUID_NUM_TYPE) {
-      return 0;
-    }
+            setting = (fluid_num_setting_t*) value;
 
-    setting = (fluid_num_setting_t*) value;
+            if (val < setting->min)
+                  val = setting->min;
+            else if (val > setting->max)
+                  val = setting->max;
 
-    if (val < setting->min) {
-      val = setting->min;
-    } else if (val > setting->max) {
-      val = setting->max;
-    }
+            setting->value = val;
 
-    setting->value = val;
+            if (setting->update)
+                  (*setting->update)(setting->data, name, val);
 
-    if (setting->update) {
-      (*setting->update)(setting->data, name, val);
-    }
+            return 1;
 
-    return 1;
+            }
+      else {
+            /* insert a new setting */
+            fluid_num_setting_t* setting;
+            setting = new_fluid_num_setting(-1e10, 1e10, 0.0f, 0, 0, 0);
+            setting->value = val;
+            return fluid_settings_set(settings, tokens, ntokens, setting, FLUID_NUM_TYPE);
+            }
+      }
 
-  } else {
-    /* insert a new setting */
-    fluid_num_setting_t* setting;
-    setting = new_fluid_num_setting(-1e10, 1e10, 0.0f, 0, 0, 0);
-    setting->value = val;
-    return fluid_settings_set(settings, tokens, ntokens, setting, FLUID_NUM_TYPE);
-  }
-}
-
-int fluid_settings_getnum(fluid_settings_t* settings, char* name, double* val)
+int fluid_settings_getnum(fluid_settings_t* settings, const char* name, double* val)
 {
   int type;
   void* value;
@@ -681,7 +678,7 @@ int fluid_settings_getnum(fluid_settings_t* settings, char* name, double* val)
 }
 
 
-void fluid_settings_getnum_range(fluid_settings_t* settings, char* name, double* min, double* max)
+void fluid_settings_getnum_range(fluid_settings_t* settings, const char* name, double* min, double* max)
 {
   int type;
   void* value;
@@ -699,8 +696,7 @@ void fluid_settings_getnum_range(fluid_settings_t* settings, char* name, double*
   }
 }
 
-double
-fluid_settings_getnum_default(fluid_settings_t* settings, char* name)
+double fluid_settings_getnum_default(fluid_settings_t* settings, const char* name)
 {
   int type;
   void* value;
@@ -720,7 +716,7 @@ fluid_settings_getnum_default(fluid_settings_t* settings, char* name)
 }
 
 
-int fluid_settings_setint(fluid_settings_t* settings, char* name, int val)
+int fluid_settings_setint(fluid_settings_t* settings, const char* name, int val)
 {
   int type;
   void* value;
@@ -762,7 +758,7 @@ int fluid_settings_setint(fluid_settings_t* settings, char* name, int val)
   }
 }
 
-int fluid_settings_getint(fluid_settings_t* settings, char* name, int* val)
+int fluid_settings_getint(fluid_settings_t* settings, const char* name, int* val)
 {
   int type;
   void* value;
@@ -782,7 +778,7 @@ int fluid_settings_getint(fluid_settings_t* settings, char* name, int* val)
 }
 
 
-void fluid_settings_getint_range(fluid_settings_t* settings, char* name, int* min, int* max)
+void fluid_settings_getint_range(fluid_settings_t* settings, const char* name, int* min, int* max)
 {
   int type;
   void* value;
@@ -801,7 +797,7 @@ void fluid_settings_getint_range(fluid_settings_t* settings, char* name, int* mi
 }
 
 int
-fluid_settings_getint_default(fluid_settings_t* settings, char* name)
+fluid_settings_getint_default(fluid_settings_t* settings, const char* name)
 {
   int type;
   void* value;
@@ -823,7 +819,7 @@ fluid_settings_getint_default(fluid_settings_t* settings, char* name)
 
 
 
-void fluid_settings_foreach_option(fluid_settings_t* settings, char* name, void* data,
+void fluid_settings_foreach_option(fluid_settings_t* settings, const char* name, void* data,
 				  fluid_settings_foreach_option_t func)
 {
   int type;
