@@ -472,7 +472,7 @@ bool Score::importBB(const QString& name)
                   h->setBaseTpc(table[c.bass-1]);
             else
                   h->setBaseTpc(INVALID_TPC);
-            h->setExtension(c.extension);
+            h->setChordId(c.extension);
             h->buildText();
             m->add(h);
             }
@@ -548,7 +548,7 @@ int Score::processPendingNotes(QList<MNote*>* notes, int len, int track)
       Chord* chord = new Chord(this);
       chord->setTick(tick);
       chord->setTrack(track);
-      chord->setTickLen(len);
+      chord->setLen(len);
       Segment* s = measure->getSegment(chord);
       s->add(chord);
 
