@@ -3,7 +3,7 @@
 //  Linux Music Score Editor
 //  $Id: chord.cpp,v 1.6 2006/03/28 14:58:58 wschweer Exp $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2008 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -38,7 +38,10 @@
 #include "arpeggio.h"
 #include "score.h"
 #include "tremolo.h"
+#include "glissando.h"
 #include "staff.h"
+#include "utils.h"
+#include "articulation.h"
 
 //---------------------------------------------------------
 //   Stem
@@ -61,7 +64,7 @@ void Stem::draw(QPainter& p) const
       pen.setWidthF(lw);
       p.setPen(pen);
 
-      p.drawLine(QLineF(0.0, 0.0, 0.0, point(_len)));
+      p.drawLine(QLineF(0.0, 0.0, 0.0, point(_len + _userLen)));
       }
 
 //---------------------------------------------------------
@@ -73,6 +76,34 @@ void Stem::setLen(const Spatium& l)
       _len = l;
       }
 
+//---------------------------------------------------------
+//   write
+//---------------------------------------------------------
+
+void Stem::write(Xml& xml) const
+      {
+      xml.stag("Stem");
+      Element::writeProperties(xml);
+      if (_userLen.val() != 0.0)
+            xml.tag("userLen", _userLen.val());
+      xml.etag();
+      }
+
+//---------------------------------------------------------
+//   read
+//---------------------------------------------------------
+
+void Stem::read(QDomElement e)
+      {
+      for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
+            QString tag(e.tagName());
+            if (tag == "userLen")
+                  _userLen = Spatium(e.text().toDouble());
+            else if (!Element::readProperties(e))
+                  domError(e);
+            }
+      }
+
 //---------------------------------------------------------
 //   bbox
 //---------------------------------------------------------
@@ -80,8 +111,50 @@ void Stem::setLen(const Spatium& l)
 QRectF Stem::bbox() const
       {
       double w = point(score()->style()->stemWidth) * mag();
-      double l = point(_len);
-      return QRectF(-w * .5, 0, w, l);
+      double l = point(_len + _userLen);
+      return QRectF(-w * .5, 0, w, l).normalized();
+      }
+
+//---------------------------------------------------------
+//   startEdit
+//---------------------------------------------------------
+
+bool Stem::startEdit(Viewer*, const QPointF&)
+      {
+      return true;
+      }
+
+//---------------------------------------------------------
+//   updateGrips
+//---------------------------------------------------------
+
+void Stem::updateGrips(int* grips, QRectF* grip) const
+      {
+      *grips   = 1;
+      QPointF p(0.0, point(_len) + point(_userLen));
+      grip[0].translate(canvasPos() + p);
+      }
+
+//---------------------------------------------------------
+//   editDrag
+//---------------------------------------------------------
+
+void Stem::editDrag(int, const QPointF& delta)
+      {
+      _userLen += delta.y();
+      Chord* c = static_cast<Chord*>(parent());
+      if (c->hook())
+            c->hook()->move(0.0, delta.y());
+      }
+
+//---------------------------------------------------------
+//   resetUserOffsets
+//---------------------------------------------------------
+
+void Stem::resetUserOffsets()
+      {
+      _userLen = Spatium(0.0);
+      setUserOff(QPointF());
       }
 
 //---------------------------------------------------------
@@ -151,8 +224,48 @@ Chord::Chord(Score* s)
       _stemDirection = AUTO;
       _arpeggio      = 0;
       _tremolo       = 0;
+      _glissando     = 0;
       _noteType      = NOTE_NORMAL;
       _stemSlash     = 0;
+      _noStem        = false;
+      }
+
+Chord::Chord(const Chord& c)
+   : ChordRest(c)
+      {
+      for (ciNote i = c.notes.begin(); i != c.notes.end(); ++i)
+            add(new Note(*(i->second)));
+
+      foreach(const LedgerLine* ll, c._ledgerLines) {
+            LedgerLine* l = new LedgerLine(*ll);
+            l->setParent(this);
+            l->setTrack(track());
+            _ledgerLines.append(new LedgerLine(*ll));
+            }
+
+      _stem          = 0;
+      _hook          = 0;
+      _glissando     = 0;
+      _arpeggio      = 0;
+      _stemSlash     = 0;
+
+      _noStem = c._noStem;
+      if (c._stem)
+            add(new Stem(*(c._stem)));
+      if (c._hook)
+            add(new Hook(*(c._hook)));
+      if (c._glissando)
+            add(new Glissando(*(c._glissando)));
+      if (c._arpeggio)
+            add(new Arpeggio(*(c._arpeggio)));
+      if (c._stemSlash) {
+            _stemSlash = new StemSlash(*(c._stemSlash));
+            _stemSlash->setParent(this);
+            _stemSlash->setTrack(track());
+            }
+      _stemDirection = c._stemDirection;
+      _tremolo       = 0;
+      _noteType      = c._noteType;
       }
 
 //---------------------------------------------------------
@@ -165,6 +278,10 @@ Chord::~Chord()
             delete _arpeggio;
       if (_tremolo)
             delete _tremolo;
+      if (_glissando)
+            delete _glissando;
+      if (_stemSlash)
+            delete _stemSlash;
       }
 
 //---------------------------------------------------------
@@ -226,19 +343,35 @@ void Chord::add(Element* e)
       e->setParent(this);
       e->setTrack(track());
       if (e->type() == NOTE) {
-            Note* note = (Note*)e;
+            Note* note = static_cast<Note*>(e);
             notes.add(note);
             if (note->tieFor()) {
                   if (note->tieFor()->endNote())
                         note->tieFor()->endNote()->setTieBack(note->tieFor());
                   }
+            note->setType(duration());
             }
       else if (e->type() == ATTRIBUTE)
-            attributes.push_back((NoteAttribute*)e);
+            articulations.push_back(static_cast<Articulation*>(e));
       else if (e->type() == ARPEGGIO)
-            _arpeggio = (Arpeggio*)e;
-      else if (e->type() == TREMOLO)
-            _tremolo = (Tremolo*)e;
+            _arpeggio = static_cast<Arpeggio*>(e);
+      else if (e->type() == TREMOLO) {
+            Tremolo* tr = static_cast<Tremolo*>(e);
+            if (tr->twoNotes()) {
+                  Duration d  = duration().shift(-1);
+                  if (tr->chord1())
+                        tr->chord1()->setDuration(d);
+                  if (tr->chord2())
+                        tr->chord2()->setDuration(d);
+                  }
+            _tremolo = tr;
+            }
+      else if (e->type() == GLISSANDO)
+            _glissando = static_cast<Glissando*>(e);
+      else if (e->type() == STEM)
+            _stem = static_cast<Stem*>(e);
+      else if (e->type() == HOOK)
+            _hook = static_cast<Hook*>(e);
       else
             printf("Chord::add: unknown element\n");
       }
@@ -266,17 +399,28 @@ void Chord::remove(Element* e)
                   printf("Chord::remove() note %p not found!\n", e);
             }
       else if (e->type() == ATTRIBUTE) {
-            int idx = attributes.indexOf((NoteAttribute*)e);
-            if (idx == -1)
+            if (!articulations.removeOne(static_cast<Articulation*>(e)))
                   printf("Chord::remove(): attribute not found\n");
-            else {
-                  attributes.removeAt(idx);
-                  }
             }
       else if (e->type() == ARPEGGIO)
             _arpeggio = 0;
-      else if (e->type() == TREMOLO)
+      else if (e->type() == TREMOLO) {
+            Tremolo* tremolo = static_cast<Tremolo*>(e);
+            if (tremolo->twoNotes()) {
+                  Duration d       = duration().shift(1);
+                  if (tremolo->chord1())
+                        tremolo->chord1()->setDuration(d);
+                  if (tremolo->chord2())
+                        tremolo->chord2()->setDuration(d);
+                  }
             _tremolo = 0;
+            }
+      else if (e->type() == GLISSANDO)
+            _glissando = 0;
+      else if (e->type() == STEM)
+            _stem = 0;
+      else if (e->type() == HOOK)
+            _hook = 0;
       else
             printf("Chord::remove: unknown element\n");
       }
@@ -292,7 +436,7 @@ QRectF Chord::bbox() const
             _bbox |= i->second->bbox().translated(i->second->pos());
       foreach(LedgerLine* l, _ledgerLines)
             _bbox |= l->bbox().translated(l->pos());
-      for (ciAttribute i = attributes.begin(); i != attributes.end(); ++i)
+      for (ciArticulation i = articulations.begin(); i != articulations.end(); ++i)
             _bbox |= (*i)->bbox().translated((*i)->pos());
       if (_hook)
             _bbox |= _hook->bbox().translated(_hook->pos());
@@ -300,8 +444,12 @@ QRectF Chord::bbox() const
             _bbox |= _stem->bbox().translated(_stem->pos());
       if (_arpeggio)
             _bbox |= _arpeggio->bbox().translated(_arpeggio->pos());
+      if (_glissando)
+            _bbox |= _glissando->bbox().translated(_glissando->pos());
       if (_stemSlash)
             _bbox |= _stemSlash->bbox().translated(_stemSlash->pos());
+      if (_tremolo)
+            _bbox |= _tremolo->bbox().translated(_tremolo->pos());
       return _bbox;
       }
 
@@ -318,46 +466,27 @@ QRectF Chord::bbox() const
 
 void Chord::layoutStem1(ScoreLayout* layout)
       {
+      int istaff      = staffIdx();
       double _spatium = layout->spatium();
-      System* s      = segment()->measure()->system();
+      System* s       = segment()->measure()->system();
       if (s == 0)       //DEBUG
             return;
-      double sy      = s->staff(staffIdx())->bbox().y();
+      double sy      = s->staff(istaff)->bbox().y();
       Note* upnote   = upNote();
       Note* downnote = downNote();
 
-      double uppos   = s->staff(staffIdx() + upnote->staffMove())->bbox().y();
+      double uppos   = s->staff(istaff + upnote->staffMove())->bbox().y();
             uppos    = (uppos - sy)/_spatium * 2.0 + upnote->line();
 
-      double downpos = s->staff(staffIdx() + downnote->staffMove())->bbox().y();
+      double downpos = s->staff(istaff + downnote->staffMove())->bbox().y();
             downpos  = (downpos - sy)/_spatium * 2.0 + downnote->line();
 
       //-----------------------------------------
       //  process stem
       //-----------------------------------------
 
-      int ticks = tuplet() ? tuplet()->baseLen() : tickLen();
-      int hookIdx;
-
-      bool hasStem = true;
-      if (ticks < division/16)
-            hookIdx = 5;
-      else if (ticks < division/8)
-            hookIdx = 4;
-      else if (ticks < division/4)
-            hookIdx = 3;
-      else if (ticks < division/2)
-            hookIdx = 2;
-      else if (ticks < division)
-            hookIdx = 1;
-      else if (ticks < division*2)
-            hookIdx = 0;
-      else if (ticks < division*4)  // < 1/1
-            hookIdx = 0;
-      else {
-            hookIdx = 0;
-            hasStem = false;
-            }
+      bool hasStem = duration().hasStem() && !(_noStem || measure()->slashStyle(istaff));
+      int hookIdx  = hasStem ? duration().hooks() : 0;
 
       if (hasStem) {
             if (!_stem)
@@ -383,11 +512,9 @@ void Chord::layoutStem1(ScoreLayout* layout)
       if (hookIdx) {
             if (!up())
                   hookIdx = -hookIdx;
-            if (!_hook) {
-                  _hook = new Hook(score());
-                  _hook->setParent(this);
-                  _hook->setMag(mag());
-                  }
+            if (!_hook)
+                  setHook(new Hook(score()));
+            _hook->setMag(mag());
             _hook->setSubtype(hookIdx);
             qreal lw     = point(score()->style()->stemWidth) * .5;
             QPointF npos = (up() ? downNote() : upNote())->stemPos(up());
@@ -428,44 +555,38 @@ void Chord::layoutStem(ScoreLayout* layout)
       //  process stem
       //-----------------------------------------
 
-      int ticks = tuplet() ? tuplet()->baseLen() : tickLen();
-      int hookIdx;
       Spatium stemLen;
+      Spatium normalLen(3.5 * staffMag);  // stem length is one octave
 
-      Spatium normalLen(3.5 * staffMag);
+      int hookIdx  = _stem ? duration().hooks() : 0;
 
-      bool hasStem = true;
       if (_noteType != NOTE_NORMAL) {
             // stemLen = Spatium(2.5 * mag());
             stemLen = normalLen * score()->style()->graceNoteMag;
             }
       else {
-            stemLen = Spatium((up() ? uppos - 3.0 * staffMag : 3.0 * staffMag - downpos) * .5);
-            Spatium normalLen(3.5 * staffMag);
-            if (stemLen < normalLen)
+            if (up()) {
+                  stemLen = Spatium((uppos - 4.0 * staffMag) * .5);
+                  }
+            else {
+                  stemLen = Spatium((4.0 * staffMag - downpos) * .5);
+                  }
+
+            // stems in the "wrong" direction are shorter than one octave:
+            if (stemLen.val() < 0.0)
+                  if (hookIdx == 0)
+                        stemLen = Spatium(2.5 * staffMag);
+                  else if (hookIdx <= 2)
+                        stemLen = Spatium(3.0 * staffMag);  // 16th and 8th
+                  else
+                        stemLen = Spatium(3.5 * staffMag);  // 32nd, 64th, ...
+
+            else if (stemLen < normalLen)
                   stemLen = normalLen;
             }
-      if (ticks < division/16)
-            hookIdx = 5;
-      else if (ticks < division/8)
-            hookIdx = 4;
-      else if (ticks < division/4)
-            hookIdx = 3;
-      else if (ticks < division/2)
-            hookIdx = 2;
-      else if (ticks < division)
-            hookIdx = 1;
-      else if (ticks < division*2)
-            hookIdx = 0;
-      else if (ticks < division*4)  // < 1/1
-            hookIdx = 0;
-      else {
-            hookIdx = 0;
-            hasStem = false;
-            }
 
       int extraStemLen = hookIdx - 2;
-      if (_tremolo) {
+      if (_tremolo && !_tremolo->twoNotes()) {
             int extraStemLen2 = _tremolo->subtype();
             if (hookIdx == 0)
                   extraStemLen2 -= 1;
@@ -474,24 +595,21 @@ void Chord::layoutStem(ScoreLayout* layout)
             }
 
       if (extraStemLen > 0)
-            stemLen += extraStemLen * Spatium(0.5 * staffMag);
+            stemLen += extraStemLen * Spatium(.6 * staffMag);
 
       double headCorrection = 0.2;
 
       stemLen        += Spatium((downpos - uppos) * .5 - headCorrection);
-      double pstemLen = point(stemLen);
 
       QPointF npos = (up() ? downNote() : upNote())->stemPos(up());
 
       if (up())
-            npos += QPointF(0, -pstemLen);
+            stemLen *= -1.0;
 
-      if (hasStem) {
+      if (_stem) {
             _stem->setLen(stemLen);
             _stem->setPos(npos);
             }
-      else
-            setStem(0);
 
       if (_stemSlash) {
             double x = _stem->pos().x();
@@ -500,25 +618,15 @@ void Chord::layoutStem(ScoreLayout* layout)
             y += l;
             double h2 = l * .5;
             double w  = upnote->headWidth() * .7;
-            _stemSlash->setLine(QLineF(QPointF(x - w, y + h2), QPointF(x + w, y - h2)));
+            _stemSlash->setLine(QLineF(QPointF(x + w, y + h2), QPointF(x - w, y - h2)));
             }
 
       //-----------------------------------------
       //    process tremolo
       //-----------------------------------------
 
-      if (_tremolo) {
+      if (_tremolo)
             _tremolo->layout(layout);
-            qreal x  = npos.x();
-            if (!hasStem) {
-                  // center tremolo above note
-                  x = upnote->x() + upnote->headWidth() * .5;
-                  }
-            qreal y  = npos.y();
-            qreal h  = pstemLen;
-            qreal th = _tremolo->bbox().height();
-            _tremolo->setPos(x, y + h * .5 - th * .5);
-            }
 
       //-----------------------------------------
       //  process hook
@@ -528,8 +636,8 @@ void Chord::layoutStem(ScoreLayout* layout)
             if (!up())
                   hookIdx = -hookIdx;
             _hook->setSubtype(hookIdx);
-            qreal lw = point(score()->style()->stemWidth) * .5;
-            QPointF p = npos + QPointF(lw, up() ? -lw : pstemLen);
+            qreal lw  = point(score()->style()->stemWidth) * .5;
+            QPointF p = npos + QPointF(lw, point(_stem->stemLen()));
             _hook->setPos(p);
             }
       else
@@ -539,7 +647,7 @@ void Chord::layoutStem(ScoreLayout* layout)
 //---------------------------------------------------------
 //   addLedgerLine
 ///   Add a ledger line to a chord.
-///   \arg x          horizontal position of line start
+///   \arg x          center of note head
 ///   \arg staffIdx   determines the y origin
 ///   \arg line       vertical position of line
 //---------------------------------------------------------
@@ -549,6 +657,8 @@ void Chord::addLedgerLine(double x, int staffIdx, int line)
       double staffMag = score()->staff(staffIdx)->mag();
       LedgerLine* h   = new LedgerLine(score());
       Spatium len     = h->len() * staffMag;
+      if (_noteType != NOTE_NORMAL)
+            len *= score()->style()->graceNoteMag;
       h->setParent(this);
       h->setTrack(staffIdx * VOICES);
 
@@ -557,6 +667,8 @@ void Chord::addLedgerLine(double x, int staffIdx, int line)
       // Experimental:
       //  shorten ledger line to avoid collisions with accidentals
       //
+
+      x -= len.point() * .5;
       for (iNote in = notes.begin(); in != notes.end(); ++in) {
             Note* n = in->second;
             if (n->line() >= (line-1) && n->line() <= (line+1) && n->accidental()) {
@@ -582,6 +694,21 @@ void Chord::layout(ScoreLayout* layout)
       Note* upnote     = notes.back();
       double headWidth = upnote->headWidth();
 
+      if (!segment()) {
+            //
+            // hack for use in palette
+            //
+            for (iNote in = notes.begin(); in != notes.end(); ++in) {
+                  Note* note = in->second;
+                  note->layout(layout);
+
+                  double x = 0.0;
+                  double y = note->line() * _spatium * .5;
+                  note->setPos(x, y);
+                  }
+            return;
+            }
+
       //-----------------------------------------
       //  process notes
       //    - position
@@ -590,9 +717,9 @@ void Chord::layout(ScoreLayout* layout)
       int minMove = 1;
       int maxMove = -1;
 
-      double lx = 0.0;
-      System* s = segment()->measure()->system();
+      double lx       = 0.0;
       double staffMag = staff()->mag();
+      _dotPosX        = 0.0;
       for (iNote in = notes.begin(); in != notes.end(); ++in) {
             Note* note = in->second;
             note->layout(layout);
@@ -604,13 +731,23 @@ void Chord::layout(ScoreLayout* layout)
                   minMove = move;
             if (move > maxMove)
                   maxMove = move;
-            double y = s->staff(staffIdx() + move)->bbox().y();
-            y        -= s->staff(staffIdx())->bbox().y();
-            y        += in->second->line() * _spatium * .5 * staffMag;
 
+            double y = note->line() * _spatium * .5 * staffMag;
+
+            bool stemUp = isUp();
+            if (note->staffMove() == -1) {
+                  stemUp = false;
+                  }
+            else if (note->staffMove() == 1) {
+                  stemUp = true;
+                  }
             if (note->mirror())
-                  x += up() ? headWidth : - headWidth;
+                  x += stemUp ? headWidth : - headWidth;
+
             note->setPos(x, y);
+            double xx = x + headWidth;
+            if (xx > _dotPosX)
+                  _dotPosX = xx;
 
             Accidental* accidental = note->accidental();
             if (accidental) {
@@ -632,6 +769,9 @@ void Chord::layout(ScoreLayout* layout)
       //    moved to upper staff
       //---------------------------------------------------
 
+      double x1 = upnote->pos().x() + headWidth * .5; //  - _spatium * staffMag;
+      double x2 = notes.front()->pos().x() + headWidth * .5; // - _spatium * staffMag;
+
       int uppos;
       int downpos;
       if (minMove == -1) {
@@ -645,14 +785,12 @@ void Chord::layout(ScoreLayout* layout)
                               downpos = in->second->line();
                         }
                   }
-            if (uppos < 0 || downpos >= 10) {
-                  double x = upnote->pos().x();
-                  x += headWidth/2 - _spatium * staffMag;
 
+            if (uppos < 0 || downpos >= 10) {
                   for (int i = -2; i >= uppos; i -= 2)
-                        addLedgerLine(x, staffIdx() - 1, i);
+                        addLedgerLine(x1, staffIdx() - 1, i);
                   for (int i = 10; i <= downpos; i += 2)
-                        addLedgerLine(x, staffIdx() - 1, i);
+                        addLedgerLine(x2, staffIdx() - 1, i);
                   }
             }
 
@@ -666,20 +804,16 @@ void Chord::layout(ScoreLayout* layout)
                         downpos = in->second->line();
                   }
             }
-      if (uppos < 0 || downpos >= 10) {
-            double x = upnote->pos().x();
-            x += headWidth/2 - _spatium * staffMag;
 
-            for (int i = -2; i >= uppos; i -= 2) {
-                  addLedgerLine(x, staffIdx(), i);
-                  }
-            for (int i = 10; i <= downpos; i += 2) {
-                  addLedgerLine(x, staffIdx(), i);
-                  }
+      if (uppos < 0 || downpos >= 10) {
+            for (int i = -2; i >= uppos; i -= 2)
+                  addLedgerLine(x1, staffIdx(), i);
+            for (int i = 10; i <= downpos; i += 2)
+                  addLedgerLine(x2, staffIdx(), i);
             }
 
       //---------------------------------------------------
-      //    process help lines for notes
+      //    process ledger lines for notes
       //    moved to lower staff
       //---------------------------------------------------
 
@@ -695,22 +829,18 @@ void Chord::layout(ScoreLayout* layout)
                         }
                   }
             if (uppos < 0 || downpos >= 10) {
-                  double x = upnote->pos().x();
-                  x += headWidth/2 - _spatium * staffMag;
-
                   for (int i = -2; i >= uppos; i -= 2)
-                        addLedgerLine(x, staffIdx() + 1, i);
+                        addLedgerLine(x1, staffIdx() + 1, i);
                   for (int i = 10; i <= downpos; i += 2)
-                        addLedgerLine(x, staffIdx() * 1, i);
+                        addLedgerLine(x2, staffIdx() + 1, i);
                   }
             }
 
-      // LedgerLine does not use layout() method
-      // foreach(LedgerLine* l, _ledgerLines)
-      //      l->layout(layout);
+      foreach(LedgerLine* l, _ledgerLines)
+            l->layout(layout);
 
       //-----------------------------------------
-      //  Note Attributes
+      //  articulation
       //-----------------------------------------
 
       layoutAttributes(layout);
@@ -784,11 +914,17 @@ void Chord::computeUp()
             _up = true;
             return;
             }
-      if (tickLen() >= division*4) {
+      if (!duration().hasStem()) {
             _up = true;
             return;
             }
 
+      int si = staffIdx();
+      if (measure()->mstaff(si)->hasVoices) {
+            _up = score()->style()->stemDir[voice()] == UP;
+            return;
+            }
+
       Note* upnote = upNote();
       if (notes.size() == 1) {
             if (upnote->staffMove() > 0)
@@ -851,63 +987,72 @@ Note* Chord::selectedNote() const
 //   Chord::write
 //---------------------------------------------------------
 
-void Chord::write(Xml& xml) const
+void Chord::write(Xml& xml, int startTick, int endTick) const
       {
-      ciNote in = notes.begin();
-      Note* note = in->second;
+      int oldTickLen = tickLen();
+      int totalLen   = oldTickLen;
+      if (xml.clipboardmode) {
+            const Chord* c = this;
+            while (c->downNote()->tieFor()) {
+                  c = c->downNote()->tieFor()->endNote()->chord();
+                  totalLen += c->tickLen();
+                  }
+            }
+      setTickLen(totalLen);   // set temporary new tickLen
 
-      if (ChordRest::isSimple(xml)
-         && (notes.size() == 1)
-         && note->isSimple(xml)
-         && (_stemDirection == AUTO)
-         && !_arpeggio
-         && !_tremolo
-         && (_noteType == NOTE_NORMAL)
-         ) {
-            if (tick() != xml.curTick)
-                  xml.tagE(QString("Note tick=\"%1\" pitch=\"%2\" tpc=\"%3\" ticks=\"%4\"")
-                     .arg(tick()).arg(note->pitch()).arg(note->tpc()).arg(tickLen()));
-            else
-                  xml.tagE(QString("Note pitch=\"%1\" tpc=\"%2\" ticks=\"%3\"")
-                     .arg(note->pitch()).arg(note->tpc()).arg(tickLen()));
-            }
-      else {
-            xml.stag("Chord");
-            ChordRest::writeProperties(xml);
-            if (_noteType != NOTE_NORMAL) {
-                  switch(_noteType) {
-                        case NOTE_INVALID:
-                        case NOTE_NORMAL:
-                              break;
-                        case NOTE_ACCIACCATURA:
-                              xml.tagE("acciaccatura");
-                              break;
-                        case NOTE_APPOGGIATURA:
-                              xml.tagE("appoggiatura");
-                              break;
-                        }
+      xml.stag("Chord");
+      ChordRest::writeProperties(xml);
+      if (_noteType != NOTE_NORMAL) {
+            switch(_noteType) {
+                  case NOTE_INVALID:
+                  case NOTE_NORMAL:
+                        break;
+                  case NOTE_ACCIACCATURA:
+                        xml.tagE("acciaccatura");
+                        break;
+                  case NOTE_APPOGGIATURA:
+                        xml.tagE("appoggiatura");
+                        break;
+     	            case NOTE_GRACE4:
+                        xml.tagE("grace4");
+                        break;
+                  case NOTE_GRACE16:
+                        xml.tagE("grace16");
+                        break;
+                  case NOTE_GRACE32:
+                        xml.tagE("grace32");
+                        break;
                   }
-            switch(_stemDirection) {
-                  case UP:   xml.tag("StemDirection", QVariant("up")); break;
-                  case DOWN: xml.tag("StemDirection", QVariant("down")); break;
-                  case AUTO: break;
-                  }
-            for (; in != notes.end(); ++in)
-                  in->second->write(xml);
-            if (_arpeggio)
-                  _arpeggio->write(xml);
-            if (_tremolo)
-                  _tremolo->write(xml);
-            xml.etag();
             }
-      xml.curTick = tick() + tickLen();
+      if (_noStem)
+            xml.tag("noStem", _noStem);
+      if (_stem && (!_stem->userOff().isNull() || (_stem->userLen().point() != 0.0)))
+            _stem->write(xml);
+      switch(_stemDirection) {
+            case UP:   xml.tag("StemDirection", QVariant("up")); break;
+            case DOWN: xml.tag("StemDirection", QVariant("down")); break;
+            case AUTO: break;
+            }
+      ciNote in = notes.begin();
+      for (; in != notes.end(); ++in)
+            in->second->write(xml, startTick, endTick);
+      if (_arpeggio)
+            _arpeggio->write(xml);
+      if (_glissando)
+            _glissando->write(xml);
+      if (_tremolo)
+            _tremolo->write(xml);
+      xml.etag();
+      xml.curTick = tick() + totalLen;
+      if (xml.clipboardmode)
+            setTickLen(oldTickLen);
       }
 
 //---------------------------------------------------------
 //   Chord::readNote
 //---------------------------------------------------------
 
-void Chord::readNote(QDomElement e, int /*staffIdx*/)
+void Chord::readNote(QDomElement e, const QList<Tuplet*>& tuplets, const QList<Beam*>& beams)
       {
       Note* note = new Note(score());
       int ptch   = e.attribute("pitch", "-1").toInt();
@@ -954,23 +1099,37 @@ void Chord::readNote(QDomElement e, int /*staffIdx*/)
                   }
             else if (tag == "move")
                   note->setStaffMove(i);
-            else if (!ChordRest::readProperties(e))
+            else if (!ChordRest::readProperties(e, tuplets, beams))
                   domError(e);
             }
-      note->setParent(this);
-      note->setTrack(track());
       if (ptch != -1)
             note->setPitch(ptch);
       if (tpc != -1)
             note->setTpc(tpc);
-      notes.add(note);
+      if (!duration().isValid()) {
+            Duration dt;
+            headType(tickLen(), &dt, &_dots);
+            setDuration(dt);
+            }
+      add(note);
+      }
+
+//---------------------------------------------------------
+//   read
+//---------------------------------------------------------
+
+void Chord::read(QDomElement e)
+      {
+      QList<Tuplet*> tl;
+      QList<Beam*> bl;
+      read(e, tl, bl);
       }
 
 //---------------------------------------------------------
 //   Chord::read
 //---------------------------------------------------------
 
-void Chord::read(QDomElement e, int /*staffIdx*/)
+void Chord::read(QDomElement e, const QList<Tuplet*>& tuplets, const QList<Beam*>& beams)
       {
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
             QString tag(e.tagName());
@@ -984,12 +1143,18 @@ void Chord::read(QDomElement e, int /*staffIdx*/)
                   note->read(e);
                   notes.add(note);
                   }
-            else if (tag == "GraceNote")
-                  _noteType = NOTE_APPOGGIATURA;
+//            else if (tag == "GraceNote")
+//                  _noteType = NOTE_APPOGGIATURA;
             else if (tag == "appoggiatura")
                   _noteType = NOTE_APPOGGIATURA;
             else if (tag == "acciaccatura")
                   _noteType = NOTE_ACCIACCATURA;
+            else if (tag == "grace4")
+                  _noteType = NOTE_GRACE4;
+            else if (tag == "grace16")
+                  _noteType = NOTE_GRACE16;
+            else if (tag == "grace32")
+                  _noteType = NOTE_GRACE32;
             else if (tag == "StemDirection") {
                   if (val == "up")
                         _stemDirection = UP;
@@ -998,12 +1163,20 @@ void Chord::read(QDomElement e, int /*staffIdx*/)
                   else
                         _stemDirection = Direction(i);
                   }
+            else if (tag == "noStem")
+                  _noStem = i;
             else if (tag == "Arpeggio") {
                   _arpeggio = new Arpeggio(score());
                   _arpeggio->setTrack(track());
                   _arpeggio->read(e);
                   _arpeggio->setParent(this);
                   }
+            else if (tag == "Glissando") {
+                  _glissando = new Glissando(score());
+                  _glissando->setTrack(track());
+                  _glissando->read(e);
+                  _glissando->setParent(this);
+                  }
             else if (tag == "Tremolo") {
                   _tremolo = new Tremolo(score());
                   _tremolo->setTrack(track());
@@ -1012,9 +1185,26 @@ void Chord::read(QDomElement e, int /*staffIdx*/)
                   }
             else if (tag == "tickOffset")       // obsolete
                   ;
-            else if (!ChordRest::readProperties(e))
+            else if (tag == "Stem") {
+                  _stem = new Stem(score());
+                  _stem->read(e);
+                  add(_stem);
+                  }
+            else if (!ChordRest::readProperties(e, tuplets, beams))
                   domError(e);
             }
+      int len = tickLen();
+      if (tuplet())
+            len = (len * (tuplet()->actualNotes()) + (tuplet()->normalNotes()/2)) / tuplet()->normalNotes();
+      if (!duration().isValid()) {
+            Duration dt;
+            headType(len, &dt, &_dots);
+            setDuration(dt);
+            }
+      for (iNote i = notes.begin(); i != notes.end(); ++i) {
+            Note* note = i->second;
+            note->setType(duration());
+            }
       }
 
 //---------------------------------------------------------
@@ -1038,6 +1228,8 @@ void Chord::space(double& min, double& extra) const
 
       if (_arpeggio)
             extra = _arpeggio->width() + _spatium * .5;
+      if (_glissando)
+            extra += _spatium * .5;
 
       for (ciNote i = notes.begin(); i != notes.end(); ++i) {
             Note* note = i->second;
@@ -1047,12 +1239,8 @@ void Chord::space(double& min, double& extra) const
             double prefixWidth  = 0.0;
             if (note->accidental()) {
                   prefixWidth = -note->accidental()->pos().x();
-#if 0
-                  prefixWidth = note->accidental()->width();
-                  prefixWidth += point(score()->style()->prefixNoteDistance) * mag();
-#endif
                   if (prefixWidth > extra)
-                        extra += prefixWidth;
+                        extra = prefixWidth;
                   }
             if (note->mirror()) {
                   if (up()) {
@@ -1062,18 +1250,13 @@ void Chord::space(double& min, double& extra) const
                   else {
                         // note head on left side of stem
                         if ((lhw + prefixWidth) > extra)
-                              extra += lhw + prefixWidth;
+                              extra = lhw + prefixWidth;
                         }
                   }
             }
       min = mirror + hw;
       if (up() && _hook)
             min += _hook->width();
-/*      if (_noteType != NOTE_NORMAL) {
-            extra += min;
-            min = 0.0;
-            }
-      */
       }
 
 //---------------------------------------------------------
@@ -1117,20 +1300,16 @@ qreal Chord::downPos() const
 
 //---------------------------------------------------------
 //   centerX
+//    return x position for attributes
 //---------------------------------------------------------
 
 qreal Chord::centerX() const
       {
-      qreal x;
-      if (_up) {
-            const Note* upnote = upNote();
-            x  = upnote->pos().x();
-            x += upnote->headWidth() * .5;
-            }
-      else {
-            const Note* downnote = downNote();
-            x = downnote->pos().x();
-            x += downnote->headWidth() * .5;
+      const Note* note = isUp() ? upNote() : downNote();
+      qreal x = note->pos().x();
+      x += note->headWidth() * .5;
+      if (note->mirror()) {
+            x += note->headWidth() * (isUp() ? -1.0 : 1.0);
             }
       return x;
       }
@@ -1149,8 +1328,12 @@ void Chord::collectElements(QList<const Element*>& el) const
             el.append(_stemSlash);
       if (_arpeggio)
             el.append(_arpeggio);
-      if (_tremolo)
+      if (_tremolo) {
+            QRectF r(_tremolo->abbox());
             el.append(_tremolo);
+            }
+      if (_glissando)
+            el.append(_glissando);
 
       foreach(LedgerLine* h, _ledgerLines)
             el.append(h);
@@ -1174,6 +1357,8 @@ void Chord::setTrack(int val)
             _stemSlash->setTrack(val);
       if (_arpeggio)
             _arpeggio->setTrack(val);
+      if (_glissando)
+            _glissando->setTrack(val);
       if (_tremolo)
             _tremolo->setTrack(val);
 
@@ -1184,6 +1369,18 @@ void Chord::setTrack(int val)
             in->second->setTrack(val);
       }
 
+//---------------------------------------------------------
+//   setDuration
+//---------------------------------------------------------
+
+void Chord::setDuration(Duration t)
+      {
+      DurationElement::setDuration(t);
+      NoteList* nl = noteList();
+      for (iNote in = nl->begin(); in != nl->end(); ++in)
+            in->second->setType(t);
+      }
+
 //---------------------------------------------------------
 //   LedgerLine
 //---------------------------------------------------------
@@ -1191,8 +1388,8 @@ void Chord::setTrack(int val)
 LedgerLine::LedgerLine(Score* s)
    : Line(s, false)
       {
-      setLineWidth(score()->style()->ledgerLineWidth);
-      setLen(Spatium(2));
+      setLineWidth(score()->style()->ledgerLineWidth * mag());
+      setLen(Spatium(2.0) * mag());
       }
 
 //---------------------------------------------------------
