@@ -32,150 +32,75 @@
 #include "sym.h"
 #include "slur.h"
 #include "beam.h"
+#include "breath.h"
+#include "barline.h"
+#include "articulation.h"
+#include "tempotext.h"
 
 //---------------------------------------------------------
-//   NoteAttribute::atrList
+//   DurationElement
 //---------------------------------------------------------
 
-AttributeInfo NoteAttribute::atrList[] = {
-      { ufermataSym,       QString("ufermata"),        A_TOP_STAFF    },
-      { dfermataSym,       QString("dfermata"),        A_BOTTOM_STAFF },
-      { thumbSym,          QString("thumb"),           A_CHORD        },
-      { sforzatoaccentSym, QString("sforzato"),        A_CHORD        },
-      { esprSym,           QString("espressivo"),      A_CHORD        },
-      { staccatoSym,       QString("staccato"),        A_CHORD        },
-      { ustaccatissimoSym, QString("ustaccatissimo"),  A_CHORD },
-      { dstaccatissimoSym, QString("dstaccatissimo"),  A_CHORD },
-      { tenutoSym,         QString("tenuto"),          A_CHORD },
-      { uportatoSym,       QString("uportato"),        A_CHORD },
-      { dportatoSym,       QString("dportato"),        A_CHORD },
-      { umarcatoSym,       QString("umarcato"),        A_CHORD },
-      { dmarcatoSym,       QString("dmarcato"),        A_CHORD },
-      { ouvertSym,         QString("ouvert"),          A_CHORD },
-      { plusstopSym,       QString("plusstop"),        A_CHORD },
-      { upbowSym,          QString("upbow"),           A_CHORD },
-      { downbowSym,        QString("downbow"),         A_CHORD },
-      { reverseturnSym,    QString("reverseturn"),     A_CHORD },
-      { turnSym,           QString("turn"),            A_CHORD },
-      { trillSym,          QString("trill"),           A_TOP_STAFF },
-      { prallSym,          QString("prall"),           A_TOP_STAFF },
-      { mordentSym,        QString("mordent"),         A_TOP_STAFF },
-      { prallprallSym,     QString("prallprall"),      A_TOP_STAFF },
-      { prallmordentSym,   QString("prallmordent"),    A_TOP_STAFF },
-      { upprallSym,        QString("upprall"),         A_TOP_STAFF },
-	{ downprallSym,      QString("downprall"),       A_TOP_STAFF },
-	{ upmordentSym,      QString("upmordent"),       A_TOP_STAFF },
-	{ downmordentSym,    QString("downmordent"),     A_TOP_STAFF },
-	};
-
-//---------------------------------------------------------
-//   NoteAttribute
-//---------------------------------------------------------
-
-NoteAttribute::NoteAttribute(Score* s)
-   : Symbol(s)
+DurationElement::DurationElement(Score* s)
+   : Element(s)
       {
+      _tuplet = 0;
       }
 
 //---------------------------------------------------------
-//   setSubtype
+//   DurationElement
 //---------------------------------------------------------
 
-void NoteAttribute::setSubtype(int idx)
+DurationElement::DurationElement(const DurationElement& e)
+   : Element(e)
       {
-      Element::setSubtype(idx);
-      setSym(atrList[subtype()].sym);
+      _duration = e._duration;
+      _tuplet   = e._tuplet;
       }
 
 //---------------------------------------------------------
-//   read
+//   hasArticulation
 //---------------------------------------------------------
 
-void NoteAttribute::read(QDomElement e)
-      {
-      for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
-            QString tag(e.tagName());
-            if (tag == "idx")                         // obsolete
-                  setSubtype(e.text().toInt());
-            else if (!Element::readProperties(e))
-                  domError(e);
-            }
-      setSubtype(subtype());
-      }
-
-//---------------------------------------------------------
-//   write
-//---------------------------------------------------------
-
-void NoteAttribute::write(Xml& xml) const
-      {
-      xml.stag("Attribute");
-      Element::writeProperties(xml);
-      xml.etag();
-      }
-
-//---------------------------------------------------------
-//   hasAttribute
-//---------------------------------------------------------
-
-NoteAttribute* ChordRest::hasAttribute(const NoteAttribute* a)
+Articulation* ChordRest::hasArticulation(const Articulation* a)
       {
       int idx = a->subtype();
-      for (iAttribute l = attributes.begin(); l != attributes.end(); ++l) {
+      for (iArticulation l = articulations.begin(); l != articulations.end(); ++l) {
             if (idx == (*l)->subtype())
                   return *l;
             }
       return 0;
       }
 
-//---------------------------------------------------------
-//   subtypeName
-//---------------------------------------------------------
-
-const QString NoteAttribute::subtypeName() const
-      {
-      return atrList[subtype()].name;
-      }
-
-//---------------------------------------------------------
-//   setSubtype
-//---------------------------------------------------------
-
-void NoteAttribute::setSubtype(const QString& s)
-      {
-      if (s[0].isDigit()) {         // for backward compatibility
-            setSubtype(s.toInt());
-            return;
-            }
-      for (int i = 0; i < NOTE_ATTRIBUTES; ++i) {
-            if (atrList[i].name == s) {
-                  setSubtype(i);
-                  return;
-                  }
-            }
-      printf("NoteAttribute <%s> unknown\n", qPrintable(s));
-      }
-
 //---------------------------------------------------------
 //   ChordRest
 //---------------------------------------------------------
 
 ChordRest::ChordRest(Score* s)
-   : Element(s)
+   : DurationElement(s)
       {
       _beam     = 0;
-      _tuplet   = 0;
       _small    = false;
       _beamMode = BEAM_AUTO;
+      _dots     = 0;
+      _up       = true;
       }
 
-//---------------------------------------------------------
-//   up
-//---------------------------------------------------------
-
-bool ChordRest::up() const
+ChordRest::ChordRest(const ChordRest& cr)
+   : DurationElement(cr)
       {
-      return _up;
+      _beam     = 0;
+      _up       = cr._up;
+      _small    = cr._small;
+      _beamMode = cr._beamMode;
+      _dots     = cr._dots;
+
+      foreach(Articulation* a, cr.articulations) {            // make deep copy
+            Articulation* na = new Articulation(*a);
+            na->setParent(this);
+            na->setTrack(track());
+            articulations.append(na);
+            }
       }
 
 //---------------------------------------------------------
@@ -193,49 +118,23 @@ bool ChordRest::isUp() const
 
 QPointF ChordRest::canvasPos() const
       {
+      if (parent() == 0)
+            return QPointF(x(), 0.0);
       double xp = x();
       for (Element* e = parent(); e; e = e->parent())
             xp += e->x();
       System* system = measure()->system();
+      if (system == 0)
+            return QPointF();
       double yp = y() + system->staff(staffIdx())->y() + system->y();
       return QPointF(xp, yp);
       }
 
-//---------------------------------------------------------
-//   setBeamMode
-//---------------------------------------------------------
-
-void ChordRest::setBeamMode(BeamMode m)
-      {
-//      if (!_tuplet)
-            _beamMode = m;
-      }
-
-//---------------------------------------------------------
-//   beams
-//---------------------------------------------------------
-
-int ChordRest::beams() const
-      {
-      int t = tickLen();
-      if (t < division/8)
-            return 4;
-      else if (t < division/4)
-            return 3;
-      else if (t < division/2)
-            return 2;
-      else if (t < division)
-            return 1;
-      else
-            printf("ChordRest::beams(): unsupported tickLen %d\n", t);
-      return 0;
-      }
-
 //---------------------------------------------------------
 //   properties
 //---------------------------------------------------------
 
-QList<Prop> ChordRest::properties(Xml& xml) const
+QList<Prop> ChordRest::properties(Xml& xml, bool clipboardmode) const
       {
       QList<Prop> pl = Element::properties(xml);
       //
@@ -257,15 +156,17 @@ QList<Prop> ChordRest::properties(Xml& xml) const
                   }
             pl.append(Prop("BeamMode", s));
             }
-      if (_tuplet) {
-            int idx = measure()->tuplets()->indexOf(_tuplet);
-            if (idx == -1)
-                  printf("ChordRest::writeProperties(): tuplet not found\n");
-            else
-                  pl.append(Prop("Tuplet", idx));
-            }
+      if (tuplet())
+            pl.append(Prop("Tuplet", tuplet()->id()));
       if (_small)
             pl.append(Prop("small", _small));
+      if (!clipboardmode) {
+            if (tickLen() != duration().ticks(_dots)) {
+                  if (_dots)
+                        pl.append(Prop("dots", _dots));
+                  pl.append(Prop("durationType", duration().name()));
+                  }
+            }
       return pl;
       }
 
@@ -277,30 +178,25 @@ void ChordRest::writeProperties(Xml& xml) const
       {
       QList<Prop> pl = properties(xml);
       xml.prop(pl);
-      for (ciAttribute ia = attributes.begin(); ia != attributes.end(); ++ia)
+      for (ciArticulation ia = articulations.begin(); ia != articulations.end(); ++ia)
             (*ia)->write(xml);
-      foreach(Slur* s, _slurFor)
-            xml.tagE(QString("Slur type=\"start\" number=\"%1\"").arg(s->id()+1));
-      foreach(Slur* s, _slurBack)
-            xml.tagE(QString("Slur type=\"stop\" number=\"%1\"").arg(s->id()+1));
+      if (!xml.noSlurs) {
+            foreach(Slur* s, _slurFor)
+                  xml.tagE(QString("Slur type=\"start\" number=\"%1\"").arg(s->id()+1));
+            foreach(Slur* s, _slurBack)
+                  xml.tagE(QString("Slur type=\"stop\" number=\"%1\"").arg(s->id()+1));
+            }
+      if (!xml.clipboardmode && _beam)
+            xml.tag("Beam", _beam->id());
       xml.curTick = tick() + tickLen();
       }
 
-//---------------------------------------------------------
-//   isSimple
-//---------------------------------------------------------
-
-bool ChordRest::isSimple(Xml& xml) const
-      {
-      QList<Prop> pl = properties(xml);
-      return (pl.size() <= 1) && attributes.empty() && _slurFor.empty() && _slurBack.empty();
-      }
-
 //---------------------------------------------------------
 //   readProperties
 //---------------------------------------------------------
 
-bool ChordRest::readProperties(QDomElement e)
+bool ChordRest::readProperties(QDomElement e, const QList<Tuplet*>& tuplets,
+   const QList<Beam*>& beams)
       {
       if (Element::readProperties(e))
             return true;
@@ -327,21 +223,34 @@ bool ChordRest::readProperties(QDomElement e)
             _beamMode = BeamMode(bm);
             }
       else if (tag == "Attribute") {
-            NoteAttribute* atr = new NoteAttribute(score());
+            Articulation* atr = new Articulation(score());
             atr->read(e);
             add(atr);
             }
       else if (tag == "Tuplet") {
-            // while reading Measure, parent of Chord or Rest is set
-            // to measure; after inserting Chord or Rest into Measure
-            // parent is Segment
-            Measure* m = (Measure*)parent();
-            _tuplet    = m->tuplets()->value(i);
-            if (_tuplet == 0)
-                  printf("Tuplet index %d not found\n", i);
+            setTuplet(0);
+            foreach(Tuplet* t, tuplets) {
+                  if (t->id() == i) {
+                        setTuplet(t);
+                        break;
+                        }
+                  }
+            if (tuplet() == 0)
+                  printf("Tuplet id %d not found\n", i);
             else
                   setTickLen(tickLen());  // set right symbol + dots
             }
+      else if (tag == "Beam") {
+            foreach(Beam* b, beams) {
+                  if (b->id() == i) {
+                        setBeam(b);
+                        b->add(this);
+                        break;
+                        }
+                  }
+            if (beam() == 0)
+                  printf("Beam id %d not found\n", i);
+            }
       else if (tag == "small")
             _small = i;
       else if (tag == "Slur") {
@@ -370,6 +279,13 @@ bool ChordRest::readProperties(QDomElement e)
                   printf("Note::read(): Slur not found\n");
                   }
             }
+      else if (tag == "durationType") {
+            Duration d;
+            d.setVal(val);
+            setDuration(d);
+            }
+      else if (tag == "dots")
+            _dots = i;
       else
             return false;
       return true;
@@ -413,12 +329,12 @@ void ChordRest::layoutAttributes(ScoreLayout* layout)
 
       //
       //    pass 1
-      //    place all attributes with anchor at chord/rest
+      //    place all articulations with anchor at chord/rest
       //
-      for (iAttribute ia = attributes.begin(); ia != attributes.end(); ++ia) {
-            NoteAttribute* a = *ia;
+      for (iArticulation ia = articulations.begin(); ia != articulations.end(); ++ia) {
+            Articulation* a = *ia;
             qreal y = 0;
-            AttrAnchor aa = NoteAttribute::atrList[a->subtype()].anchor;
+            ArticulationAnchor aa = Articulation::articulationList[a->subtype()].anchor;
             if (aa == A_CHORD)
                   aa = isUp() ? A_BOTTOM_CHORD : A_TOP_CHORD;
             if (aa == A_TOP_CHORD) {
@@ -426,7 +342,7 @@ void ChordRest::layoutAttributes(ScoreLayout* layout)
                   //
                   // check for collision with staff line
                   //
-                  if (y >= staffTopY-.1) {
+                  if (y >= staffTopY+.1) {
                         qreal l = y / _spatium;
                         qreal delta = fabs(l - round(l));
                         if (delta < 0.4)
@@ -453,7 +369,7 @@ void ChordRest::layoutAttributes(ScoreLayout* layout)
 
       //
       //    pass 1
-      //    now place all attributes with staff top or bottom anchor
+      //    now place all articulations with staff top or bottom anchor
       //
       if (chordTopY - dyTop > staffTopY)
             dyTop = 0;
@@ -465,10 +381,10 @@ void ChordRest::layoutAttributes(ScoreLayout* layout)
       else
             dyBot = (chordBotY + dyBot) - staffBotY;
 
-      for (iAttribute ia = attributes.begin(); ia != attributes.end(); ++ia) {
-            NoteAttribute* a = *ia;
+      for (iArticulation ia = articulations.begin(); ia != articulations.end(); ++ia) {
+            Articulation* a = *ia;
             qreal y = 0;
-            AttrAnchor aa = NoteAttribute::atrList[a->subtype()].anchor;
+            ArticulationAnchor aa = Articulation::articulationList[a->subtype()].anchor;
             if (aa == A_TOP_STAFF) {
                   y = staffTopY - dyTop;
                   a->setPos(x, y);
@@ -547,3 +463,93 @@ void ChordRest::removeSlurBack(Slur* s)
       _slurBack.removeAt(idx);
       }
 
+//---------------------------------------------------------
+//   setLen
+//---------------------------------------------------------
+
+void ChordRest::setLen(int ticks)
+      {
+      setTickLen(ticks);
+      Duration dt;
+      int dts;
+      headType(ticks, &dt, &dts);
+      setDuration(dt);
+      setDots(dts);
+      }
+
+//---------------------------------------------------------
+//   drop
+//---------------------------------------------------------
+
+Element* ChordRest::drop(const QPointF& p1, const QPointF& p2, Element* e)
+      {
+      Measure* m  = measure();
+      switch (e->type()) {
+            case BREATH:
+                  {
+                  Breath* b = static_cast<Breath*>(e);
+                  b->setTrack(staffIdx() * VOICES);
+
+                  // TODO: insert automatically in all staves?
+
+                  Segment* seg = m->findSegment(Segment::SegBreath, tick());
+                  if (seg == 0) {
+                        seg = m->createSegment(Segment::SegBreath, tick());
+                        score()->undoAddElement(seg);
+                        }
+                  b->setParent(seg);
+                  score()->undoAddElement(b);
+                  }
+                  break;
+
+            case BAR_LINE:
+                  {
+                  BarLine* bl = static_cast<BarLine*>(e);
+                  bl->setTrack(staffIdx() * VOICES);
+
+                  if ((bl->tick() == m->tick())
+                     || (bl->tick() == m->tick() + m->tickLen())) {
+                        return m->drop(p1, p2, e);
+                        }
+
+                  Segment* seg = m->findSegment(Segment::SegBarLine, tick());
+                  if (seg == 0) {
+                        seg = m->createSegment(Segment::SegBarLine, tick());
+                        score()->undoAddElement(seg);
+                        }
+                  bl->setParent(seg);
+                  score()->undoAddElement(bl);
+                  }
+                  break;
+
+            case CLEF:
+                  staff()->changeClef(tick(), e->subtype());
+                  break;
+
+            case TEMPO_TEXT:
+                  {
+                  TempoText* tt = static_cast<TempoText*>(e);
+                  score()->tempomap->addTempo(tick(), tt->tempo());
+                  tt->setTick(tick());
+                  tt->setTrack(0);
+                  tt->setParent(m);
+                  score()->undoAddElement(tt);
+                  }
+                  break;
+
+            default:
+                  printf("cannot drop %s\n", e->name());
+                  return 0;
+            }
+      return 0;
+      }
+
+//---------------------------------------------------------
+//   setBeam
+//---------------------------------------------------------
+
+void ChordRest::setBeam(Beam* b)
+      {
+      _beam = b;
+      }
+
