@@ -3,7 +3,7 @@
 //  Linux Music Score Editor
 //  $Id: element.cpp,v 1.79 2006/04/12 14:58:10 wschweer Exp $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2008 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -57,35 +57,45 @@
 #include "repeat.h"
 #include "page.h"
 #include "system.h"
+#include "stafftext.h"
+#include "canvas.h"
+#include "glissando.h"
+#include "articulation.h"
+#include "chord.h"
+#include "spacer.h"
+#include "mscore.h"
+#include "tempotext.h"
+#include "harmony.h"
+#include "lyrics.h"
+#include "rest.h"
 
 extern bool debugMode;
 extern bool showInvisible;
 
-// for debugging:
 const char* elementNames[] = {
       "Symbol", "Text", "SlurSegment", "BarLine",
-      "StemSlash", "Line", "SystemBracket",
+      "StemSlash", "Line", "Bracket",
       "Arpeggio",
       "Accidental",
       "Note",
       "Stem",
       "Clef", "KeySig", "TimeSig", "Rest",
-      "Breath",
+      "Breath", "Glissando",
       "RepeatMeasure",
       "Image",
       "Tie",
       "Attribute", "Dynamic", "Page", "Beam", "Hook", "Lyrics", "Marker", "Jump",
-      "Tuplet", "VSpacer",
-      "TempoText", "StaffText",
+      "Tuplet",
+      "Tempo", "StaffText",
       "Harmony",
       "Volta",
       "HairpinSegment", "OttavaSegment", "PedalSegment", "TrillSegment", "TextLineSegment",
       "VoltaSegment",
-      "LayoutBreak",
+      "LayoutBreak", "Spacer",
       "LedgerLine",
+      "NoteHead", "Tremolo",
       "Measure", "StaffLines",
       "Cursor", "Selection", "Lasso", "ShadowNote", "RubberBand",
-      "NoteHead", "Tremolo",
       "HairPin", "Ottava", "Pedal", "Trill", "TextLine",
       "Segment", "System", "Compound", "Chord", "Slur",
       "Element", "ElementList", "StaffList", "MeasureList", "Layout",
@@ -138,6 +148,8 @@ Element::~Element()
 
 void Element::init()
       {
+      _tick       = -1;
+      _duration   = -1;
       _parent     = 0;
       _selected   = false;
       _selectable = true;
@@ -145,8 +157,7 @@ void Element::init()
       _visible    = true;
       _generated  = false;
       _track      = -1;
-//      _track      = 0;
-      _color      = Qt::black;
+      _color      = preferences.defaultColor;
       _mxmlOff    = 0;
       _pos.setX(0.0);
       _pos.setY(0.0);
@@ -174,6 +185,35 @@ Element::Element(Score* s)
       _mag = 1.0;
       }
 
+Element::Element(const Element& e)
+      {
+      _parent     = e._parent;
+      _selected   = e._selected;
+      _selectable = e._selectable;
+      _dropTarget = e._dropTarget;
+      _generated  = e._generated;
+      _visible    = e._visible;
+      _systemFlag = e._systemFlag;
+      _subtype    = e._subtype;
+      _track      = e._track;
+      _tick       = e._tick;
+      _color      = e._color;
+      _mag        = e._mag;
+      _score      = e._score;
+      _pos        = e._pos;
+      _align      = e._align;
+      _xoff       = e._xoff;
+      _yoff       = e._yoff;
+      _rxoff      = e._rxoff;
+      _ryoff      = e._ryoff;
+      _offsetType = e._offsetType;
+      _userOff    = e._userOff;
+      _mxmlOff    = e._mxmlOff;
+      _bbox       = e._bbox;
+      _duration   = e._duration;
+      itemDiscovered = 0;
+      }
+
 //---------------------------------------------------------
 //   staff
 //---------------------------------------------------------
@@ -191,16 +231,19 @@ Staff* Element::staff() const
 
 QColor Element::curColor() const
       {
+      // the default element color is always interpreted as black in
+      // printing
       if (score() && score()->printing())
-            return _color;
+            return (_color == preferences.defaultColor) ? Qt::black : _color;
+
+      if (_dropTarget)
+            return preferences.dropColor;
       if (_selected) {
             if (track() == -1)
                   return preferences.selectColor[0];
             else
                   return preferences.selectColor[voice()];
             }
-      if (_dropTarget)
-            return preferences.dropColor;
       if (!_visible)
             return Qt::gray;
       return _color;
@@ -218,18 +261,9 @@ QRectF Element::drag(const QPointF& pos)
       {
       QRectF r(abbox());
       setUserOff(pos / _spatium);
-      score()->setLayoutAll(false); // debug
       return abbox() | r;
       }
 
-//---------------------------------------------------------
-//   endDrag
-//---------------------------------------------------------
-
-void Element::endDrag()
-      {
-      }
-
 //---------------------------------------------------------
 //   canvasPos
 //    return position in canvas coordinates
@@ -306,7 +340,10 @@ void Element::layout(ScoreLayout* layout)
       {
       QPointF o(QPointF(_xoff, _yoff));
       if (_offsetType == OFFSET_SPATIUM)
-            o *= layout->spatium();
+            if (layout)
+                  o *= layout->spatium();
+            else
+                  o *= _spatium;
       else
             o *= DPI;
       if (parent())
@@ -318,8 +355,8 @@ void Element::layout(ScoreLayout* layout)
             p.setY(-h);
       else if (_align & ALIGN_VCENTER)
             p.setY(-(h * .5));
-      else if (_align & ALIGN_BASELINE)   // undefined
-            p.setY(-h);
+      else if (_align & ALIGN_BASELINE)
+            p.setY(-baseLine());
       if (_align & ALIGN_RIGHT)
             p.setX(-w);
       else if (_align & ALIGN_HCENTER)
@@ -341,11 +378,12 @@ QList<Prop> Element::properties(Xml& xml) const
             }
       if (!_userOff.isNull())
             pl.append(Prop("offset", _userOff));
-      if (track() != xml.curTrack) {
+      if ((track() != xml.curTrack) && (track() != -1)) {
             int t;
-            if (track() == -1)
+/*            if (track() == -1)
                   t = -1;
             else
+                  */
                   t = track() + xml.trackDiff;
             pl.append(Prop("track", t));
             }
@@ -353,11 +391,11 @@ QList<Prop> Element::properties(Xml& xml) const
             pl.append(Prop("selected", selected()));
       if (!visible())
             pl.append(Prop("visible", visible()));
-      if (_time.isValid() && (_time.tick() != xml.curTick))
-            pl.append(Prop("tick", _time.tick()));
-      if (_duration.isValid())
-            pl.append(Prop("ticklen", _duration.tick()));
-      if (_color != Qt::black)
+      if (_tick != -1 && (_tick != xml.curTick))
+            pl.append(Prop("tick", _tick));
+      if (_duration != -1)
+            pl.append(Prop("ticklen", _duration));
+      if (_color != preferences.defaultColor)
             pl.append(Prop("color", _color));
       if (_systemFlag)
             pl.append(Prop("systemFlag", _systemFlag));
@@ -371,8 +409,8 @@ QList<Prop> Element::properties(Xml& xml) const
 void Element::writeProperties(Xml& xml) const
       {
       xml.prop(properties(xml));
-      if (_time.isValid() && (_time.tick() != xml.curTick || debugMode))
-            xml.curTick = _time.tick();
+      if ((_tick != -1) && (_tick != xml.curTick || debugMode))
+            xml.curTick = _tick;
       }
 
 //---------------------------------------------------------
@@ -384,11 +422,10 @@ bool Element::readProperties(QDomElement e)
       QString tag(e.tagName());
       QString val(e.text());
       int i = val.toInt();
-//      setTrack(score()->curTrack);
 
       if (tag == "tick") {
             setTick(score()->fileDivision(i));
-            score()->curTick = _time.tick();
+            score()->curTick = _tick;
             }
       else if (tag == "subtype") {
             // do not always call Element::setSubtype():
@@ -406,12 +443,8 @@ bool Element::readProperties(QDomElement e)
             setTrack(i);
       else if (tag == "selected")
             setSelected(i);
-      else if (tag == "color") {
-            int r = e.attribute("r", "0").toInt();
-            int g = e.attribute("g", "0").toInt();
-            int b = e.attribute("b", "0").toInt();
-            _color.setRgb(r, g, b);
-            }
+      else if (tag == "color")
+            _color = readColor(e);
       else if (tag == "systemFlag") {
             _systemFlag = i;
             if (_systemFlag)
@@ -588,6 +621,8 @@ QRectF StaffLines::bbox() const
                   return QRectF(0.0, - 2.0 * _spatium - lw*.5, _width, 4 * _spatium + lw);
             case 1:
                   return QRectF(0.0,  -lw*.5, _width, 4 * _spatium + lw);
+            case 2:
+                  return QRectF(0.0, -lw*.5, _width, l * _spatium * 2.0 + lw);
             default:
                   return QRectF(0.0, -lw*.5, _width, l * _spatium + lw);
             }
@@ -601,8 +636,9 @@ void StaffLines::draw(QPainter& p) const
       {
       QPointF _pos(0.0, 0.0);
 
-      p.save();
-      p.setRenderHint(QPainter::Antialiasing, false);
+//      p.save();
+//      p.setRenderHint(QPainter::Antialiasing, false);
+//      p.setRenderHint(QPainter::NonCosmeticDefaultPen, true);
 
       QPen pen(p.pen());
       pen.setWidthF(point(score()->style()->staffLineWidth) * mag());
@@ -629,6 +665,13 @@ void StaffLines::draw(QPainter& p) const
                   p.drawLine(QLineF(x1, y, x2, y));
                   }
                   break;
+            case 3:
+                  for (int i = 0; i < lines(); ++i) {
+                        qreal y = _pos.y() + i * _spatium * mag() * 2.0;
+                        p.drawLine(QLineF(x1, y, x2, y));
+                        }
+                  break;
+
             default:
                   for (int i = 0; i < lines(); ++i) {
                         qreal y = _pos.y() + i * _spatium * mag();
@@ -636,10 +679,9 @@ void StaffLines::draw(QPainter& p) const
                         }
                   break;
             }
-      p.restore();
+//      p.restore();
       }
 
-
 //---------------------------------------------------------
 //   y1
 //---------------------------------------------------------
@@ -652,6 +694,7 @@ double StaffLines::y1() const
                   return y + _pos.y() + 1 * _spatium * mag();
             case 2:
                   return y + _pos.y() + 1 * _spatium * mag();
+            case 3:
             default:
                   return y + _pos.y();
             }
@@ -669,11 +712,41 @@ double StaffLines::y2() const
                   return y + _pos.y() + 3 * _spatium * mag();
             case 2:
                   return y + _pos.y() + 3 * _spatium * mag();
+            case 3:
             default:
                   return y + _pos.y() + 4 * _spatium * mag();
             }
       }
 
+//---------------------------------------------------------
+//   StaffLines::write
+//---------------------------------------------------------
+
+void StaffLines::write(Xml& xml) const
+      {
+      xml.stag("Staff");
+      if (lines() != 5)
+            xml.tag("lines", lines());
+      Element::writeProperties(xml);
+      xml.etag();
+      }
+
+//---------------------------------------------------------
+//   StaffLines::read
+//---------------------------------------------------------
+
+void StaffLines::read(QDomElement e)
+      {
+      for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
+            QString tag(e.tagName());
+            if (tag == "lines")
+                  setLines(e.text().toInt());
+            else if (!Element::readProperties(e))
+                  domError(e);
+            }
+      }
+
+
 //---------------------------------------------------------
 //   Line
 //---------------------------------------------------------
@@ -810,7 +883,6 @@ void Compound::addElement(Element* e, double x, double y)
       {
       e->setPos(x, y);
       e->setParent(this);
-      e->setMag(mag());
       elemente.push_back(e);
       }
 
@@ -861,17 +933,6 @@ void Compound::clear()
       elemente.clear();
       }
 
-//---------------------------------------------------------
-//   setMag
-//---------------------------------------------------------
-
-void Compound::setMag(double val)
-      {
-      for (ciElement i = elemente.begin(); i != elemente.end(); ++i)
-            (*i)->setMag(val);
-      Element::setMag(val);
-      }
-
 //---------------------------------------------------------
 //   Cursor
 //---------------------------------------------------------
@@ -882,8 +943,19 @@ Cursor::Cursor(Score* s, Viewer* v)
       viewer    = v;
       _on       = false;
       _blink    = true;
-      double w  = 2.0 / viewer->matrix().m11();
-      setbbox(QRectF(0.0, 0.0, w, 6 * _spatium));
+      _h        = 6 * _spatium;
+      _seg      = 0;
+      }
+
+//---------------------------------------------------------
+//   bbox
+//---------------------------------------------------------
+
+QRectF Cursor::bbox() const
+      {
+      double w  = 2.0 / static_cast<Viewer*>(score()->canvas())->matrix().m11();
+      _bbox = QRectF(0.0, 0.0, w, _h);
+      return _bbox;
       }
 
 //---------------------------------------------------------
@@ -894,8 +966,12 @@ void Cursor::draw(QPainter& p) const
       {
       if (!(_on && _blink))
             return;
+      double x = pos().x();
+      if (_seg)
+            x = _seg->canvasPos().x() - _spatium;
+
       int v = track() == -1 ? 0 : voice();
-      p.fillRect(abbox(), QBrush(preferences.selectColor[v]));
+      p.fillRect(QRectF(x, pos().y(), _bbox.width(), _bbox.height()), QBrush(preferences.selectColor[v]));
       }
 
 //---------------------------------------------------------
@@ -951,40 +1027,6 @@ void RubberBand::draw(QPainter& p) const
       p.drawLine(QLineF(_p1, _p2));
       }
 
-//---------------------------------------------------------
-//   VSpacer
-//---------------------------------------------------------
-
-VSpacer::VSpacer(Score* s, double h)
-   : Element(s)
-      {
-      height = h;
-      }
-
-//---------------------------------------------------------
-//   draw
-//---------------------------------------------------------
-
-void VSpacer::draw(QPainter&) const
-      {
-//      int lw       = lrint(.5 * tf->mag() * _spatium);
-//      int len      = lrint(height * tf->mag() * _spatium);
-//      QPoint _pos  = tf->fpos2ipoint(QPointF(0, 0));
-
-//      p.setPen(QPen(QColor(Qt::blue), lw));
-//TODO      p.drawLine(_pos.x(), _pos.y(), _pos.x(), _pos.y() + height);
-      }
-
-//---------------------------------------------------------
-//   space
-//---------------------------------------------------------
-
-void Element::space(double& min, double& extra) const
-      {
-      min   = 0.0;
-      extra = width();
-      }
-
 //---------------------------------------------------------
 //   mimeData
 //---------------------------------------------------------
@@ -1003,8 +1045,35 @@ QByteArray Element::mimeData(const QPointF& dragOffset) const
       return buffer.buffer();
       }
 
+//---------------------------------------------------------
+//   name2type
+//---------------------------------------------------------
+
+int Element::name2type(const QString& s)
+      {
+      int n = sizeof(elementNames)/sizeof(*elementNames);
+      for (int i = 0; i < n; ++i) {
+            if (s == elementNames[i])
+                  return i;
+            }
+      return -1;
+      }
+
+//---------------------------------------------------------
+//   name2Element
+//---------------------------------------------------------
+
+Element* Element::name2Element(const QString& s, Score* sc)
+      {
+      int type = Element::name2type(s);
+      if (type == -1)
+            return 0;
+      return Element::create(type, sc);
+      }
+
 //---------------------------------------------------------
 //   readType
+//    return -1 if no valid type found
 //---------------------------------------------------------
 
 int Element::readType(QDomElement& e, QPointF* dragOffset)
@@ -1012,105 +1081,10 @@ int Element::readType(QDomElement& e, QPointF* dragOffset)
       int type = -1;
 
       for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
-            //
-            // DEBUG:
-            // check names; remove non needed elements
-            //
             if (e.tagName() == "dragOffset")
                   *dragOffset = readPoint(e);
-            else if (e.tagName() == "Dynamic")
-                  type = DYNAMIC;
-            else if (e.tagName() == "Symbol")
-                  type = SYMBOL;
-            else if (e.tagName() == "Text")
-                  type = TEXT;
-            else if (e.tagName() == "StaffLines")
-                  type = STAFF_LINES;
-            else if (e.tagName() == "Slur")
-                  type = SLUR_SEGMENT;
-            else if (e.tagName() == "Note")
-                  type = NOTE;
-            else if (e.tagName() == "BarLine")
-                  type = BAR_LINE;
-            else if (e.tagName() == "Stem")
-                  type = STEM;
-            else if (e.tagName() == "Bracket")
-                  type = BRACKET;
-            else if (e.tagName() == "Accidental")
-                  type = ACCIDENTAL;
-            else if (e.tagName() == "Clef")
-                  type = CLEF;
-            else if (e.tagName() == "KeySig")
-                  type = KEYSIG;
-            else if (e.tagName() == "TimeSig")
-                  type = TIMESIG;
-            else if (e.tagName() == "Chord")
-                  type = CHORD;
-            else if (e.tagName() == "Rest")
-                  type = REST;
-            else if (e.tagName() == "Tie")
-                  type = TIE;
-            else if (e.tagName() == "Slur")
-                  type = SLUR;
-            else if (e.tagName() == "Measure")
-                  type = MEASURE;
-            else if (e.tagName() == "Attribute")
-                  type = ATTRIBUTE;
-            else if (e.tagName() == "Page")
-                  type = PAGE;
-            else if (e.tagName() == "Beam")
-                  type = BEAM;
-            else if (e.tagName() == "Hook")
-                  type = HOOK;
-            else if (e.tagName() == "Lyric")
-                  type = LYRICS;
-            else if (e.tagName() == "System")
-                  type = SYSTEM;
-            else if (e.tagName() == "HairPin")
-                  type = HAIRPIN;
-            else if (e.tagName() == "Tuplet")
-                  type = TUPLET;
-            else if (e.tagName() == "VSpacer")
-                  type = VSPACER;
-            else if (e.tagName() == "Segment")
-                  type = SEGMENT;
-            else if (e.tagName() == "TempoText")
-                  type = TEMPO_TEXT;
-            else if (e.tagName() == "Volta")
-                  type = VOLTA;
-            else if (e.tagName() == "Ottava")
-                  type = OTTAVA;
-            else if (e.tagName() == "TextLine")
-                  type = TEXTLINE;
-            else if (e.tagName() == "Pedal")
-                  type = PEDAL;
-            else if (e.tagName() == "Trill")
-                  type = TRILL;
-            else if (e.tagName() == "LayoutBreak")
-                  type = LAYOUT_BREAK;
-            else if (e.tagName() == "LedgerLine")
-                  type = LEDGER_LINE;
-            else if (e.tagName() == "Image")
-                  type = IMAGE;
-            else if (e.tagName() == "Breath")
-                  type = BREATH;
-            else if (e.tagName() == "Arpeggio")
-                  type = ARPEGGIO;
-            else if (e.tagName() == "NoteHead")
-                  type = NOTEHEAD;
-            else if (e.tagName() == "Tremolo")
-                  type = TREMOLO;
-            else if (e.tagName() == "RepeatMeasure")
-                  type = REPEAT_MEASURE;
-            else if (e.tagName() == "Jump")
-                  type = JUMP;
-            else if (e.tagName() == "Marker")
-                  type = MARKER;
-            else if (e.tagName() == "Icon")
-                  type = ICON;
-            else {
+            else if ((type = name2type(e.tagName())) == -1) {
                   domError(e);
-                  type = 0;
                   break;
                   }
             if (type >= 0)
@@ -1164,11 +1138,14 @@ Element* Element::create(int type, Score* score)
             case BREATH:
                   el = new Breath(score);
                   break;
+            case GLISSANDO:
+                  el = new Glissando(score);
+                  break;
             case BRACKET:
                   el = new Bracket(score);
                   break;
             case ATTRIBUTE:
-                  el = new NoteAttribute(score);
+                  el = new Articulation(score);
                   break;
             case ACCIDENTAL:
                   el = new Accidental(score);
@@ -1179,6 +1156,9 @@ Element* Element::create(int type, Score* score)
             case TEXT:
                   el = new Text(score);
                   break;
+            case STAFF_TEXT:
+                  el = new StaffText(score);
+                  break;
             case NOTEHEAD:
                   el = new NoteHead(score);
                   break;
@@ -1206,8 +1186,27 @@ Element* Element::create(int type, Score* score)
             case SYMBOL:
                   el = new Symbol(score);
                   break;
+            case CHORD:
+                  el = new Chord(score);
+                  break;
+            case REST:
+                  el = new Rest(score);
+                  break;
+            case SPACER:
+                  el = new Spacer(score);
+                  break;
+            case TEMPO_TEXT:
+                  el = new TempoText(score);
+                  break;
+            case HARMONY:
+                  el = new Harmony(score);
+                  break;
+            case LYRICS:
+                  el = new Lyrics(score);
+                  break;
             default:
-                  printf("Element::create(): cannot create element type %d\n", type);
+                  printf("Element::create(): cannot create element type %s\n",
+                     elementNames[type]);
                   break;
             }
       return el;
@@ -1217,7 +1216,7 @@ Element* Element::create(int type, Score* score)
 //   startEdit
 //---------------------------------------------------------
 
-bool Element::startEdit(const QPointF&)
+bool Element::startEdit(Viewer*, const QPointF&)
       {
       return !_generated;
       }
@@ -1236,9 +1235,9 @@ void Element::editDrag(int, const QPointF& delta)
 //    return true if event is accepted
 //---------------------------------------------------------
 
-bool Element::edit(int, QKeyEvent* ev)
+bool Element::edit(Viewer*, int, int key, Qt::KeyboardModifiers, const QString&)
       {
-      if (ev->key() ==  Qt::Key_Home) {
+      if (key ==  Qt::Key_Home) {
             setUserOff(QPoint());
             return true;
             }
@@ -1263,3 +1262,32 @@ void Element::remove(Element* e)
       printf("cannot remove %s from %s\n", e->name(), name());
       }
 
+//---------------------------------------------------------
+//   write
+//---------------------------------------------------------
+
+void Icon::write(Xml& xml) const
+      {
+      xml.stag(name());
+      Element::writeProperties(xml);
+      xml.tag("action", _action->data().toString());
+      xml.etag();
+      }
+
+//---------------------------------------------------------
+//   read
+//---------------------------------------------------------
+
+void Icon::read(QDomElement e)
+      {
+      for (e = e.firstChildElement(); !e.isNull(); e = e.nextSiblingElement()) {
+            QString tag(e.tagName());
+            if (tag == "action") {
+                  _action = getAction(qPrintable(e.text()));
+                  }
+            else if (!Element::readProperties(e))
+                  domError(e);
+            }
+      }
+
+
