@@ -35,6 +35,7 @@
 #include "editinstrument.h"
 #include "drumset.h"
 #include "slur.h"
+#include "seq.h"
 
 //---------------------------------------------------------
 //   StaffListItem
@@ -67,8 +68,7 @@ StaffListItem::StaffListItem()
 void StaffListItem::setPartIdx(int val)
       {
       _partIdx = val;
-      QString s("Staff %1");
-      setText(0, s.arg(_partIdx + 1));
+      setText(0, InstrumentsDialog::tr("Staff %1").arg(_partIdx + 1));
       }
 
 //---------------------------------------------------------
@@ -78,7 +78,7 @@ void StaffListItem::setPartIdx(int val)
 void StaffListItem::setClef(int val)
       {
       _clef = val;
-      setText(1, clefTable[_clef].name);
+      setText(1, InstrumentsDialog::tr(clefTable[_clef].name));
       }
 
 //---------------------------------------------------------
@@ -100,7 +100,7 @@ PartListItem::PartListItem(const InstrumentTemplate* i, QTreeWidget* lv)
       part = 0;
       it   = i;
       op   = ITEM_ADD;
-      setText(0, it->name);
+      setText(0, it->trackName);
       }
 
 //---------------------------------------------------------
@@ -118,14 +118,14 @@ InstrumentTemplateListItem::InstrumentTemplateListItem(InstrumentTemplate* i, In
    : QTreeWidgetItem(item) {
       _instrumentTemplate = i;
       _group = _instrumentTemplate->group;
-      setText(0, i->name);
+      setText(0, i->trackName);
       }
 
 InstrumentTemplateListItem::InstrumentTemplateListItem(InstrumentTemplate* i, QTreeWidget* parent)
    : QTreeWidgetItem(parent) {
       _instrumentTemplate = i;
       _group = _instrumentTemplate->group;
-      setText(0, i->name);
+      setText(0, i->trackName);
       }
 
 //---------------------------------------------------------
@@ -137,7 +137,7 @@ QString InstrumentTemplateListItem::text(int col) const
       switch (col) {
             case 0:
                   return _instrumentTemplate ?
-                     _instrumentTemplate->name : _group;
+                     _instrumentTemplate->trackName : _group;
             default:
                   return QString("");
             }
@@ -157,7 +157,7 @@ InstrumentsDialog::InstrumentsDialog(QWidget* parent)
       instrumentList->setSelectionMode(QAbstractItemView::SingleSelection);
       partiturList->setSelectionMode(QAbstractItemView::SingleSelection);
 
-      instrumentList->setHeaderLabels(QStringList("Instrument List"));
+      instrumentList->setHeaderLabels(QStringList(tr("Instrument List")));
       QStringList header = (QStringList() << tr("Staves") << tr("Clef"));
       partiturList->setHeaderLabels(header);
 
@@ -206,7 +206,7 @@ void InstrumentsDialog::genPartList()
                   sli->staff    = s;
                   sli->setPartIdx(s->rstaff());
                   sli->staffIdx = s->idx();
-                  sli->setClef(s->clef()->clef(0));
+                  sli->setClef(s->clefList()->clef(0));
                   }
             partiturList->setItemExpanded(pli, true);
             }
@@ -246,7 +246,7 @@ void InstrumentsDialog::on_partiturList_itemSelectionChanged()
 //   on_instrumentList
 //---------------------------------------------------------
 
-void InstrumentsDialog::on_instrumentList_itemActivated(QTreeWidgetItem*, int)
+void InstrumentsDialog::on_instrumentList_itemDoubleClicked(QTreeWidgetItem*, int)
       {
       on_addButton_clicked();
       }
@@ -501,27 +501,32 @@ void MuseScore::editInstrList()
       int rstaff   = 0;
 
       QTreeWidgetItem* item = 0;
+      for (int idx = 0; (item = pl->topLevelItem(idx)); ++idx) {
+            PartListItem* pli = static_cast<PartListItem*>(item);
+            // check if the part contains any remaining staves
+            // mark to remove part if not
+            QTreeWidgetItem* ci = 0;
+            int staves = 0;
+            for (int cidx = 0; (ci = pli->child(cidx)); ++cidx) {
+                  StaffListItem* sli = static_cast<StaffListItem*>(ci);
+                  if (sli->op != ITEM_DELETE)
+                        ++staves;
+                  }
+            if (staves == 0)
+                  pli->op = ITEM_DELETE;
+            }
+
+      item = 0;
       for (int idx = 0; (item = pl->topLevelItem(idx)); ++idx) {
             rstaff = 0;
-            PartListItem* pli = (PartListItem*)item;
+            PartListItem* pli = static_cast<PartListItem*>(item);
             if (pli->op == ITEM_DELETE) {
                   cs->cmdRemovePart(pli->part);
                   }
             else if (pli->op == ITEM_ADD) {
                   const InstrumentTemplate* t = ((PartListItem*)item)->it;
                   part = new Part(cs);
-                  part->setMidiProgram(t->midiProgram);
-                  part->setMinPitch(t->minPitch);
-                  part->setMaxPitch(t->maxPitch);
-                  part->setShortName(t->shortName);
-                  part->setTrackName(t->name);
-                  part->setLongName(t->name);
-                  part->setPitchOffset(t->transpose);
-                  part->instrument()->midiActions = t->midiActions;
-                  if (t->useDrumset) {
-                        part->setUseDrumset(true);
-                        part->setDrumset(new Drumset(*smDrumset));
-                        }
+                  part->initFromInstrTemplate(t);
 
                   pli->part = part;
                   QTreeWidgetItem* ci = 0;
@@ -532,15 +537,14 @@ void MuseScore::editInstrList()
                         sli->staff = staff;
                         staff->setRstaff(rstaff);
                         // ++rstaff;
-                        staff->clef()->setClef(0, sli->clef());
+                        staff->clefList()->setClef(0, sli->clef());
                         staff->setLines(t->staffLines[cidx]);
                         staff->setSmall(t->smallStaff[cidx]);
                         if (cidx == 0) {
                               staff->setBracket(0, t->bracket);
                               staff->setBracketSpan(0, t->staves);
                               }
-                        cs->insertStaff(staff, staffIdx + rstaff);
-                        cs->undoOp(UndoOp::InsertStaff, staff, staffIdx + rstaff);
+                        cs->undoInsertStaff(staff, staffIdx + rstaff);
                         ++rstaff;
                         }
                   cs->cmdInsertPart(part, staffIdx);
@@ -569,10 +573,9 @@ void MuseScore::editInstrList()
                               sli->staff = staff;
                               staff->setRstaff(rstaff);
                               ++rstaff;
-                              staff->clef()->setClef(0, sli->clef());
+                              staff->clefList()->setClef(0, sli->clef());
 
-                              cs->insertStaff(staff, staffIdx);
-                              cs->undoOp(UndoOp::InsertStaff, staff, staffIdx);
+                              cs->undoInsertStaff(staff, staffIdx);
 
                               for (MeasureBase* mb = cs->measures()->first(); mb; mb = mb->next()) {
                                     if (mb->type() != MEASURE)
@@ -581,6 +584,7 @@ void MuseScore::editInstrList()
                                     m->cmdAddStaves(staffIdx, staffIdx+1);
                                     }
 
+                              cs->adjustBracketsIns(staffIdx, staffIdx+1);
                               ++staffIdx;
                               }
                         else {
@@ -590,6 +594,7 @@ void MuseScore::editInstrList()
                         }
                   }
             }
+
       //
       //    sort staves
       //
@@ -606,8 +611,8 @@ void MuseScore::editInstrList()
                   dst.push_back(sli->staff);
                   }
             }
-      QList<int> dl;
 
+      QList<int> dl;
       foreach(Staff* staff, dst) {
             int idx = cs->staves().indexOf(staff);
             if (idx == -1)
@@ -627,8 +632,11 @@ void MuseScore::editInstrList()
             cs->sortStaves(dl);
             cs->undoOp(dl);
             }
+
       cs->setLayoutAll(true);
       cs->endCmd();
+      cs->rebuildMidiMapping();
+      seq->initInstruments();
       }
 
 //---------------------------------------------------------
@@ -638,8 +646,7 @@ void MuseScore::editInstrList()
 
 void Score::cmdInsertPart(Part* part, int staffIdx)
       {
-      undoOp(UndoOp::InsertPart, part, staffIdx);
-      insertPart(part, staffIdx);
+      undoInsertPart(part, staffIdx);
 
       int sidx = this->staffIdx(part);
       int eidx = sidx + part->nstaves();
@@ -649,24 +656,7 @@ void Score::cmdInsertPart(Part* part, int staffIdx)
             Measure* m = (Measure*)mb;
             m->cmdAddStaves(sidx, eidx);
             }
-      //
-      //    adjust brackets
-      //
-      for (int staffIdx = 0; staffIdx < _staves.size(); ++staffIdx) {
-            Staff* staff = _staves[staffIdx];
-            int bl = staff->bracketLevels();
-            for (int i = 0; i < bl; ++i) {
-                  int span = staff->bracketSpan(i);
-                  if ((span == 0) || ((staffIdx + span) < sidx) || (staffIdx > eidx))
-                        continue;
-                  if ((sidx >= staffIdx) && (eidx <= (staffIdx + span)))
-                        staff->setBracketSpan(i, span + (eidx-sidx));
-                  else {
-                        printf("TODO: adjust brackets\n");
-                        }
-                  }
-            }
-
+      adjustBracketsIns(sidx, eidx);
       }
 
 //---------------------------------------------------------
@@ -678,12 +668,16 @@ void Score::cmdRemovePart(Part* part)
       int sidx   = staffIdx(part);
       int n      = part->nstaves();
       int eidx   = sidx + n;
-      int strack = sidx * VOICES;
-      int etrack = eidx * VOICES;
+
+// printf("cmdRemovePart %d-%d\n", sidx, eidx);
+
+//      int strack = sidx * VOICES;
+//      int etrack = eidx * VOICES;
 
       //
       //    remove/adjust slurs in _gel
       //
+#if 0
       foreach(Element* e, _gel) {
             if (e->type() != SLUR) {
                   printf("gel element %s %d\n", e->name(), e->track());
@@ -700,23 +694,8 @@ void Score::cmdRemovePart(Part* part)
                         slur->setTrack2(slur->track2() - VOICES);
                   }
             }
-      //
-      //    adjust brackets
-      //
-      for (int staffIdx = 0; staffIdx < _staves.size(); ++staffIdx) {
-            Staff* staff = _staves[staffIdx];
-            int bl = staff->bracketLevels();
-            for (int i = 0; i < bl; ++i) {
-                  int span = staff->bracketSpan(i);
-                  if ((span == 0) || ((staffIdx + span) < sidx) || (staffIdx > eidx))
-                        continue;
-                  if ((sidx >= staffIdx) && (eidx <= (staffIdx + span)))
-                        staff->setBracketSpan(i, span - (eidx-sidx));
-                  else {
-                        printf("TODO: adjust brackets\n");
-                        }
-                  }
-            }
+#endif
+      adjustBracketsDel(sidx, eidx);
 
       //
       //    adjust measures
@@ -728,14 +707,11 @@ void Score::cmdRemovePart(Part* part)
             m->cmdRemoveStaves(sidx, eidx);
             }
 
-      int idx = eidx - 1;
-      for (QList<Staff*>::iterator i = _staves.begin() + idx; n > 0; --n, --idx, --i) {
-            undoOp(UndoOp::RemoveStaff, *i, idx);
-            part->staves()->removeAll(*i);
+      for (int i = 0; i < n; ++i) {
+            Staff* staff = _staves[sidx];
+            undoRemoveStaff(staff, sidx);
             }
-      _staves.erase(_staves.begin() + sidx, _staves.begin() + eidx);
-      _parts.removeAt(_parts.indexOf(part));
-      undoOp(UndoOp::RemovePart, part, sidx);
+      undoRemovePart(part, sidx);
       }
 
 //---------------------------------------------------------
@@ -786,6 +762,55 @@ void Score::insertStaff(Staff* staff, int idx)
             }
       }
 
+//---------------------------------------------------------
+//   adjustBracketsDel
+//---------------------------------------------------------
+
+void Score::adjustBracketsDel(int sidx, int eidx)
+      {
+      for (int staffIdx = 0; staffIdx < _staves.size(); ++staffIdx) {
+            Staff* staff = _staves[staffIdx];
+            for (int i = 0; i < staff->bracketLevels(); ++i) {
+                  int span = staff->bracketSpan(i);
+                  if ((span == 0) || ((staffIdx + span) < sidx) || (staffIdx > eidx))
+                        continue;
+                  if ((sidx >= staffIdx) && (eidx <= (staffIdx + span)))
+                        undoChangeBracketSpan(staff, i, span - (eidx-sidx));
+//                  else {
+//                        printf("TODO: adjust brackets, span %d\n", span);
+//                        }
+                  }
+            int span = staff->barLineSpan();
+            if ((sidx >= staffIdx) && (eidx < (staffIdx + span)))
+                  undoChangeBarLineSpan(staff, span - (eidx-sidx));
+            }
+      }
+
+//---------------------------------------------------------
+//   adjustBracketsIns
+//---------------------------------------------------------
+
+void Score::adjustBracketsIns(int sidx, int eidx)
+      {
+      for (int staffIdx = 0; staffIdx < _staves.size(); ++staffIdx) {
+            Staff* staff = _staves[staffIdx];
+            int bl = staff->bracketLevels();
+            for (int i = 0; i < bl; ++i) {
+                  int span = staff->bracketSpan(i);
+                  if ((span == 0) || ((staffIdx + span) < sidx) || (staffIdx > eidx))
+                        continue;
+                  if ((sidx >= staffIdx) && (eidx <= (staffIdx + span)))
+                        undoChangeBracketSpan(staff, i, span + (eidx-sidx));
+//                  else {
+//                        printf("TODO: adjust brackets\n");
+//                        }
+                  }
+            int span = staff->barLineSpan();
+            if ((sidx >= staffIdx) && (eidx < (staffIdx + span)))
+                  undoChangeBarLineSpan(staff, span + (eidx-sidx));
+            }
+      }
+
 //---------------------------------------------------------
 //   cmdRemoveStaff
 //---------------------------------------------------------
@@ -801,8 +826,8 @@ void Score::cmdRemoveStaff(int staffIdx)
                   }
             }
       Staff* s = staff(staffIdx);
-      undoOp(UndoOp::RemoveStaff, s, staffIdx);
-      removeStaff(s);
+      undoRemoveStaff(s, staffIdx);
+      adjustBracketsDel(staffIdx, staffIdx+1);
       }
 
 //---------------------------------------------------------
@@ -830,7 +855,7 @@ void Score::removeStaff(Staff* staff)
 //   sortStaves
 //---------------------------------------------------------
 
-void Score::sortStaves(QList<int> dst)
+void Score::sortStaves(QList<int>& dst)
       {
       _layout->systems()->clear();  //??
       _parts.clear();
@@ -851,16 +876,16 @@ void Score::sortStaves(QList<int> dst)
       for (MeasureBase* mb = _measures.first(); mb; mb = mb->next()) {
             if (mb->type() != MEASURE)
                   continue;
-            Measure* m = (Measure*)mb;
+            Measure* m = static_cast<Measure*>(mb);
             m->sortStaves(dst);
             }
       foreach(Element* e, _gel) {
             if (e->type() == SLUR) {
-                  Slur* slur = (Slur*)e;
-                  int staffIdx = slur->staffIdx();
-                  int voice    = slur->voice();
-                  int staffIdx2 = slur->track() / VOICES;
-                  int voice2    = slur->track() % VOICES;
+                  Slur* slur    = static_cast<Slur*>(e);
+                  int staffIdx  = slur->startElement()->staffIdx();
+                  int voice     = slur->startElement()->voice();
+                  int staffIdx2 = slur->endElement()->staffIdx();
+                  int voice2    = slur->endElement()->voice();
                   slur->setTrack(dst[staffIdx] * VOICES + voice);
                   slur->setTrack2(dst[staffIdx2] * VOICES + voice2);
                   }
@@ -884,7 +909,7 @@ void InstrumentsDialog::on_saveButton_clicked()
       QString ext(".xml");
       QFileInfo info(name);
 
-      if (info.completeSuffix().isEmpty())
+      if (info.suffix().isEmpty())
             info.setFile(info.filePath() + ext);
       QFile f(info.filePath());
       if (!f.open(QIODevice::WriteOnly)) {
@@ -902,7 +927,7 @@ void InstrumentsDialog::on_saveButton_clicked()
             if (curGroup != t->group) {
                   if (!curGroup.isEmpty())
                         xml.etag();
-                  xml.stag(QString("instrument-group name=\"%1\"").arg(t->group));
+                  xml.stag(QString("InstrumentGroup name=\"%1\"").arg(t->group));
                   curGroup = t->group;
                   }
             t->write(xml);
