@@ -40,16 +40,40 @@
 #include "repeat.h"
 #include "box.h"
 #include "system.h"
-
-// #define OPTIMIZE_LAYOUT
+#include "part.h"
+#include "utils.h"
 
 //---------------------------------------------------------
-//   intmaxlog
+//   rebuildBspTree
 //---------------------------------------------------------
 
-static inline int intmaxlog(int n)
+void ScoreLayout::rebuildBspTree()
       {
-      return (n > 0 ? qMax(int(::ceil(::log(double(n))/::log(double(2)))), 5) : 0);
+      QRectF r;
+      QList<const Element*> el;
+      for (const MeasureBase* m = first(); m; m = m->next()) {
+            if (m->type() == MEASURE && static_cast<const Measure*>(m)->multiMeasure() < 0)
+                  continue;
+            m->collectElements(el);
+            }
+      foreach(const Page* page, _pages) {
+            r |= page->abbox();
+            page->collectElements(el);
+            }
+      foreach (const Element* element, *score()->gel()) {
+            if (element->track() != -1) {
+                  if (!element->staff()->show())
+                        continue;
+                  }
+            element->collectElements(el);
+            }
+
+      bspTree.initialize(r, el.size());
+      for (int i = 0; i < el.size(); ++i) {
+            const Element* e = el.at(i);
+            if (e)                              // DEBUG
+                  bspTree.insert(e);
+            }
       }
 
 //---------------------------------------------------------
@@ -77,18 +101,14 @@ MeasureBase* ScoreLayout::last()  const
 ScoreLayout::ScoreLayout(Score* score)
    : Element(score)
       {
-      _spatium     = ::_spatium;
-      _pageFormat  = new PageFormat;
-      _paintDevice = 0;
-
-      //DEBUG:
       _spatium = ::_spatium;
+
       _systems.clear();
       _pages.clear();
-      _needLayout = false;
-      _pageFormat = new PageFormat;
-      _paintDevice = _score->canvas();
-      startLayout = 0;
+      _needLayout  = false;
+      _pageFormat  = new PageFormat;
+      _paintDevice = 0;
+      startLayout  = 0;
       }
 
 ScoreLayout::~ScoreLayout()
@@ -137,7 +157,7 @@ Element* Score::searchNote(int tick, int track) const
 
 int Score::clefOffset(int tick, Staff* staff) const
       {
-      return clefTable[staff->clef()->clef(tick)].yOffset;
+      return clefTable[staff->clefList()->clef(tick)].yOffset;
       }
 
 //---------------------------------------------------------
@@ -149,16 +169,17 @@ int Score::clefOffset(int tick, Staff* staff) const
 
 void ScoreLayout::doLayout()
       {
-      ::_spatium = _spatium;        // needed for preview
+      ::_spatium  = _spatium;        // needed for preview
+      _spatiumMag = _spatium / (DPI * SPATIUM20);
       _needLayout = false;
 
-#ifdef OPTIMIZE_LAYOUT
       if (startLayout) {
-            doReLayout();
-            startLayout = 0;
-            return;
+            startLayout->setDirty();
+            if (doReLayout()) {
+                  startLayout = 0;
+                  return;
+                  }
             }
-#endif
 
       if (first() == 0) {
             // score is empty
@@ -171,30 +192,33 @@ void ScoreLayout::doLayout()
             page->setNo(0);
             page->setPos(0.0, 0.0);
 
-            //---------------------------------------------------
-            //    clear bspTree
-            //---------------------------------------------------
-
             QRectF r = page->abbox();
-            bspTree.initialize(r, 0);
+            bspTree.initialize(r, 0);     // clear bspTree
             return;
             }
 
+      //-----------------------------------------------------------------------
+      //    pass 0:  layout chords
+      //             set line & accidental & mirror for notes depending
+      //             on context
+      //-----------------------------------------------------------------------
+
       for (int staffIdx = 0; staffIdx < _score->nstaves(); ++staffIdx) {
             for (MeasureBase* mb = first(); mb; mb = mb->next()) {
-                  if (mb->type() != MEASURE)
-                        continue;
-                  Measure* m = (Measure*)mb;
-                  m->layout0(staffIdx);
+                  mb->setDirty();
+                  if (mb->type() == MEASURE) {
+                        Measure* m = static_cast<Measure*>(mb);
+                        m->layout0(staffIdx);
+                        }
                   }
             }
 
       //-----------------------------------------
-      //    pass I:  process pages
+      //    pass 1:  process pages
       //-----------------------------------------
 
-      curMeasure = first();
-      curSystem  = 0;
+      curMeasure  = first();
+      curSystem   = 0;
       firstSystem = true;
       for (curPage = 0; curMeasure; curPage++) {
             getCurPage();
@@ -208,14 +232,12 @@ void ScoreLayout::doLayout()
             }
 
       //---------------------------------------------------
-      //   pass II:  place ties & slurs & hairpins & beams
+      //   pass 2:  place ties & slurs & hairpins & beams
       //---------------------------------------------------
 
       for (MeasureBase* mb = first(); mb; mb = mb->next()) {
-            if (mb->type() != MEASURE)
-                  continue;
-            Measure* m = (Measure*)mb;
-            m->layout2(this);
+            if (mb->type() == MEASURE)
+                  static_cast<Measure*>(mb)->layout2(this);
             }
 
       foreach(Element* el, *score()->gel())
@@ -230,41 +252,13 @@ void ScoreLayout::doLayout()
             Page* page = _pages.takeLast();
             delete page;
             }
-
       n = _systems.size() - curSystem;
       for (int i = 0; i < n; ++i) {
             System* system = _systems.takeLast();
             delete system;
             }
 
-      searchHiddenNotes();
-
-      //---------------------------------------------------
-      //    rebuild bspTree
-      //---------------------------------------------------
-
-      QRectF r;
-      QList<const Element*> el;
-      for (const MeasureBase* m = first(); m; m = m->next())
-            m->collectElements(el);
-      foreach(const Page* page, _pages) {
-            r |= page->abbox();
-            page->collectElements(el);
-            }
-      foreach (const Element* element, *score()->gel()) {
-            if (element->track() != -1) {
-                  if (!element->staff()->show())
-                        continue;
-                  }
-            element->collectElements(el);
-            }
-
-      int depth = intmaxlog(el.size());
-      bspTree.initialize(r, depth);
-      for (int i = 0; i < el.size(); ++i) {
-            const Element* e = el.at(i);
-            bspTree.insert(e);
-            }
+      rebuildBspTree();
       }
 
 //---------------------------------------------------------
@@ -276,10 +270,9 @@ void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
       {
       int tick = m->tick();
       int nstaves = _score->nstaves();
-
       for (int i = 0; i < nstaves; ++i) {
             Staff* staff   = _score->staff(i);
-            if (!staff->show())
+            if (!m->system()->staff(i)->show())
                   continue;
 
             KeySig* hasKeysig = 0;
@@ -300,13 +293,17 @@ void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
                         continue;
                   switch (el->type()) {
                         case KEYSIG:
-                              hasKeysig = (KeySig*)el;
+                              hasKeysig = static_cast<KeySig*>(el);
                               hasKeysig->setSubtype(keyIdx);
                               hasKeysig->setMag(staff->mag());
                               break;
                         case CLEF:
-                              hasClef = (Clef*)el;
+                              hasClef = static_cast<Clef*>(el);
                               hasClef->setMag(staff->mag());
+                              hasClef->setSmall(false);
+                              break;
+                        case TIMESIG:
+                              el->setMag(staff->mag());
                               break;
                         default:
                               break;
@@ -325,30 +322,40 @@ void ScoreLayout::processSystemHeader(Measure* m, bool isFirstSystem)
                   ks->setMag(staff->mag());
                   Segment* seg = m->getSegment(ks);
                   seg->add(ks);
+                  m->setDirty();
                   }
             else if (!needKeysig && hasKeysig) {
                   int track = hasKeysig->track();
                   Segment* seg = hasKeysig->segment();
                   seg->setElement(track, 0);    // TODO: delete element
+                  m->setDirty();
                   }
             bool needClef = isFirstSystem || _score->style()->genClef;
-            if (needClef && !hasClef) {
-                  //
-                  // create missing clef
-                  //
-                  int idx = staff->clef()->clef(tick);
-                  Clef* cs = new Clef(_score, idx);
-                  cs->setTrack(i * VOICES);
-                  cs->setTick(tick);
-                  cs->setGenerated(true);
-                  cs->setMag(staff->mag());
-                  Segment* s = m->getSegment(cs);
-                  s->add(cs);
+            if (needClef) {
+                  int idx = staff->clefList()->clef(tick);
+                  if (!hasClef) {
+                        //
+                        // create missing clef
+                        //
+                        hasClef = new Clef(_score);
+                        hasClef->setTrack(i * VOICES);
+                        hasClef->setTick(tick);
+                        hasClef->setGenerated(true);
+                        hasClef->setSmall(false);
+                        hasClef->setMag(staff->mag());
+                        Segment* s = m->getSegment(hasClef);
+                        s->add(hasClef);
+                        m->setDirty();
+                        }
+                  hasClef->setSubtype(idx);
                   }
-            else if (!needClef && hasClef) {
-                  int track = hasClef->track();
-                  Segment* seg = hasClef->segment();
-                  seg->setElement(track, 0);    // TODO: delete element
+            else {
+                  if (hasClef) {
+                        int track = hasClef->track();
+                        Segment* seg = hasClef->segment();
+                        seg->setElement(track, 0);    // TODO: delete element
+                        m->setDirty();
+                        }
                   }
             }
       }
@@ -374,8 +381,9 @@ System* ScoreLayout::getNextSystem(bool isFirstSystem, bool isVbox)
             int nstaves = _score->nstaves();
             for (int i = system->staves()->size(); i < nstaves; ++i)
                   system->insertStaff(i);
-            for (int i = nstaves; i < system->staves()->size(); ++i)
-                  system->removeStaff(nstaves);
+            int dn = system->staves()->size() - nstaves;
+            for (int i = 0; i < dn; ++i)
+                  system->removeStaff(system->staves()->size()-1);
             }
       return system;
       }
@@ -402,17 +410,21 @@ void ScoreLayout::getCurPage()
 bool ScoreLayout::layoutPage()
       {
       Page* page = _pages[curPage];
+      const double slb = point(score()->style()->staffLowerBorder);
+      const double sub = point(score()->style()->staffUpperBorder);
 
       // usable width of page:
       qreal w  = page->loWidth() - page->lm() - page->rm();
       qreal x  = page->lm();
-      qreal ey = page->loHeight() - page->bm() - point(score()->style()->staffLowerBorder);
+      qreal ey = page->loHeight() - page->bm() - slb;
 
       page->clear();
       qreal y = page->tm();
 
       int rows = 0;
       bool firstSystemOnPage = true;
+
+      double nettoHeight = 0.0;
       while (curMeasure) {
             if (curMeasure->type() == VBOX) {
                   System* system = getNextSystem(false, true);
@@ -440,15 +452,17 @@ bool ScoreLayout::layoutPage()
                   curMeasure = curMeasure->next();
                   ++curSystem;
                   y += bh + score()->style()->boxSystemDistance.point();
+                  nettoHeight += bh;
                   if (y > ey)
                         break;
+                  nettoHeight += score()->style()->boxSystemDistance.point();
                   }
             else {
                   if (firstSystemOnPage) {
-                        y += point(score()->style()->staffUpperBorder);
+                        y += sub;
                         }
-                  int cs            = curSystem;
-                  MeasureBase* cm   = curMeasure;
+                  int cs          = curSystem;
+                  MeasureBase* cm = curMeasure;
                   double h;
                   QList<System*> sl = layoutSystemRow(x, y, w, firstSystem, &h);
                   if (sl.isEmpty()) {
@@ -468,28 +482,32 @@ bool ScoreLayout::layoutPage()
                         page->appendSystem(system);
                         system->setYpos(y);
                         }
-                  firstSystem = false;
+                  firstSystem       = false;
                   firstSystemOnPage = false;
                   y += h;
-                  if (sl.back()->pageBreak())
+                  nettoHeight += h;
+                  if (sl.back()->pageBreak()) {
+                        ++rows;
                         break;
+                        }
                   }
             ++rows;
             }
 
       //-----------------------------------------------------------------------
       // if remaining y space on page is greater (pageHeight*pageFillLimit)
-      // then insert space between staffs to fill page
+      // then increase system distance to fill page
       //-----------------------------------------------------------------------
 
       double restHeight = ey - y; //  + systemDistance;
-      double ph = page->height()
-            - point(score()->style()->staffLowerBorder + score()->style()->staffUpperBorder);
+
+      double ph = page->loHeight() - page->bm() - page->tm() - slb - sub;
 
       if (restHeight > (ph * (1.0 - score()->style()->pageFillLimit)))
             return true;
 
-      double dist = restHeight / (rows - 1);
+      double systemDistance = score()->style()->systemDistance.point();
+      double extraDist = (rows > 1) ? ((ph - nettoHeight + systemDistance) / (rows - 1.0)) : 0.0;
       y = 0;
       int n = page->systems()->size();
       for (int i = 0; i < n;) {
@@ -501,11 +519,43 @@ bool ScoreLayout::layoutPage()
                   break;
             System* nsystem = page->systems()->at(i);
             if (nsystem->pos().y() != yy)
-                  y += dist;                    // next system row
+                  y += extraDist;               // next system row
             }
       return true;
       }
 
+//---------------------------------------------------------
+//   skipEmptyMeasures
+//    search for empty measures; return number if empty
+//    measures in sequence
+//---------------------------------------------------------
+
+Measure* ScoreLayout::skipEmptyMeasures(Measure* m)
+      {
+      Measure* sm = m;
+      int n       = 0;
+      while (m->isEmpty()) {
+            MeasureBase* mb = m->next();
+            if (m->breakMultiMeasureRest() && n)
+                  break;
+            ++n;
+            if (!mb || (mb->type() != MEASURE))
+                  break;
+            m = static_cast<Measure*>(mb);
+            }
+      m = sm;
+      if (n >= score()->style()->minEmptyMeasures) {
+            for (int i = 0; i < (n-1); ++i) {
+                  m->setMultiMeasure(-1);
+                  m = static_cast<Measure*>(m->next());
+                  }
+            m->setMultiMeasure(n);
+            }
+      else
+            m->setMultiMeasure(0);
+      return m;
+      }
+
 //---------------------------------------------------------
 //   layoutSystem1
 //---------------------------------------------------------
@@ -514,30 +564,44 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
       {
       System* system = getNextSystem(isFirstSystem, false);
 
-      system->layout(this);
+      double xo = 0;
+      if (curMeasure->type() == HBOX)
+            xo = curMeasure->width();
+
+      system->setInstrumentNames();
+      system->layout(this, xo);
       minWidth            = system->leftMargin();
       double systemWidth  = w;
 
-      double uStretch     = 1.0;
       bool continueFlag   = false;
 
       int nstaves = _score->nstaves();
       bool isFirstMeasure = true;
+//      MeasureBase* firstMeasure = curMeasure;
 
-      for (; curMeasure; curMeasure = curMeasure->next()) {
+      for (; curMeasure;) {
+            if (curMeasure->type() == MEASURE) {
+                  Measure* m = static_cast<Measure*>(curMeasure);
+                  if (score()->style()->createMultiMeasureRests)
+                        curMeasure = skipEmptyMeasures(m);
+                  else
+                        m->setMultiMeasure(0);
+                  }
+
+            System* oldSystem = curMeasure->system();
             curMeasure->setSystem(system);
             double ww      = 0.0;
             double stretch = 0.0;
 
             if (curMeasure->type() == HBOX) {
-                  ww = ((Box*)curMeasure)->boxWidth().point();
+                  ww = static_cast<Box*>(curMeasure)->boxWidth().point();
                   if (!isFirstMeasure)
                         continueFlag = true;    //try to put another system on current row
                   }
             else if (curMeasure->type() == MEASURE) {
-                  Measure* m = (Measure*)curMeasure;
+                  Measure* m = static_cast<Measure*>(curMeasure);
                   if (isFirstMeasure)
-                        processSystemHeader((Measure*)curMeasure, isFirstSystem);
+                        processSystemHeader(m, isFirstSystem);
 
                   //
                   // remove generated elements
@@ -550,13 +614,20 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
                         for (int staffIdx = 0;  staffIdx < nstaves; ++staffIdx) {
                               int track = staffIdx * VOICES;
                               Element* el = seg->element(track);
-                              if (el) {
-                                    if (el->generated()) {
-                                          if (!isFirstMeasure || (seg->subtype() == Segment::SegTimeSigAnnounce))
-                                                seg->setElement(track, 0);
-                                          }
-                                    if (!isFirstMeasure && el->type() == CLEF)
-                                          el->setMag(el->staff()->mag() * score()->style()->smallClefMag);
+                              if (el == 0)
+                                    continue;
+                              if (el->generated()) {
+                                    if (!isFirstMeasure || (seg->subtype() == Segment::SegTimeSigAnnounce))
+                                          seg->setElement(track, 0);
+                                    }
+                              if (el->type() == CLEF) {
+                                    Clef* clef = static_cast<Clef*>(el);
+                                    if (!isFirstMeasure || (seg != m->first()))
+                                          clef->setSmall(true);
+                                    else
+                                          clef->setSmall(false);
+                                    double staffMag = score()->staff(staffIdx)->mag();
+                                    clef->setMag(staffMag);
                                     }
                               }
                         }
@@ -575,19 +646,67 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
                   }
 
             // collect at least one measure
-            if ((minWidth + ww > systemWidth) && !system->measures().isEmpty())
+            if ((minWidth + ww > systemWidth) && !system->measures().isEmpty()) {
+                  curMeasure->setSystem(oldSystem);
                   break;
+                  }
 
             minWidth += ww;
-            uStretch += stretch;
             system->measures().append(curMeasure);
             if (continueFlag || curMeasure->pageBreak() || curMeasure->lineBreak() || (curMeasure->next() && curMeasure->next()->type() == VBOX)) {
                   system->setPageBreak(curMeasure->pageBreak());
                   curMeasure = curMeasure->next();
                   break;
                   }
+            else
+                  curMeasure = curMeasure->next();
             }
-      return continueFlag;
+
+      //
+      //    hide empty staves
+      //
+      bool showChanged = false;
+      int staves = system->staves()->size();
+      int staffIdx = 0;
+      foreach (Part* p, *score()->parts()) {
+            int nstaves   = p->nstaves();
+            bool hidePart = false;
+
+            if (score()->style()->hideEmptyStaves && (staves > 1)) {
+                  hidePart = true;
+                  for (int i = staffIdx; i < staffIdx + nstaves; ++i) {
+                        foreach(MeasureBase* m, system->measures()) {
+                              if (m->type() != MEASURE)
+                                    continue;
+                              if (!((Measure*)m)->isMeasureRest(i)) {
+                                    hidePart = false;
+                                    break;
+                                    }
+                              }
+                        }
+                  }
+
+            for (int i = staffIdx; i < staffIdx + nstaves; ++i) {
+                  SysStaff* s  = system->staff(i);
+                  Staff* staff = score()->staff(i);
+                  bool oldShow = s->show();
+                  s->setShow(hidePart ? false : staff->show());
+                  if (oldShow != s->show()) {
+                        showChanged = true;
+                        }
+                  }
+            staffIdx += nstaves;
+            }
+#if 0 // DEBUG: endless recursion can happen if number of measures change
+      // relayout if stave's show status has changed
+      if (showChanged) {
+            minWidth = 0;
+            curMeasure = firstMeasure;
+            bool val = layoutSystem1(minWidth, w, isFirstSystem);
+            return val;
+            }
+#endif
+      return continueFlag && curMeasure;
       }
 
 //---------------------------------------------------------
@@ -598,6 +717,8 @@ bool ScoreLayout::layoutSystem1(double& minWidth, double w, bool isFirstSystem)
 QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
    bool isFirstSystem, double* h)
       {
+      bool raggedRight = layoutDebug;
+
       *h = 0.0;
       QList<System*> sl;
 
@@ -614,30 +735,31 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
       // dont stretch last system row, if minWidth is <= lastSystemFillLimit
       //
       if (curMeasure == 0 && ((minWidth / rowWidth) <= score()->style()->lastSystemFillLimit))
-            rowWidth = minWidth;
+            raggedRight = true;
 
       //-------------------------------------------------------
       //    Round II
       //    stretch measures
       //    "nm" measures fit on this line of score
-      //    "minWidth"   is the minimum width they use
-      //    uStretch is the accumulated userStretch
       //-------------------------------------------------------
 
       bool needRelayout = false;
 
-
       foreach(System* system, sl) {
             //
             //    add cautionary time signatures if needed
             //
 
+            if (system->measures().isEmpty()) {
+                  printf("system %p is empty\n", system);
+                  abort();
+                  }
             MeasureBase* lm = system->measures().back();
             int tick        = lm->tick() + lm->tickLen();
             SigEvent sig1   = _score->sigmap->timesig(tick - 1);
             SigEvent sig2   = _score->sigmap->timesig(tick);
 
-            if (!(sig1 == sig2)) {
+            if (score()->style()->genCourtesyTimesig && !sig1.nominalEqual(sig2)) {
                   while (lm && lm->type() != MEASURE)
                         lm = lm->prev();
                   if (lm) {
@@ -656,11 +778,12 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
                         }
                   }
 
+            const QList<MeasureBase*>& ml = system->measures();
+            int n                         = ml.size();
+
             //
             //    compute repeat bar lines
             //
-            const QList<MeasureBase*>& ml = system->measures();
-            int n                     = ml.size();
             for (int i = 0; i < n; ++i) {
                   MeasureBase* mb = ml[i];
                   if (mb->type() != MEASURE)
@@ -672,6 +795,7 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
                               m->setEndBarLineType(END_REPEAT, true);
                         else if (m->endBarLineGenerated())
                               m->setEndBarLineType(NORMAL_BAR, true);
+                        needRelayout |= m->setStartRepeatBarLine((i == 0) && (m->repeatFlags() & RepeatStart));
                         }
                   else {
                         MeasureBase* mb = m->next();
@@ -715,22 +839,29 @@ QList<System*> ScoreLayout::layoutSystemRow(qreal x, qreal y, qreal rowWidth,
             minWidth += system->leftMargin();
             }
 
-      double rest = layoutDebug ? 0.0 : rowWidth - minWidth;
-      double xx = 0.0;
-      foreach(System* system, sl) {
-            QPointF pos(system->leftMargin(), 0);
+      double rest = (raggedRight ? 0.0 : rowWidth - minWidth) / totalWeight;
+      double xx   = 0.0;
 
+      foreach(System* system, sl) {
+            QPointF pos;
+
+            bool firstMeasure = true;
             foreach(MeasureBase* mb, system->measures()) {
-                  mb->setPos(pos);
                   double ww = 0.0;
                   if (mb->type() == MEASURE) {
-                        Measure* m    = (Measure*)mb;
+                        if (firstMeasure) {
+                              pos.rx() += system->leftMargin();
+                              firstMeasure = false;
+                              }
+                        mb->setPos(pos);
+                        Measure* m    = static_cast<Measure*>(mb);
                         double weight = m->tickLen() * m->userStretch();
-                        ww            = m->layoutWidth().stretchable + rest * weight / totalWeight;
+                        ww            = m->layoutWidth().stretchable + rest * weight;
                         m->layout(this, ww);
                         }
                   else if (mb->type() == HBOX) {
-                        ww = ((Box*)mb)->boxWidth().point();
+                        mb->setPos(pos);
+                        ww = static_cast<Box*>(mb)->boxWidth().point();
                         mb->layout(this);
                         }
                   pos.rx() += ww;
@@ -845,70 +976,6 @@ void ScoreLayout::connectTies()
             }
       }
 
-//---------------------------------------------------------
-//   searchHiddenNotes
-//---------------------------------------------------------
-
-void ScoreLayout::searchHiddenNotes()
-      {
-      // TODO: should be coded more efficient
-
-      int staves = _score->nstaves();
-      int tracks = staves * VOICES;
-      for (MeasureBase* mb = first(); mb; mb = mb->next()) {
-            if (mb->type() != MEASURE)
-                  continue;
-            Measure* m = (Measure*)mb;
-            for (Segment* s = m->first(); s; s = s->next()) {
-                  if (s->subtype() != Segment::SegChordRest)
-                        continue;
-                  for (int track = 0; track < tracks; ++track) {
-                        Element* el = s->element(track);
-                        if (el == 0 || el->type() != CHORD)
-                              continue;
-                        NoteList* nl = ((Chord*)el)->noteList();
-                        for (iNote in = nl->begin(); in != nl->end(); ++in)
-                              in->second->setHidden(false);
-                        }
-                  }
-            }
-      for (MeasureBase* mb = first(); mb; mb = mb->next()) {
-            if (mb->type() != MEASURE)
-                  continue;
-            Measure* m = (Measure*)mb;
-            for (Segment* s = m->first(); s; s = s->next()) {
-                  if (s->subtype() != Segment::SegChordRest)
-                        continue;
-                  for (int staffIdx = 0; staffIdx < staves; ++staffIdx) {
-                        for (int voice = 0; voice < VOICES; ++voice) {
-                              Element* el = s->element(staffIdx * VOICES + voice);
-                              if (el == 0 || el->type() != CHORD)
-                                    continue;
-                              NoteList* nl = ((Chord*)el)->noteList();
-                              for (iNote in = nl->begin(); in != nl->end(); ++in) {
-                                    Note* note = in->second;
-                                    for (int v2 = voice; v2 < VOICES; ++v2) {
-                                          Element* e = s->element(staffIdx * VOICES + v2);
-                                          if (e == 0 || e->type() != CHORD)
-                                                continue;
-                                          NoteList* nl2 = ((Chord*)e)->noteList();
-                                          for (iNote in2 = nl2->begin(); in2 != nl2->end(); ++in2) {
-                                                Note* note2 = in2->second;
-                                                if (note2->hidden())
-                                                      continue;
-                                                if (note2->pitch() == note->pitch()) {
-                                                      if (e->tickLen() > el->tickLen())
-                                                            note->setHidden(true);
-                                                      }
-                                                }
-                                          }
-                                    }
-                              }
-                        }
-                  }
-            }
-      }
-
 //---------------------------------------------------------
 //   add
 //---------------------------------------------------------
@@ -968,105 +1035,118 @@ void ScoreLayout::reLayout(Measure* m)
 
 //---------------------------------------------------------
 //   doReLayout
+//    return true, if relayout was successful; if false
+//    a full layout must be done starting at "startLayout"
 //---------------------------------------------------------
 
-void ScoreLayout::doReLayout()
+bool ScoreLayout::doReLayout()
       {
       if (startLayout->type() == MEASURE) {
             for (int staffIdx = 0; staffIdx < _score->nstaves(); ++staffIdx)
-                  ((Measure*)startLayout)->layout0(staffIdx);
+                  static_cast<Measure*>(startLayout)->layout0(staffIdx);
             }
 
-      // collect row of systems
-      System* curSystem   = startLayout->system();
-      Page* page  = curSystem->page();
-      QList<System*>* psl = page->systems();
+      System* system  = startLayout->system();
+      qreal sysWidth  = system->width();
+      double minWidth = system->leftMargin();
 
-      int i = 0;
-      for(; i < psl->size(); ++i) {
-            if (psl->at(i) == curSystem) {
-                  printf("curSystem %d\n", i);
-                  break;
+      //
+      //  check if measures still fit in system
+      //
+      MeasureBase* m = 0;
+      foreach(m, system->measures()) {
+            double ww;
+            if (m->type() == HBOX) {
+                  ww = static_cast<Box*>(m)->boxWidth().point();
+                  }
+            else if (m->type() == MEASURE) {
+                  Measure* measure = static_cast<Measure*>(m);
+                  measure->layoutBeams1(this);
+                  measure->layoutX(this, 1.0);
+                  ww      = measure->layoutWidth().stretchable;
+                  double stretch = measure->userStretch() * score()->style()->measureSpacing;
+
+                  ww *= stretch;
+                  if (ww < point(score()->style()->minMeasureWidth))
+                        ww = point(score()->style()->minMeasureWidth);
+                  }
+            minWidth += ww;
+            }
+      if (minWidth > sysWidth)            // measure does not fit: do full layout
+            return false;
+
+      //
+      // check if another measure will fit into system
+      //
+      m = m->next();
+      if (m && m->subtype() == MEASURE) {
+            Measure* measure = static_cast<Measure*>(m);
+            measure->layoutX(this, 1.0);
+            double ww      = measure->layoutWidth().stretchable;
+            double stretch = measure->userStretch() * score()->style()->measureSpacing;
+
+            ww *= stretch;
+            if (ww < point(score()->style()->minMeasureWidth))
+                  ww = point(score()->style()->minMeasureWidth);
+            if ((minWidth + ww) <= sysWidth)    // if another measure fits, do full layout
+                  return false;
+            }
+      //
+      // stretch measures
+      //
+      minWidth    = system->leftMargin();
+      double totalWeight = 0.0;
+      foreach (MeasureBase* mb, system->measures()) {
+            if (mb->type() == HBOX)
+                  minWidth += static_cast<Box*>(mb)->boxWidth().point();
+            else {
+                  Measure* m   = static_cast<Measure*>(mb);
+                  minWidth    += m->layoutWidth().stretchable;
+                  totalWeight += m->tickLen() * m->userStretch();
                   }
             }
-      QList<System*> sl;
-      double y = curSystem->y();
-      while (i) {
-            if (psl->at(i-1)->y() != y)
-                  break;
-            --i;
-            }
-      for (; i < psl->size(); ++i) {
-            if (psl->at(i)->y() != y)
-                  break;
-            sl.append(psl->at(i));
-            }
 
-#if 0
-      //-----------------------------------------
-      //    pass I:  process pages
-      //-----------------------------------------
+      double rest = (sysWidth - minWidth) / totalWeight;
 
-      curSystem   = startLayout->system();
-      curMeasure  = curSystem->measures()->front();
-      firstSystem = false;
-
-      //---------------------------------------------------
-      //   pass II:  place ties & slurs & hairpins & beams
-      //---------------------------------------------------
-
-      foreach(MeasureBase* mb, curSystem->measures()) {
-            if (mb->type() != MEASURE)
-                  continue;
-            Measure* m = (Measure*)mb;
-            m->layout2(this);
-            }
-
-      foreach(Element* el, *score()->gel())
-            el->layout(this);
-
-      //---------------------------------------------------
-      //    remove remaining pages and systems
-      //---------------------------------------------------
-
-      int n = _pages.size() - curPage;
-      for (int i = 0; i < n; ++i) {
-            Page* page = _pages.takeLast();
-            delete page;
-            }
-
-      n = _systems.size() - curSystem;
-      for (int i = 0; i < n; ++i) {
-            System* system = _systems.takeLast();
-            delete system;
-            }
-#endif
-
-      //---------------------------------------------------
-      //    rebuild bspTree
-      //---------------------------------------------------
-
-      QRectF r;
-      QList<const Element*> el;
-      for (const MeasureBase* m = first(); m; m = m->next())
-            m->collectElements(el);
-      foreach(const Page* page, _pages) {
-            r |= page->abbox();
-            page->collectElements(el);
-            }
-      foreach (const Element* element, *score()->gel()) {
-            if (element->track() != -1) {
-                  if (!element->staff()->show())
-                        continue;
+      bool firstMeasure = true;
+      QPointF pos;
+      foreach(MeasureBase* mb, system->measures()) {
+            double ww = 0.0;
+            if (mb->type() == MEASURE) {
+                  if (firstMeasure) {
+                        pos.rx() += system->leftMargin();
+                        firstMeasure = false;
+                        }
+                  mb->setPos(pos);
+                  Measure* m    = static_cast<Measure*>(mb);
+                  double weight = m->tickLen() * m->userStretch();
+                  ww            = m->layoutWidth().stretchable + rest * weight;
+                  m->layout(this, ww);
                   }
-            element->collectElements(el);
+            else if (mb->type() == HBOX) {
+                  mb->setPos(pos);
+                  ww = static_cast<Box*>(mb)->boxWidth().point();
+                  mb->layout(this);
+                  }
+            pos.rx() += ww;
             }
 
-      int depth = intmaxlog(el.size());
-      bspTree.initialize(r, depth);
-      for (int i = 0; i < el.size(); ++i) {
-            const Element* e = el.at(i);
-            bspTree.insert(e);
+      foreach(MeasureBase* mb, system->measures()) {
+            if (mb->type() == MEASURE)
+                  static_cast<Measure*>(mb)->layout2(this);
             }
+
+      rebuildBspTree();
+      return true;
+      }
+
+//---------------------------------------------------------
+//   clear
+//---------------------------------------------------------
+
+void ScoreLayout::clear()
+      {
+      _pages.clear();
+      _systems.clear();
       }
 
