@@ -19,6 +19,7 @@
 //=============================================================================
 
 #include "line.h"
+#include "textline.h"
 #include "segment.h"
 #include "measure.h"
 #include "score.h"
@@ -61,33 +62,29 @@ QPointF LineSegment::gripAnchor(int grip) const
       QPointF sp(sbb.topLeft() + _system->canvasPos());
 
       System* s;  // dummy
-      switch(_segmentType) {
-            case SEGMENT_SINGLE:
-                  if (grip == 0)
-                        return line()->tick2pos(grip, line()->tick(), staffIdx, &s);
-                  else if (grip == 1)
-                        return line()->tick2pos(grip, line()->tick2(), staffIdx, &s);
-                  return QPointF();
-            case SEGMENT_BEGIN:
-                  if (grip == 0)
-                        return line()->tick2pos(grip, line()->tick(), staffIdx, &s);
-                  else if (grip == 1) {
-                        double x = sbb.width() + _system->canvasPos().x();
-                        return QPointF(x, sp.y());
-                        }
-            case SEGMENT_MIDDLE:
-                  if (grip == 0)
+      QPointF p1(line()->tick2pos(grip, line()->tick(), staffIdx, &s));
+      QPointF p2(sbb.x() + sbb.width() + _system->canvasPos().x(), sp.y());
+      QPointF p3(line()->tick2pos(grip, line()->tick2(), staffIdx, &s));
+
+      if (grip == 0) {
+            switch(_segmentType) {
+                  case SEGMENT_SINGLE:
+                  case SEGMENT_BEGIN:
+                        return p1;
+                  case SEGMENT_MIDDLE:
+                  case SEGMENT_END:
                         return sp;
-                  else if (grip == 1) {
-                        double x = sbb.width() + _system->canvasPos().x();
-                        return QPointF(x, sp.y());
-                        }
-            case SEGMENT_END:
-                  if (grip == 0)
-                        return sp;
-                  else if (grip == 1)
-                        return line()->tick2pos(grip, line()->tick2(), staffIdx, &s);
-                  break;
+                  }
+            }
+      else {
+            switch(_segmentType) {
+                  case SEGMENT_SINGLE:
+                  case SEGMENT_END:
+                        return p3;
+                  case SEGMENT_BEGIN:
+                  case SEGMENT_MIDDLE:
+                        return p2;
+                  }
             }
       return QPointF();
       }
@@ -96,7 +93,7 @@ QPointF LineSegment::gripAnchor(int grip) const
 //   startEdit
 //---------------------------------------------------------
 
-bool LineSegment::startEdit(const QPointF&)
+bool LineSegment::startEdit(Viewer*, const QPointF&)
       {
       return true;
       }
@@ -106,37 +103,33 @@ bool LineSegment::startEdit(const QPointF&)
 //    return true if event is accepted
 //---------------------------------------------------------
 
-bool LineSegment::edit(int curGrip, QKeyEvent* ev)
+bool LineSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modifiers, const QString&)
       {
-      if ((ev->modifiers() & Qt::ShiftModifier)
+      if ((modifiers & Qt::ShiftModifier)
          && ((_segmentType == SEGMENT_SINGLE)
               || (_segmentType == SEGMENT_BEGIN && curGrip == 0)
               || (_segmentType == SEGMENT_END && curGrip == 1)
          )) {
+            int segments = line()->lineSegments().size();
             int track = line()->track();
             int tick1 = line()->tick();
             int tick2 = line()->tick2();
 
-            if (ev->key() == Qt::Key_Left) {
+            if (key == Qt::Key_Left) {
                   if (curGrip == 0) {
                         int t1 = score()->prevSeg1(tick1, track);
                         if (t1 >= 0)
                               tick1 = t1;
                         }
                   else if (curGrip == 1) {
-                        int segments = line()->lineSegments().size();
                         int t2 = score()->prevSeg1(tick2, track);
                         if (t2 >= 0)
                               tick2 = t2;
-                        if (tick1 > tick2)
+                        if (tick1 >= tick2)
                               return true;
-                        line()->setTick2(tick2);
-                        line()->layout(score()->layout());
-                        if (line()->lineSegments().size() != segments)
-                              score()->changeLineSegment(true);
                         }
                   }
-            else if (ev->key() == Qt::Key_Right) {
+            else if (key == Qt::Key_Right) {
                   if (curGrip == 0) {
                         int t1 = score()->nextSeg1(tick1, track);
                         if (t1 >= 0)
@@ -145,18 +138,17 @@ bool LineSegment::edit(int curGrip, QKeyEvent* ev)
                               return true;
                         }
                   else if (curGrip == 1) {
-                        int segments = line()->lineSegments().size();
                         int t2 = score()->nextSeg1(tick2, track);
                         if (t2 >= 0)
                               tick2 = t2;
-                        line()->setTick2(tick2);
-                        line()->layout(score()->layout());
-                        if (line()->lineSegments().size() != segments)
-                              score()->changeLineSegment(true);
                         }
                   }
             line()->setTick(tick1);
             line()->setTick2(tick2);
+
+            line()->layout(score()->layout());
+            if (line()->lineSegments().size() != segments)
+                  score()->changeLineSegment(curGrip == 1);
             return true;
             }
       return false;
@@ -179,7 +171,13 @@ QPointF LineSegment::pos2anchor(const QPointF& pos, int* tick) const
 
 void LineSegment::editDrag(int curGrip, const QPointF& d)
       {
-      QPointF delta(d.x() / _spatium, 0);    // only x-axis move
+      QPointF delta(d.x() / _spatium, 0);
+
+      if (line()->type() == TEXTLINE) {
+            TextLine* tl = static_cast<TextLine*>(line());
+            if (tl->diagonal())
+                  delta.setY(d.y() / _spatium);
+            }
 
       if (curGrip == 0) {
             _userOff  += delta;
@@ -189,22 +187,6 @@ void LineSegment::editDrag(int curGrip, const QPointF& d)
             _userOff2 += delta;
       }
 
-//---------------------------------------------------------
-//   endEditDrag
-//---------------------------------------------------------
-
-void LineSegment::endEditDrag()
-      {
-      }
-
-//---------------------------------------------------------
-//   endEdit
-//---------------------------------------------------------
-
-void LineSegment::endEdit()
-      {
-      }
-
 //---------------------------------------------------------
 //   resetUserOffsets
 //---------------------------------------------------------
@@ -224,6 +206,7 @@ SLine::SLine(Score* s)
       {
       setTick(0);
       _tick2 = 0;
+      _diagonal = false;
       }
 
 //---------------------------------------------------------
@@ -239,7 +222,7 @@ void SLine::setTick2(int t)
 //   tick2pos
 //---------------------------------------------------------
 
-QPointF SLine::tick2pos(int, int tick, int staffIdx, System** system)
+QPointF SLine::tick2pos(int grip, int tick, int staffIdx, System** system)
       {
       Segment* seg = _score->tick2segment(tick);
       if (seg == 0) {
@@ -259,6 +242,15 @@ QPointF SLine::tick2pos(int, int tick, int staffIdx, System** system)
             seg = m->last();
             }
       System* sys = seg->measure()->system();
+
+      // do not go into next system when tick2 is start of system
+      if (grip == 1 && sys->firstMeasure()->tick() == seg->tick()) {
+            Measure* m = sys->firstMeasure()->prevMeasure();
+            if (m) {
+                  seg = m->last();
+                  sys = seg->measure()->system();
+                  }
+            }
       *system     = sys;
       return QPointF(seg->canvasPos().x(), sys->staff(staffIdx)->bbox().y() + sys->canvasPos().y());
       }
@@ -293,20 +285,25 @@ void SLine::layout(ScoreLayout* layout)
       QPointF p2 = tick2pos(1, _tick2, staffI, &s2);
 
       QList<System*>* systems = layout->systems();
-
       int sysIdx1        = systems->indexOf(s1);
-      int sysIdx2        = systems->indexOf(s2, sysIdx1);
+      int sysIdx2        = systems->indexOf(s2);
       int segmentsNeeded = sysIdx2 - sysIdx1 + 1;
 
       int segCount       = segments.size();
       if (segmentsNeeded != segCount) {
             if (segmentsNeeded > segCount) {
                   int n = segmentsNeeded - segCount;
-                  for (int i = 0; i < n; ++i)
-                        add(createLineSegment());
+                  for (int i = 0; i < n; ++i) {
+                        LineSegment* ls = createLineSegment();
+                        add(ls);
+                        // set user offset to previous segment's offset
+                        if (segCount > 0)
+                              ls->setUserOff(QPointF(0, segments[segCount+i-1]->userOff().y()));
+                        }
                   }
             else {
                   int n = segCount - segmentsNeeded;
+                  printf("SLine: segments %d needed %d, remove %d\n", segCount, segmentsNeeded, n);
                   for (int i = 0; i < n; ++i) {
                         if (segments.isEmpty()) {
                               printf("SLine::layout(): no segment %d, %d expected\n", i, n);
@@ -361,8 +358,14 @@ void SLine::layout(ScoreLayout* layout)
 void SLine::writeProperties(Xml& xml) const
       {
       Element::writeProperties(xml);
+      if (parent() == 0) {
+            // when used as icon
+            LineSegment* s = segments.front();
+            xml.tag("length", s->pos2().x());
+            return;
+            }
       xml.tag("tick2", _tick2);
-//      xml.tag("track", track());
+      xml.tag("diagonal", _diagonal);
       //
       // check if user has modified the default layout
       //
@@ -393,11 +396,16 @@ void SLine::writeProperties(Xml& xml) const
 
 bool SLine::readProperties(QDomElement e)
       {
+      foreach(LineSegment* seg, segments)
+            delete seg;
+      segments.clear();
+
       if (Element::readProperties(e))
             return true;
       QString tag(e.tagName());
       QString val(e.text());
       int i = val.toInt();
+
       if (tag == "tick2")
             _tick2 = score()->fileDivision(i);
       else if (tag == "Segment") {
@@ -412,8 +420,12 @@ bool SLine::readProperties(QDomElement e)
                   }
             add(ls);
             }
-      else if (e.tagName() == "track")
-            setTrack(e.text().toInt());
+      else if (tag == "track")
+            setTrack(i);
+      else if (tag == "length")
+            setLen(val.toDouble());
+      else if (tag == "diagonal")
+            setDiagonal(i);
       else
             return false;
       return true;
