@@ -45,6 +45,12 @@
 #include "line.h"
 #include "textline.h"
 #include "system.h"
+#include "arpeggio.h"
+#include "glissando.h"
+#include "tremolo.h"
+#include "articulation.h"
+
+extern bool useFactorySettings;
 
 //---------------------------------------------------------
 //   ElementItem
@@ -108,11 +114,11 @@ void ElementItem::init()
 //   PageListEditor
 //---------------------------------------------------------
 
-PageListEditor::PageListEditor(Score* s)
-   : QWidget()
+PageListEditor::PageListEditor(QWidget* parent)
+   : QWidget(parent)
       {
+      setWindowFlags(Qt::Dialog);
       setWindowTitle(tr("MuseScore: Object Inspector"));
-      cs = s;
       QHBoxLayout* hbox = new QHBoxLayout;
       setLayout(hbox);
 
@@ -208,12 +214,14 @@ PageListEditor::PageListEditor(Score* s)
       connect(list, SIGNAL(itemExpanded(QTreeWidgetItem*)), SLOT(itemExpanded(QTreeWidgetItem*)));
       connect(list, SIGNAL(itemCollapsed(QTreeWidgetItem*)), SLOT(itemExpanded(QTreeWidgetItem*)));
       list->resizeColumnToContents(0);
-      QSettings settings;
-      settings.beginGroup("Inspector");
-      split->restoreState(settings.value("splitter").toByteArray());
-      resize(settings.value("size", QSize(1000, 500)).toSize());
-      move(settings.value("pos", QPoint(10, 10)).toPoint());
-      settings.endGroup();
+      if (!useFactorySettings) {
+            QSettings settings;
+            settings.beginGroup("Inspector");
+            split->restoreState(settings.value("splitter").toByteArray());
+            resize(settings.value("size", QSize(1000, 500)).toSize());
+            move(settings.value("pos", QPoint(10, 10)).toPoint());
+            settings.endGroup();
+            }
       }
 
 //---------------------------------------------------------
@@ -240,12 +248,40 @@ void PageListEditor::layoutScore()
       cs->end();
       }
 
+//---------------------------------------------------------
+//   addSymbol
+//---------------------------------------------------------
+
+void PageListEditor::addSymbol(ElementItem* parent, BSymbol* bs)
+      {
+      const QList<Element*>el = bs->getLeafs();
+      ElementItem* i = new ElementItem(parent, bs);
+      if (!el.isEmpty()) {
+            foreach(Element* g, el)
+                  addSymbol(i, static_cast<BSymbol*>(g));
+            }
+      }
+
+//---------------------------------------------------------
+//   addMeasureBaseToList
+//---------------------------------------------------------
+
+static void addMeasureBaseToList(ElementItem* mi, MeasureBase* mb)
+      {
+      foreach(Element* e, *mb->el()) {
+            ElementItem* mmi = new ElementItem(mi, e);
+            if (e->type() == HBOX || e->type() == VBOX)
+                  addMeasureBaseToList(mmi, static_cast<MeasureBase*>(e));
+            }
+      }
+
 //---------------------------------------------------------
 //   updateList
 //---------------------------------------------------------
 
-void PageListEditor::updateList()
+void PageListEditor::updateList(Score* s)
       {
+      cs = s;
       list->clear();
       ElementItem* li = new ElementItem(list, cs->layout());
       foreach(Element* el, *cs->gel()) {
@@ -260,8 +296,10 @@ void PageListEditor::updateList()
                   SLine* line = (SLine*)el;
                   foreach(LineSegment* ls, line->lineSegments()) {
                         ElementItem* sse = new ElementItem(se, ls);
-                        if (ls->type() == TEXTLINE_SEGMENT)
-                              new ElementItem(sse, ((TextLineSegment*)ls)->text());
+                        if (ls->type() == TEXTLINE_SEGMENT) {
+                              if (static_cast<TextLineSegment*>(ls)->text())
+                                    new ElementItem(sse, static_cast<TextLineSegment*>(ls)->text());
+                              }
                         }
                   }
             else
@@ -289,12 +327,13 @@ void PageListEditor::updateList()
                   foreach (MeasureBase* mb, system->measures()) {
                         ElementItem* mi = new ElementItem(si, mb);
 
-                        foreach(Element* e, *mb->el())
-                              new ElementItem(mi, e);
+                        addMeasureBaseToList(mi, mb);
 
                         if (mb->type() != MEASURE)
                               continue;
                         Measure* measure = (Measure*) mb;
+                        if (measure->noText())
+                              new ElementItem(mi, measure->noText());
                         for (Segment* segment = measure->first(); segment; segment = segment->next()) {
                               ElementItem* segItem = new ElementItem(mi, segment);
                               for (int track = 0; track < tracks; ++track) {
@@ -308,7 +347,14 @@ void PageListEditor::updateList()
                                                 new ElementItem(sei, chord->hook());
                                           if (chord->stem())
                                                 new ElementItem(sei, chord->stem());
-                                          foreach(NoteAttribute* a, *chord->getAttributes())
+                                          if (chord->arpeggio())
+                                                new ElementItem(sei, chord->arpeggio());
+                                          if (chord->tremolo())
+                                                new ElementItem(sei, chord->tremolo());
+                                          if (chord->glissando())
+                                                new ElementItem(sei, chord->glissando());
+
+                                          foreach(Articulation* a, *chord->getArticulations())
                                                 new ElementItem(sei, a);
                                           foreach(LedgerLine* h, *chord->ledgerLines())
                                                 new ElementItem(sei, h);
@@ -319,8 +365,14 @@ void PageListEditor::updateList()
                                                 if (note->accidental()) {
                                                       new ElementItem(ni, note->accidental());
                                                       }
-                                                foreach(Element* f, *note->el())
-                                                      new ElementItem(ni, f);
+                                                foreach(Element* f, *note->el()) {
+                                                      if (f->type() == SYMBOL || f->type() == IMAGE) {
+                                                            BSymbol* bs = static_cast<BSymbol*>(f);
+                                                            addSymbol(ni, bs);
+                                                            }
+                                                      else
+                                                            new ElementItem(ni, f);
+                                                      }
                                                 if (note->tieFor()) {
                                                       Tie* tie = note->tieFor();
                                                       ElementItem* ti = new ElementItem(ni, tie);
@@ -337,7 +389,7 @@ void PageListEditor::updateList()
                                           }
                                     }
                               }
-                        foreach(Beam* beam, *measure->beamList())
+                        foreach(Beam* beam, *measure->beams())
 					new ElementItem(mi, beam);
                         foreach(Tuplet* tuplet, *measure->tuplets()) {
 					ElementItem* item = new ElementItem(mi, tuplet);
@@ -540,7 +592,7 @@ void MeasureView::setElement(Element* e)
 
       mb.segments->setValue(m->size());
       mb.staves->setValue(m->staffList()->size());
-      mb.beams->setValue(m->beamList()->size());
+      mb.beams->setValue(m->beams()->size());
       mb.tuplets->setValue(m->tuplets()->size());
       mb.measureNo->setValue(m->no());
       mb.noOffset->setValue(m->noOffset());
@@ -670,6 +722,8 @@ void ShowChordWidget::setElement(Element* e)
       crb.tupletButton->setEnabled(chord->tuplet());
       crb.upFlag->setChecked(chord->up());
       crb.beamMode->setCurrentIndex(int(chord->beamMode()));
+      crb.isUpFlag->setChecked(chord->isUp());
+      crb.dots->setValue(chord->dots());
 
       cb.hookButton->setEnabled(chord->hook());
       cb.stemButton->setEnabled(chord->stem());
@@ -677,7 +731,7 @@ void ShowChordWidget::setElement(Element* e)
       cb.stemDirection->setCurrentIndex(int(chord->stemDirection()));
 
       crb.attributes->clear();
-      foreach(NoteAttribute* a, *chord->getAttributes()) {
+      foreach(Articulation* a, *chord->getArticulations()) {
             QString s;
             s.setNum(long(a), 16);
             QListWidgetItem* item = new QListWidgetItem(s, 0, long(a));
@@ -796,24 +850,24 @@ void ShowNoteWidget::setElement(Element* e)
       nb.pitch->setValue(note->pitch());
       nb.line->setValue(note->line());
       nb.move->setValue(note->staffMove());
-      nb.dots->setValue(note->dots());
       nb.mirror->setChecked(note->mirror());
       nb.tpc->setValue(note->tpc());
       nb.head->setValue(note->noteHead());
       nb.headGroup->setValue(note->headGroup());
+      nb.hidden->setChecked(note->hidden());
+      nb.subchannel->setValue(note->subchannel());
 
       nb.tieFor->setEnabled(note->tieFor());
       nb.tieBack->setEnabled(note->tieBack());
       nb.accidental->setEnabled(note->accidental());
+      nb.userAccidental->setValue(note->userAccidental());
 
-#if 0 // TODO
-      foreach(Text* text, note->fingering()) {
+      foreach(Element* text, *note->el()) {
             QString s;
             s.setNum(long(text), 16);
             QListWidgetItem* item = new QListWidgetItem(s, 0, long(text));
             nb.fingering->addItem(item);
             }
-#endif
       }
 
 //---------------------------------------------------------
@@ -870,6 +924,8 @@ ShowRestWidget::ShowRestWidget()
       crb.beamMode->addItem(tr("beam end"));
       crb.beamMode->addItem(tr("no beam"));
       crb.beamMode->addItem(tr("begin 1/32"));
+      connect(crb.beamButton, SIGNAL(clicked()), SLOT(beamClicked()));
+      connect(crb.tupletButton, SIGNAL(clicked()), SLOT(tupletClicked()));
 
       QFrame* line = new QFrame(this);
       line->setFrameStyle(QFrame::HLine | QFrame::Raised);
@@ -900,7 +956,9 @@ void ShowRestWidget::setElement(Element* e)
       crb.upFlag->setChecked(rest->up());
       crb.beamMode->setCurrentIndex(int(rest->beamMode()));
       crb.attributes->clear();
-      foreach(NoteAttribute* a, *rest->getAttributes()) {
+      crb.dots->setValue(rest->dots());
+
+      foreach(Articulation* a, *rest->getArticulations()) {
             QString s;
             s.setNum(long(a), 16);
             QListWidgetItem* item = new QListWidgetItem(s, 0, long(a));
@@ -923,6 +981,24 @@ void ShowRestWidget::setElement(Element* e)
       segment->setValue(seg);
       }
 
+//---------------------------------------------------------
+//   beamClicked
+//---------------------------------------------------------
+
+void ShowRestWidget::beamClicked()
+      {
+      emit elementChanged(static_cast<Rest*>(element())->beam());
+      }
+
+//---------------------------------------------------------
+//   tupletClicked
+//---------------------------------------------------------
+
+void ShowRestWidget::tupletClicked()
+      {
+      emit elementChanged(static_cast<Rest*>(element())->tuplet());
+      }
+
 //---------------------------------------------------------
 //   ShowClefWidget
 //---------------------------------------------------------
@@ -1039,7 +1115,7 @@ void TextView::setElement(Element* e)
       {
       Text* te = (Text*)e;
       ShowElementBase::setElement(e);
-      tb.text->setDocument(te->doc());
+      tb.text->setDocument(te->doc()->clone());
       tb.xoffset->setValue(te->xoff());
       tb.yoffset->setValue(te->yoff());
       tb.rxoffset->setValue(te->rxoff());
@@ -1154,22 +1230,22 @@ TupletView::TupletView()
 //   hasNumberToggled
 //---------------------------------------------------------
 
-void TupletView::hasNumberToggled(bool val)
+void TupletView::hasNumberToggled(bool /*val*/)
       {
-      Tuplet* tuplet = (Tuplet*)element();
-      tuplet->setHasNumber(val);
-      emit scoreChanged();
+//      Tuplet* tuplet = (Tuplet*)element();
+//      tuplet->setHasNumber(val);
+//      emit scoreChanged();
       }
 
 //---------------------------------------------------------
 //   hasLineToggled
 //---------------------------------------------------------
 
-void TupletView::hasLineToggled(bool val)
+void TupletView::hasLineToggled(bool /*val*/)
       {
-      Tuplet* tuplet = (Tuplet*)element();
-      tuplet->setHasLine(val);
-      emit scoreChanged();
+//      Tuplet* tuplet = (Tuplet*)element();
+//      tuplet->setHasLine(val);
+//      emit scoreChanged();
       }
 
 //---------------------------------------------------------
@@ -1199,21 +1275,19 @@ void TupletView::setElement(Element* e)
       {
       ShowElementBase::setElement(e);
       Tuplet* tuplet = (Tuplet*)e;
-      tb.hasNumber->setChecked(tuplet->hasNumber());
-      tb.hasLine->setChecked(tuplet->hasLine());
+//      tb.hasNumber->setChecked(tuplet->hasNumber());
+//      tb.hasLine->setChecked(tuplet->hasLine());
       tb.baseLen->setValue(tuplet->baseLen());
       tb.normalNotes->setValue(tuplet->normalNotes());
       tb.actualNotes->setValue(tuplet->actualNotes());
       tb.number->setEnabled(tuplet->number());
-      ChordRestList* el = tuplet->elements();
       tb.elements->clear();
-      for (iChordRest i = el->begin(); i != el->end(); ++i) {
+      foreach(DurationElement* e, tuplet->elements()) {
             QTreeWidgetItem* item = new QTreeWidgetItem;
-            ChordRest* cr = i->second;
-            item->setText(0, cr->name());
-            item->setText(1, QString("%1").arg(cr->tick()));
-            item->setText(2, QString("%1").arg(cr->tickLen()));
-            void* p = (void*) cr;
+            item->setText(0, e->name());
+            item->setText(1, QString("%1").arg(e->tick()));
+            item->setText(2, QString("%1").arg(e->tickLen()));
+            void* p = (void*) e;
             item->setData(0, Qt::UserRole, QVariant::fromValue<void*>(p));
             tb.elements->addTopLevelItem(item);
             }
@@ -1324,7 +1398,7 @@ void ShowElementBase::selectedClicked(bool val)
       {
       QRectF r(el->abbox());
       if (val)
-            el->score()->select(el, Qt::ShiftModifier, 0);
+            el->score()->select(el, SELECT_ADD, 0);
       else
             el->score()->deselect(el);
       el->score()->addRefresh(r | el->abbox());
