@@ -34,11 +34,18 @@ class MeasureProperties : public QDialog, private Ui::MeasurePropertiesBase {
       Q_OBJECT
       Measure* m;
 
-   public:
-      MeasureProperties(Measure*, QWidget* parent = 0);
+      void apply();
       SigEvent sig() const;
       bool isIrregular() const;
       int repeatCount() const;
+      bool visible(int staffIdx);
+      bool slashStyle(int staffIdx);
+
+   private slots:
+      void bboxClicked(QAbstractButton* button);
+
+   public:
+      MeasureProperties(Measure*, QWidget* parent = 0);
       };
 
 #endif
