@@ -35,6 +35,7 @@
 #include "slur.h"
 #include "dynamics.h"
 #include "lyrics.h"
+#include "articulation.h"
 
 //---------------------------------------------------------
 //   musicalAttribute
@@ -86,7 +87,7 @@ void MuseData::musicalAttribute(QString s, Part* part)
                               break;
                         }
                   Staff* staff = part->staff(staffIdx);
-                  staff->clef()->setClef(curTick, mscoreClef);
+                  staff->clefList()->setClef(curTick, mscoreClef);
                   }
             else
                   printf("unknown $key <%s>\n", qPrintable(item));
@@ -234,6 +235,7 @@ void MuseData::readNote(Part* part, const QString& s)
                         tuplet = new Tuplet(score);
                         tuplet->setTrack(gstaff * VOICES);
                         tuplet->setBaseLen(ticks * a / b);
+                        tuplet->setTick(tick);
 
                         ntuplet = a;
                         tuplet->setNormalNotes(b);
@@ -302,42 +304,42 @@ void MuseData::readNote(Part* part, const QString& s)
             else if (an[i] == 'x')
                   closeSlur(3, tick, staff, voice);
             else if (an[i] == '.') {
-                  NoteAttribute* atr = new NoteAttribute(score);
+                  Articulation* atr = new Articulation(score);
                   atr->setSubtype(StaccatoSym);
                   chord->add(atr);
                   }
             else if (an[i] == '_') {
-                  NoteAttribute* atr = new NoteAttribute(score);
+                  Articulation* atr = new Articulation(score);
                   atr->setSubtype(TenutoSym);
                   chord->add(atr);
                   }
             else if (an[i] == 'v') {
-                  NoteAttribute* atr = new NoteAttribute(score);
+                  Articulation* atr = new Articulation(score);
                   atr->setSubtype(UpbowSym);
                   chord->add(atr);
                   }
             else if (an[i] == 'n') {
-                  NoteAttribute* atr = new NoteAttribute(score);
+                  Articulation* atr = new Articulation(score);
                   atr->setSubtype(DownbowSym);
                   chord->add(atr);
                   }
             else if (an[i] == 't') {
-                  NoteAttribute* atr = new NoteAttribute(score);
+                  Articulation* atr = new Articulation(score);
                   atr->setSubtype(TrillSym);
                   chord->add(atr);
                   }
             else if (an[i] == 'F') {
-                  NoteAttribute* atr = new NoteAttribute(score);
+                  Articulation* atr = new Articulation(score);
                   atr->setSubtype(UfermataSym);
                   chord->add(atr);
                   }
             else if (an[i] == 'E') {
-                  NoteAttribute* atr = new NoteAttribute(score);
+                  Articulation* atr = new Articulation(score);
                   atr->setSubtype(DfermataSym);
                   chord->add(atr);
                   }
             else if (an[i] == 'O') {
-                  // NoteAttribute* atr = new NoteAttribute(score);
+                  // Articulation* atr = new Articulation(score);
                   // atr->setSubtype(DownbowSym);
                   // chord->add(atr);
                   printf("%06d: open string '%c' not implemented\n", tick, an[i].toAscii());
@@ -673,7 +675,7 @@ bool MuseData::read(const QString& name)
                               staff->setBracketSpan(0, 2);
                               }
                         }
-                  score->parts()->push_back(mpart);
+                  score->appendPart(mpart);
                   mpart->setLongName(part[8]);
                   part.clear();
                   continue;
