@@ -27,6 +27,8 @@
 #include "clef.h"
 #include "part.h"
 #include "drumset.h"
+#include "palette.h"
+#include "keysig.h"
 
 //---------------------------------------------------------
 //   InstrumentWizard
@@ -305,96 +307,49 @@ void InstrumentWizard::createInstruments(Score* cs)
       QTreeWidget* pl = partiturList;
       Part* part   = 0;
       int staffIdx = 0;
-      int rstaff   = 0;
 
       QTreeWidgetItem* item = 0;
       for (int idx = 0; (item = pl->topLevelItem(idx)); ++idx) {
-            rstaff = 0;
             PartListItem* pli = (PartListItem*)item;
-            if (pli->op == ITEM_DELETE)
-                  cs->cmdRemovePart(pli->part);
-            else if (pli->op == ITEM_ADD) {
-                  const InstrumentTemplate* t = ((PartListItem*)item)->it;
-                  part = new Part(cs);
-                  part->setMidiProgram(t->midiProgram);
-                  part->setMinPitch(t->minPitch);
-                  part->setMaxPitch(t->maxPitch);
-                  part->setShortName(t->shortName);
-                  part->setTrackName(t->name);
-                  part->setLongName(t->name);
-                  part->setPitchOffset(t->transpose);
-                  if (t->useDrumset) {
-                        part->setUseDrumset(true);
-                        part->setDrumset(new Drumset(*smDrumset));
-                        }
-
-                  pli->part = part;
-                  QTreeWidgetItem* ci = 0;
-                  rstaff = 0;
-                  for (int cidx = 0; (ci = pli->child(cidx)); ++cidx) {
-                        StaffListItem* sli = (StaffListItem*)ci;
-                        Staff* staff = new Staff(cs, part, rstaff);
-                        sli->staff = staff;
-                        staff->setRstaff(rstaff);
-                        ++rstaff;
-                        staff->clef()->setClef(0, sli->clef());
-                        staff->setLines(t->staffLines[cidx]);
-                        staff->setSmall(t->smallStaff[cidx]);
-                        if (cidx == 0) {
-                              staff->setBracket(0, t->bracket);
-                              staff->setBracketSpan(0, t->staves);
-                              }
-                        part->staves()->push_back(staff);
-                        cs->staves().insert(staffIdx + rstaff, staff);
-                        cs->undoOp(UndoOp::InsertStaff, staff, staffIdx+rstaff);
-                        }
-                  cs->cmdInsertPart(part, staffIdx);
-                  staffIdx += rstaff;
+            if (pli->op != ITEM_ADD) {
+                  printf("bad op\n");
+                  continue;
                   }
-            else {
-                  part = pli->part;
-                  QTreeWidgetItem* ci = 0;
-                  for (int cidx = 0; (ci = pli->child(cidx)); ++cidx) {
-                        StaffListItem* sli = (StaffListItem*)ci;
-                        if (sli->op == ITEM_DELETE) {
-                              cs->layout()->systems()->clear();
-                              Staff* staff = sli->staff;
-                              int sidx = staff->idx();
-                              int eidx = sidx + 1;
-                              for (MeasureBase* mb = cs->layout()->first(); mb; mb = mb->next()) {
-                                    if (mb->type() != MEASURE)
-                                          continue;
-                                    Measure* m = (Measure*)mb;
-                                    m->cmdRemoveStaves(sidx, eidx);
-                                    }
-                              cs->undoOp(UndoOp::RemoveStaff, staff, sidx);
-                              cs->removeStaff(staff);
-                              }
-                        else if (sli->op == ITEM_ADD) {
-                              Staff* staff = new Staff(cs, part, rstaff);
-                              sli->staff = staff;
-                              staff->setRstaff(rstaff);
-                              ++rstaff;
-                              staff->clef()->setClef(0, sli->clef());
+            const InstrumentTemplate* t = ((PartListItem*)item)->it;
+            part = new Part(cs);
+            part->initFromInstrTemplate(t);
 
-                              cs->insertStaff(staff, staffIdx);
-                              cs->undoOp(UndoOp::InsertStaff, staff, staffIdx);
-
-                              for (MeasureBase* mb = cs->layout()->first(); mb; mb = mb->next()) {
-                                    if (mb->type() != MEASURE)
-                                          continue;
-                                    Measure* m = (Measure*)mb;
-                                    m->cmdAddStaves(staffIdx, staffIdx+1);
-                                    }
-
-                              ++staffIdx;
-                              }
-                        else {
-                              ++staffIdx;
-                              ++rstaff;
-                              }
+            pli->part = part;
+            QTreeWidgetItem* ci = 0;
+            int rstaff = 0;
+            for (int cidx = 0; (ci = pli->child(cidx)); ++cidx) {
+                  StaffListItem* sli = (StaffListItem*)ci;
+                  Staff* staff = new Staff(cs, part, rstaff);
+                  sli->staff = staff;
+                  staff->setRstaff(rstaff);
+                  ++rstaff;
+                  staff->clefList()->setClef(0, sli->clef());
+                  staff->setLines(t->staffLines[cidx]);
+                  staff->setSmall(t->smallStaff[cidx]);
+                  if (cidx == 0) {
+                        staff->setBracket(0, t->bracket);
+                        staff->setBracketSpan(0, t->staves);
                         }
+                  part->staves()->push_back(staff);
+                  cs->staves().insert(staffIdx + rstaff, staff);
                   }
+
+            // insert part
+            cs->insertPart(part, staffIdx);
+            int sidx = cs->staffIdx(part);
+            int eidx = sidx + part->nstaves();
+            for (MeasureBase* mb = cs->measures()->first(); mb; mb = mb->next()) {
+                  if (mb->type() != MEASURE)
+                        continue;
+                  Measure* m = (Measure*)mb;
+                  m->cmdAddStaves(sidx, eidx);
+                  }
+            staffIdx += rstaff;
             }
       //
       //    sort staves
@@ -412,38 +367,6 @@ void InstrumentWizard::createInstruments(Score* cs)
                   dst.push_back(sli->staff);
                   }
             }
-#if 0
-      QList<int> sl;
-      QList<int> dl;
-
-      for (int idx = 0; idx < cs->staves().size(); ++idx)
-            sl.push_back(idx);
-
-      for (QList<Staff*>::iterator i = dst.begin(); i != dst.end(); ++i) {
-            Staff* staff = *i;
-            int idx = cs->staves().indexOf(staff);
-            if (idx == -1)
-                  printf("staff in dialog(%p) not found in score\n", staff);
-            else
-                  dl.push_back(idx);
-            }
-
-      if (sl.size() != dl.size())
-            printf("cannot happen: sl(%d) != dl(%d)\n", sl.size(), dl.size());
-      bool sort = false;
-      QList<int>::iterator isl = sl.begin();
-      QList<int>::iterator dsl = dl.begin();
-      for (int i = 0; i < sl.size(); ++i, ++isl, ++dsl) {
-            if (*isl != *dsl) {
-                  sort = true;
-                  break;
-                  }
-            }
-      if (sort) {
-            cs->sortStaves(dl);
-            cs->undoOp(dl);
-            }
-#endif
       cs->setLayoutAll(true);
       }
 
@@ -476,6 +399,17 @@ void TimesigWizard::timesig(int* z, int* n) const
       *n = timesigN->value();
       }
 
+//---------------------------------------------------------
+//   pickupMeasure
+//---------------------------------------------------------
+
+bool TimesigWizard::pickup(int* z, int* n) const
+      {
+      *z = pickupTimesigZ->value();
+      *n = pickupTimesigN->value();
+      return pickupMeasure->isChecked();
+      }
+
 //---------------------------------------------------------
 //   TitleWizard
 //---------------------------------------------------------
@@ -514,6 +448,7 @@ NewWizardPage2::NewWizardPage2(QWidget* parent)
       setTitle(tr("Create New Score"));
       setSubTitle(tr("Define a set of instruments. Each instrument"
                      " is represented by one or more staves"));
+
       complete = false;
       w = new InstrumentWizard;
       QGridLayout* grid = new QGridLayout;
@@ -534,15 +469,6 @@ void NewWizardPage2::setComplete(bool val)
       emit completeChanged();
       }
 
-//---------------------------------------------------------
-//   isComplete
-//---------------------------------------------------------
-
-bool NewWizardPage2::isComplete() const
-      {
-      return complete;
-      }
-
 //---------------------------------------------------------
 //   NewWizardPage3
 //---------------------------------------------------------
@@ -569,6 +495,7 @@ NewWizardPage4::NewWizardPage4(QWidget* parent)
       setSubTitle(tr("Select Template File:"));
 
       QStringList nameFilter;
+      nameFilter.append("*.mscz");
       nameFilter.append("*.msc");
 
       model = new QDirModel;
@@ -583,9 +510,6 @@ NewWizardPage4::NewWizardPage4(QWidget* parent)
       tree->header()->hideSection(2);
       tree->header()->hideSection(3);
 
-      QString path(mscoreGlobalShare);
-      path += "/templates";
-      tree->setRootIndex(model->index(path));
       QGridLayout* grid = new QGridLayout;
       grid->addWidget(tree, 0, 0);
       setLayout(grid);
@@ -595,13 +519,80 @@ NewWizardPage4::NewWizardPage4(QWidget* parent)
          this, SLOT(templateChanged(const QItemSelection&, const QItemSelection&)));
       }
 
+//---------------------------------------------------------
+//   initializePage
+//---------------------------------------------------------
+
+void NewWizardPage4::initializePage()
+      {
+      model->refresh();
+      QString path(mscoreGlobalShare);
+      path += "/templates";
+      tree->setRootIndex(model->index(path));
+      }
+
+//---------------------------------------------------------
+//   NewWizardPage5
+//---------------------------------------------------------
+
+NewWizardPage5::NewWizardPage5(QWidget* parent)
+   : QWizardPage(parent)
+      {
+      setTitle(tr("Create New Score"));
+      setSubTitle(tr("Select Key Signature:"));
+
+      sp = new Palette;
+      sp->setMag(.8);
+      sp->setGrid(56, 45);
+      sp->setSelectable(true);
+
+      QVBoxLayout* layout = new QVBoxLayout;
+      layout->addWidget(sp);
+      setLayout(layout);
+
+      for (int i = 0; i < 7; ++i) {
+            KeySig* k = new KeySig(gscore);
+            k->setSubtype(i+1);
+            sp->append(k, tr(keyNames[i*2]));
+            }
+      for (int i = -7; i < 0; ++i) {
+            KeySig* k = new KeySig(gscore);
+            k->setSubtype(i & 0xff);
+            sp->append(k, tr(keyNames[(7 + i) * 2 + 1]));
+            }
+      KeySig* k = new KeySig(gscore);
+      k->setSubtype(0);
+      sp->append(k, keyNames[14]);
+      sp->setSelected(14);
+
+      sp->resizeWidth(300);
+      }
+
+//---------------------------------------------------------
+//   keysig
+//---------------------------------------------------------
+
+int NewWizardPage5::keysig() const
+      {
+      int idx = sp->getSelectedIdx();
+      Element* e = sp->element(idx);
+      return e->subtype();
+      }
+
 //---------------------------------------------------------
 //   isComplete
 //---------------------------------------------------------
 
 bool NewWizardPage4::isComplete() const
       {
-      return tree->selectionModel()->hasSelection();
+      QItemSelectionModel* sm = tree->selectionModel();
+      QModelIndexList l = sm->selectedRows();
+      bool hasSelection = false;
+      if (!l.isEmpty()) {
+            QModelIndex idx = l.front();
+            hasSelection = idx.isValid();
+            }
+      return hasSelection;
       }
 
 //---------------------------------------------------------
@@ -623,6 +614,8 @@ QString NewWizardPage4::templatePath() const
       if (useTemplate) {
             QItemSelectionModel* sm = tree->selectionModel();
             QModelIndexList l = sm->selectedRows();
+            if (l.isEmpty())
+                  return QString();
             QModelIndex idx = l.front();
             if (idx.isValid())
                   return model->filePath(idx);
@@ -645,12 +638,16 @@ NewWizard::NewWizard(QWidget* parent)
       p2 = new NewWizardPage2;
       p3 = new NewWizardPage3;
       p4 = new NewWizardPage4;
+      p5 = new NewWizardPage5;
 
       setPage(Page_Type, p1);
       setPage(Page_Instruments, p2);
       setPage(Page_Template, p4);
       setPage(Page_Timesig, p3);
+      setPage(Page_Keysig, p5);
       p3->setFinalPage(true);
+      p4->setFinalPage(false);
+      p5->setFinalPage(false);
       resize(700, 500);
       }
 
@@ -664,6 +661,8 @@ int NewWizard::nextId() const
             case Page_Type:
                   return useTemplate() ? Page_Template : Page_Instruments;
             case Page_Instruments:
+                  return Page_Keysig;
+            case Page_Keysig:
                   return Page_Timesig;
             case Page_Template:
                   return Page_Timesig;
