@@ -136,6 +136,7 @@ void PageSettings::setScore(Score* s)
       sl->setPageFormat(*pf);
       sl->setSpatium(ml->spatium());
 
+      pageGroup->clear();
       for (int i = 0; true; ++i) {
             if (paperSizes[i].name == 0)
                   break;
@@ -174,6 +175,8 @@ void PageSettings::setValues(ScoreLayout* lo)
       evenPageLeftMargin->blockSignals(true);
       evenPageRightMargin->blockSignals(true);
       spatiumEntry->blockSignals(true);
+      pageWidth->blockSignals(true);
+      pageHeight->blockSignals(true);
 
       const char* suffix = mm ? "mm" : "in";
       oddPageTopMargin->setSuffix(suffix);
@@ -191,8 +194,7 @@ void PageSettings::setValues(ScoreLayout* lo)
       PageFormat* pf = lo->pageFormat();
 
       QString s;
-      pageWidth->blockSignals(true);
-      pageHeight->blockSignals(true);
+
       if (mm) {
             oddPageTopMargin->setValue(pf->oddTopMargin * INCH);
             oddPageBottomMargin->setValue(pf->oddBottomMargin * INCH);
@@ -205,14 +207,8 @@ void PageSettings::setValues(ScoreLayout* lo)
             evenPageRightMargin->setValue(pf->evenRightMargin * INCH);
 
             spatiumEntry->setValue(lo->spatium()/DPMM);
-            if (pf->landscape) {
-                  pageWidth->setValue(pf->height() * INCH);
-                  pageHeight->setValue(pf->width() * INCH);
-                  }
-            else {
-                  pageWidth->setValue(pf->width() * INCH);
-                  pageHeight->setValue(pf->height()* INCH);
-                  }
+            pageHeight->setValue(pf->height() * INCH);
+            pageWidth->setValue(pf->width() * INCH);
             }
       else {
             oddPageTopMargin->setValue(pf->oddTopMargin);
@@ -226,18 +222,9 @@ void PageSettings::setValues(ScoreLayout* lo)
             evenPageRightMargin->setValue(pf->evenRightMargin);
 
             spatiumEntry->setValue(lo->spatium()/DPI);
-            if (pf->landscape) {
-                  pageWidth->setValue(pf->height());
-                  pageHeight->setValue(pf->width());
-                  }
-            else {
-                  pageWidth->setValue(pf->width());
-                  pageHeight->setValue(pf->height());
-                  }
+            pageWidth->setValue(pf->width());
+            pageHeight->setValue(pf->height());
             }
-      pageWidth->blockSignals(false);
-      pageHeight->blockSignals(false);
-
       evenPageTopMargin->setEnabled(pf->twosided);
       evenPageBottomMargin->setEnabled(pf->twosided);
       evenPageLeftMargin->setEnabled(pf->twosided);
@@ -246,6 +233,8 @@ void PageSettings::setValues(ScoreLayout* lo)
       landscape->setChecked(pf->landscape);
       twosided->setChecked(pf->twosided);
 
+      pageWidth->blockSignals(false);
+      pageHeight->blockSignals(false);
       oddPageTopMargin->blockSignals(false);
       oddPageBottomMargin->blockSignals(false);
       oddPageLeftMargin->blockSignals(false);
@@ -305,8 +294,8 @@ void PageSettings::twosidedToggled(bool flag)
 
 void PageSettings::apply()
       {
-      double f  = mmButton->isChecked() ? 1.0/INCH : 1.0;
-      double f1 = mmButton->isChecked() ? DPMM : DPI;
+      double f  = mmUnit ? 1.0/INCH : 1.0;
+      double f1 = mmUnit ? DPMM : DPI;
 
       cs->pageFormat()->size             = pageGroup->currentIndex();
       cs->pageFormat()->_width           = pageWidth->value() * f;
@@ -363,7 +352,7 @@ void PageSettings::pageFormatSelected(int pf)
 
 void PageSettings::otmChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->oddTopMargin = val;
       preview->layout();
@@ -375,7 +364,7 @@ void PageSettings::otmChanged(double val)
 
 void PageSettings::olmChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->oddLeftMargin = val;
       preview->layout();
@@ -387,7 +376,7 @@ void PageSettings::olmChanged(double val)
 
 void PageSettings::ormChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->oddRightMargin = val;
       preview->layout();
@@ -399,7 +388,7 @@ void PageSettings::ormChanged(double val)
 
 void PageSettings::obmChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->oddBottomMargin = val;
       preview->layout();
@@ -411,7 +400,7 @@ void PageSettings::obmChanged(double val)
 
 void PageSettings::etmChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->evenTopMargin = val;
       preview->layout();
@@ -423,7 +412,7 @@ void PageSettings::etmChanged(double val)
 
 void PageSettings::elmChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->evenLeftMargin = val;
       preview->layout();
@@ -435,7 +424,7 @@ void PageSettings::elmChanged(double val)
 
 void PageSettings::ermChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->evenRightMargin = val;
       preview->layout();
@@ -447,7 +436,7 @@ void PageSettings::ermChanged(double val)
 
 void PageSettings::ebmChanged(double val)
       {
-      if (mmButton->isChecked())
+      if (mmUnit)
             val /= INCH;
       preview->lo()->pageFormat()->evenBottomMargin = val;
       preview->layout();
@@ -459,8 +448,7 @@ void PageSettings::ebmChanged(double val)
 
 void PageSettings::spatiumChanged(double val)
       {
-      if (mmButton->isChecked())
-            val *= DPMM;
+      val *= mmUnit ? DPMM : DPI;
       preview->lo()->setSpatium(val);
       preview->layout();
       }
@@ -469,17 +457,36 @@ void PageSettings::spatiumChanged(double val)
 //   pageHeightChanged
 //---------------------------------------------------------
 
-void PageSettings::pageHeightChanged(double)
+void PageSettings::pageHeightChanged(double val)
       {
-      pageGroup->setCurrentIndex(paperSizeNameToIndex("Custom"));
+      double val2 = pageWidth->value();
+      if (mmUnit) {
+            val /= INCH;
+            val2 /= INCH;
+            }
+      int pf = paperSizeNameToIndex("Custom");
+      pageGroup->setCurrentIndex(pf);
+      preview->lo()->pageFormat()->size = pf;
+      preview->lo()->pageFormat()->_height = val;
+      preview->lo()->pageFormat()->_width = val2;
+      preview->layout();
       }
 
 //---------------------------------------------------------
 //   pageWidthChanged
 //---------------------------------------------------------
 
-void PageSettings::pageWidthChanged(double)
+void PageSettings::pageWidthChanged(double val)
       {
-      pageGroup->setCurrentIndex(paperSizeNameToIndex("Custom"));
+      double val2 = pageHeight->value();
+      if (mmUnit) {
+            val /= INCH;
+            val2 /= INCH;
+            }
+      int pf = paperSizeNameToIndex("Custom");
+      pageGroup->setCurrentIndex(pf);
+      preview->lo()->pageFormat()->size = pf;
+      preview->lo()->pageFormat()->_width = val;
+      preview->lo()->pageFormat()->_height = val2;
+      preview->layout();
       }
-
