@@ -28,46 +28,90 @@
 #include "pitchspelling.h"
 #include "staff.h"
 #include "chord.h"
-
-//    TPC - tonal pitch classes
-//    "line of fifth's" LOF
-
-static const int spellings[] = {
-//          bb  b   -   #  ##
-            0,  7, 14, 21, 28,  // C
-            2,  9, 16, 23, 30,  // D
-            4, 11, 18, 25, 32,  // E
-           -1,  6, 13, 20, 27,  // F
-            1,  8, 15, 22, 29,  // G
-            3, 10, 17, 24, 31,  // A
-            5, 12, 19, 26, 33,  // B
-            };
+#include "score.h"
 
 //---------------------------------------------------------
-//   line2tpc
-//    prefix -2 < 2
+//   step2tpc
+//    alter =  -2 <= 2   (bb b - # ##)
 //---------------------------------------------------------
 
-int line2tpc(int line, int prefix)
+int step2tpc(int step, int alter)
       {
-      int i = line * 5 + 2 + prefix;
+      //    TPC - tonal pitch classes
+      //    "line of fifth's" LOF
+
+      static const int spellings[] = {
+      //     bb  b   -   #  ##
+             0,  7, 14, 21, 28,  // C
+             2,  9, 16, 23, 30,  // D
+             4, 11, 18, 25, 32,  // E
+            -1,  6, 13, 20, 27,  // F
+             1,  8, 15, 22, 29,  // G
+             3, 10, 17, 24, 31,  // A
+             5, 12, 19, 26, 33,  // B
+             };
+
+      int i = step * 5 + 2 + alter;
       if (i < 0 || (i >= int(sizeof(spellings)/sizeof(*spellings))))
-            abort();
-      return spellings[line * 5 + 2 + prefix];
+            return INVALID_TPC;
+      return spellings[i];
       };
 
+//---------------------------------------------------------
+//   pitch2tpc
+//    preferred pitch spelling depending on key
+//    key -7 - +7
+//---------------------------------------------------------
+
+int pitch2tpc(int pitch, int key)
+      {
+      int step = pitch % 12;
+
+      // ======TODO: fill table with reasonable values ===========
+
+      static int ptab[15][12] = {
+//              c  c#   d  d#   e   f  f#   g  g#   a  a#   b
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // Ces
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // Ges
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // Des
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // As
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // Es
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // B
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // F
+
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // C
+
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // G
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // D
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // A
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // E
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // H
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // Fis
+            {  14, 21, 16, 23, 18, 13, 20, 15, 22, 17, 24, 19 },     // Cis
+            };
+      return ptab[key+7][step];
+      }
+
 //---------------------------------------------------------
 //   step2tpc
 //---------------------------------------------------------
 
-int step2tpc(const QString& step, int alter)
+int step2tpc(const QString& stepName, int alter)
       {
-      if (step.isEmpty())
+      if (stepName.isEmpty())
             return INVALID_TPC;
-      int r = step[0].toLower().toAscii() - 'c';
-      if (r < 0 || r > 6)
-            return INVALID_TPC;
-      return spellings[r * 5 + alter + 2];
+      char c = stepName[0].toLower().toAscii();
+      int r = -1;
+      switch (c) {
+            case 'c': r = 0; break;
+            case 'd': r = 1; break;
+            case 'e': r = 2; break;
+            case 'f': r = 3; break;
+            case 'g': r = 4; break;
+            case 'a': r = 5; break;
+            case 'b': r = 6; break;
+            }
+      return step2tpc(r, alter);
       }
 
 //---------------------------------------------------------
@@ -79,7 +123,7 @@ int tpc2pitch(int tpc)
       tpc += 1;
 
       static int pitches[] = {
-//line:     F   C   G   D   A   E   B
+//step:     F   C   G   D   A   E   B
             3, -2,  5,  0,  7,  2,  9,     // bb
             4, -1,  6,  1,  8,  3, 10,     // b
             5,  0,  7,  2,  9,  4, 11,     // -
@@ -94,13 +138,15 @@ int tpc2pitch(int tpc)
       }
 
 //---------------------------------------------------------
-//   tpc2line
+//   tpc2step
 //---------------------------------------------------------
 
-int tpc2line(int tpc)
+int tpc2step(int tpc)
       {
-      static const int lines[7] = { 3, 0, 4, 1, 5, 2, 6 };
-      return lines[(tpc+1) % 7];
+      // 14 - C
+      // 15 % 7 = 1
+      static const int steps[7] = { 3, 0, 4, 1, 5, 2, 6 };
+      return steps[(tpc+1) % 7];
       }
 
 //---------------------------------------------------------
@@ -110,7 +156,7 @@ int tpc2line(int tpc)
 int pitch2line(int pitch)
       {
       int tpc = pitch2tpc(pitch);
-      return tpc2line(tpc) + (pitch / 12) * 7;
+      return tpc2step(tpc) + (pitch / 12) * 7;
       }
 
 //---------------------------------------------------------
@@ -448,8 +494,10 @@ int computeWindow(const QList<Note*>& notes, int start, int end)
             int tick = notes[i]->chord()->tick();
             key[k]   = notes[i]->staff()->keymap()->key(tick) + 7;
             if (key[k] < 0 || key[k] > 14) {
-                  printf("illegal key at tick %d: %d\n", tick, key[k] - 7);
-                  abort();
+                  printf("illegal key at tick %d: %d, window %d-%d\n",
+                     tick, key[k] - 7, start, end);
+                  return 0;
+                  // abort();
                   }
             ++k;
             ++i;
@@ -570,7 +618,7 @@ void spell(QList<NoteEvent*>& notes, int key)
 //   spell
 //---------------------------------------------------------
 
-void spell(QList<Note*>& notes)
+void Score::spellNotelist(QList<Note*>& notes)
       {
       int n = notes.size();
 
@@ -589,16 +637,16 @@ void spell(QList<Note*>& notes)
                   tab = tab1;
 
             if (start == 0) {
-                  notes[0]->setTpc(tab[(notes[0]->pitch() % 12) * 2 + (opt & 1)]);
+                  undoChangeTpc(notes[0], tab[(notes[0]->pitch() % 12) * 2 + (opt & 1)]);
                   if (n > 1)
-                        notes[1]->setTpc(tab[(notes[1]->pitch() % 12) * 2 + ((opt & 2)>>1)]);
+                        undoChangeTpc(notes[1], tab[(notes[1]->pitch() % 12) * 2 + ((opt & 2)>>1)]);
                   if (n > 2)
-                        notes[2]->setTpc(tab[(notes[2]->pitch() % 12) * 2 + ((opt & 4)>>2)]);
+                        undoChangeTpc(notes[2], tab[(notes[2]->pitch() % 12) * 2 + ((opt & 4)>>2)]);
                   }
             if ((end - start) >= 6) {
-                  notes[start+3]->setTpc(tab[(notes[start+3]->pitch() % 12) * 2 + ((opt &  8) >> 3)]);
-                  notes[start+4]->setTpc(tab[(notes[start+4]->pitch() % 12) * 2 + ((opt & 16) >> 4)]);
-                  notes[start+5]->setTpc(tab[(notes[start+5]->pitch() % 12) * 2 + ((opt & 32) >> 5)]);
+                  undoChangeTpc(notes[start+3], tab[(notes[start+3]->pitch() % 12) * 2 + ((opt &  8) >> 3)]);
+                  undoChangeTpc(notes[start+4], tab[(notes[start+4]->pitch() % 12) * 2 + ((opt & 16) >> 4)]);
+                  undoChangeTpc(notes[start+5], tab[(notes[start+5]->pitch() % 12) * 2 + ((opt & 32) >> 5)]);
                   }
             if (end == n) {
                   int n = end - start;
@@ -606,13 +654,13 @@ void spell(QList<Note*>& notes)
                   switch(n - 6) {
                         case 3:
                               k = end - start - 3;
-                              notes[end-3]->setTpc(tab[(notes[end-3]->pitch() % 12) * 2 + ((opt & (1<<k)) >> k)]);
+                              undoChangeTpc(notes[end-3], tab[(notes[end-3]->pitch() % 12) * 2 + ((opt & (1<<k)) >> k)]);
                         case 2:
                               k = end - start - 2;
-                              notes[end-2]->setTpc(tab[(notes[end-2]->pitch() % 12) * 2 + ((opt & (1<<k)) >> k)]);
+                              undoChangeTpc(notes[end-2], tab[(notes[end-2]->pitch() % 12) * 2 + ((opt & (1<<k)) >> k)]);
                         case 1:
                               k = end - start - 1;
-                              notes[end-1]->setTpc(tab[(notes[end-1]->pitch() % 12) * 2 + ((opt & (1<<k)) >> k)]);
+                              undoChangeTpc(notes[end-1], tab[(notes[end-1]->pitch() % 12) * 2 + ((opt & (1<<k)) >> k)]);
                         }
                   break;
                   }
@@ -621,3 +669,23 @@ void spell(QList<Note*>& notes)
             }
       }
 
+//---------------------------------------------------------
+//   transposeTpc
+//---------------------------------------------------------
+
+int transposeTpc(int tpc, int semitones)
+      {
+      if (semitones == 0)
+            return tpc;
+      if (tpc == INVALID_TPC)
+            return INVALID_TPC;
+      if (semitones < 0)
+            semitones = 12 + semitones;
+      else
+            semitones = (semitones % 12);
+      if (semitones == 0)
+            return tpc;
+      int pitch = tpc2pitch(tpc) + semitones;
+      return pitch2tpc(pitch);
+      }
+
