@@ -72,9 +72,6 @@ Seq::Seq()
       heartBeatTimer = new QTimer(this);
       connect(heartBeatTimer, SIGNAL(timeout()), this, SLOT(heartBeat()));
       heartBeatTimer->stop();
-      playTimer = new QTimer(this);
-      playTimer->setSingleShot(true);
-      connect(playTimer, SIGNAL(timeout()), this, SLOT(stopNotes()));
       connect(this, SIGNAL(toGui(int)), this, SLOT(seqMessage(int)), Qt::QueuedConnection);
       }
 
@@ -113,12 +110,16 @@ void Seq::setScore(Score* s)
 
 void Seq::selectionChanged(int mode)
       {
-      int tick = cs->pos();
-      if (mode != SEL_SINGLE || state == STOP || cs == 0 || driver == 0) {
-            cs->setPlayPos(tick);
+      if (cs == 0 || driver == 0)
             return;
-            }
-      if (tick != -1)
+
+      int tick = cs->pos();
+      if (tick == -1)
+            return;
+
+      if (mode != SEL_SINGLE || state == STOP)
+            cs->setPlayPos(tick);
+      else
             seek(tick);
       }
 
@@ -187,7 +188,12 @@ bool Seq::init()
             }
 #endif
       if (driver == 0) {
-            printf("no audio driver\n");
+#if 0
+            QString s = QString("Init audio driver failed.\n"
+                                "Sequencer will be disabled.");
+            QMessageBox::critical(0, "MuseScore: Init Audio Driver", s);
+#endif
+            printf("init audio driver failed\n");
             return false;
             }
       if (!driver->start()) {
@@ -295,6 +301,11 @@ void Seq::start()
       else {
             if (events.empty() || cs->playlistDirty() || playlistChanged)
                   collectEvents();
+
+            if (events.empty() || endTick == 0) {
+                  a->setChecked(false);
+                  return;
+                  }
             seek(cs->playPos());
             if (!pauseState)
                   driver->startTransport();
@@ -341,8 +352,9 @@ void Seq::pause()
 
 void MuseScore::seqStarted()
       {
-      setState(STATE_PLAY);
-      cs->start();
+      if (cs->state() != STATE_PLAY)  // don't get stuck in play mode
+           cs->setPrevState(cs->state());
+      cs->setState(STATE_PLAY);
       foreach(Viewer* v, cs->getViewer())
             v->setCursorOn(true);
       cs->end();
@@ -356,10 +368,14 @@ void MuseScore::seqStarted()
 
 void MuseScore::seqStopped()
       {
-      setState(STATE_NORMAL);
+      cs->setState(cs->prevState());
+      // TODO: there should really be some sort of signal to the viewers
+      // instead of the state change being handled here
+      bool cursorOn = false;
+      if (cs->state() == STATE_NOTE_ENTRY)
+            cursorOn = true;
       foreach(Viewer* v, cs->getViewer())
-            v->setCursorOn(false);
-      cs->start();
+            v->setCursorOn(cursorOn);
       cs->setLayoutAll(false);
       cs->setUpdateAll();
       cs->end();
@@ -379,7 +395,6 @@ void Seq::guiStop()
       //
       // deselect all selected notes
       //
-      cs->start();
       foreach(const NoteOn* n, markedNotes) {
             n->note()->setSelected(false);
             cs->addRefresh(n->note()->abbox());
@@ -428,10 +443,10 @@ void Seq::stopTransport()
       foreach(const Event* e, activeNotes) {
             if (e->type() != ME_NOTEON)
                   continue;
-            NoteOn* no = (NoteOn*)e;
+            const NoteOn* no = static_cast<const NoteOn*>(e);
             MidiOutEvent e;
-            e.port = no->port();
-            e.type = ME_NOTEON | no->channel();
+            e.port = cs->midiPort(no->channel());
+            e.type = ME_NOTEON | cs->midiChannel(no->channel());
             e.a    = no->pitch();
             e.b    = 0;
             driver->putEvent(e);
@@ -449,10 +464,6 @@ void Seq::stopTransport()
 
 void Seq::startTransport()
       {
-      // dont start transport, if we have nothing to play
-      //
-      if (endTick == 0)
-            return;
       if (!pauseState)
             emit toGui('1');
       startTime = curTime() - playTime;
@@ -469,18 +480,20 @@ void Seq::playEvent(const Event* event)
       int type = event->type();
       if (type == ME_NOTEON) {
             NoteOn* n = (NoteOn*) event;
+
             int channel = n->channel();
             MidiOutEvent e;
-            e.port = n->port();
-            e.type = ME_NOTEON | channel;
+            e.port = cs->midiPort(channel);
+            e.type = ME_NOTEON | cs->midiChannel(channel);
             e.a    = n->pitch();
             e.b    = n->velo();
 
             bool mute;
             Note* note = n->note();
             if (note) {
-                  Instrument* instr = n->note()->staff()->part()->instrument();
-                  mute = instr->mute || instr->soloMute;
+                  Instrument* instr = note->staff()->part()->instrument();
+                  Channel* a = instr->channel[note->subchannel()];
+                  mute = a->mute || a->soloMute;
                   }
             else {
                   mute = false;
@@ -504,40 +517,11 @@ void Seq::playEvent(const Event* event)
                   }
             }
       else if (type == ME_CONTROLLER)  {
-            ControllerEvent* c = (ControllerEvent*)event;
-            if (c->controller() == CTRL_PROGRAM) {
-                  int hb = (c->value() >> 16) & 0xff;
-                  int lb = (c->value() >> 8) & 0xff;
-                  int pr = c->value() & 0xff;
-                  if (hb != 0xff) {
-                        MidiOutEvent e;
-                        e.port = c->port();
-                        e.type = ME_CONTROLLER | c->channel();
-                        e.a    = CTRL_HBANK;
-                        e.b    = hb;
+            const ControllerEvent* c = static_cast<const ControllerEvent*>(event);
+            QList<MidiOutEvent> ol;
+            if (c->midiOutEvent(&ol, cs)) {
+                  foreach(const MidiOutEvent& e, ol)
                         driver->putEvent(e);
-                        }
-                  if (lb != 0xff) {
-                        MidiOutEvent e;
-                        e.port = c->port();
-                        e.type = ME_CONTROLLER | c->channel();
-                        e.a    = CTRL_LBANK;
-                        e.b    = lb;
-                        driver->putEvent(e);
-                        }
-                  MidiOutEvent e;
-                  e.port = c->port();
-                  e.type = ME_PROGRAM | c->channel();
-                  e.a    = pr;
-                  driver->putEvent(e);
-                  }
-            else {
-                  MidiOutEvent e;
-                  e.port = c->port();
-                  e.type = ME_CONTROLLER | c->channel();
-                  e.a    = c->controller();
-                  e.b    = c->value();
-                  driver->putEvent(e);
                   }
             }
       else {
@@ -551,8 +535,9 @@ void Seq::playEvent(const Event* event)
 
 void Seq::processMessages()
       {
-      mutex.lock();
-      while (!toSeq.isEmpty()) {
+      for (;;) {
+            if (toSeq.isEmpty())
+                  break;
             SeqMsg msg = toSeq.dequeue();
             switch(msg.id) {
                   case SEQ_TEMPO_CHANGE:
@@ -571,8 +556,6 @@ void Seq::processMessages()
                         break;
                   }
             }
-      wait.wakeAll();
-      mutex.unlock();
       }
 
 //---------------------------------------------------------
@@ -607,8 +590,10 @@ void Seq::processMidi()
                         break;
                   playEvent(playPos.value());
                   }
-            if (playPos == events.constEnd())
+            if (playPos == events.constEnd()) {
                   driver->stopTransport();
+                  rewindStart();
+                  }
             }
       }
 
@@ -663,6 +648,7 @@ void Seq::process(unsigned n, float* lbuffer, float* rbuffer, int stride)
                   }
             if (playPos == events.constEnd()) {
                   driver->stopTransport();
+                  rewindStart();
                   }
             }
       else
@@ -677,6 +663,32 @@ void Seq::process(unsigned n, float* lbuffer, float* rbuffer, int stride)
             }
       }
 
+//---------------------------------------------------------
+//   initInstruments
+//---------------------------------------------------------
+
+void Seq::initInstruments()
+      {
+      foreach(Part* part, *cs->parts()) {
+            const Instrument* instr = part->instrument();
+
+            foreach(const Channel* a, instr->channel) {
+                  foreach(Event* e, a->init) {
+                        if (e == 0)
+                              continue;
+                        e->setChannel(a->channel);
+                        QList<MidiOutEvent> el;
+                        if (e->midiOutEvent(&el, cs)) {
+                              foreach(const MidiOutEvent& event, el)
+                                    sendEvent(event);
+                              }
+                        else
+                              printf("unknown event\n");
+                        }
+                  }
+            }
+      }
+
 //---------------------------------------------------------
 //   collectEvents
 //---------------------------------------------------------
@@ -688,62 +700,17 @@ void Seq::collectEvents()
 
       events.clear();
 
-      foreach(Part* part, *cs->parts()) {
-            int channel = part->midiChannel();
-            int port    = part->midiPort();
-            const Instrument* instr = part->instrument();
-
-            MidiOutEvent event;
-            event.port = port;
-
-            event.type = ME_CONTROLLER | channel;
-            if (instr->midiBankSelectH != -1) {
-                  event.a    = CTRL_HBANK;
-                  event.b    = instr->midiBankSelectH;
-                  sendEvent(event);
-                  }
-            if (instr->midiBankSelectL != -1) {
-                  event.a    = CTRL_LBANK;
-                  event.b    = instr->midiBankSelectL;
-                  sendEvent(event);
-                  }
-            if (instr->midiProgram != -1) {
-                  event.type = ME_PROGRAM | channel;
-                  event.a    = instr->midiProgram;
-                  sendEvent(event);
-                  }
-
-            event.type = ME_CONTROLLER | channel;
-            event.a    = CTRL_VOLUME;
-            event.b    = instr->volume;
-            sendEvent(event);
-
-            event.a    = CTRL_REVERB_SEND;
-            event.b    = instr->reverb;
-            sendEvent(event);
-
-            event.a    = CTRL_CHORUS_SEND;
-            event.b    = instr->chorus;
-            sendEvent(event);
-
-            event.a    = CTRL_PANPOT;
-            event.b    = instr->pan;
-            sendEvent(event);
-            }
-
       cs->toEList(&events, 0);
+      endTick = 0;
+      if (!events.empty()) {
+            EventMap::const_iterator e = events.constEnd();
+            --e;
+            endTick = e.key();
+            }
 
       PlayPanel* pp = mscore->getPlayPanel();
-      if (pp) {
-            if (events.empty())
-                  pp->setEndpos(0);
-            else {
-                  EventMap::const_iterator e = events.constEnd();
-                  --e;
-                  endTick = e.key();
-                  pp->setEndpos(endTick);
-                  }
-            }
+      if (pp)
+            pp->setEndpos(endTick);
       }
 
 //---------------------------------------------------------
@@ -756,10 +723,10 @@ void Seq::heartBeat()
       if (state != PLAY)
             return;
 
-      cs->start();
-      cs->setLayoutAll(false);
-
+      PlayPanel* pp = mscore->getPlayPanel();
       double endTime = curTime() - startTime;
+      if (pp)
+            pp->heartBeat2(lrint(endTime));
       Note* note = 0;
       for (; guiPos != events.constEnd(); ++guiPos) {
             double f = tick2time(guiPos.key());
@@ -781,9 +748,10 @@ void Seq::heartBeat()
       if (note) {
             foreach(Viewer* v, cs->getViewer())
                   v->moveCursor(note->chord()->segment());
-            PlayPanel* pp = mscore->getPlayPanel();
+            cs->adjustCanvasPosition(note, true);
             if (pp)
                   pp->heartBeat(note->chord()->tick(), guiPos.key());
+            mscore->setPos(note->chord()->tick());
             }
       cs->end();
       }
@@ -805,7 +773,7 @@ void Seq::setRelTempo(int relTempo)
       {
       SeqMsg msg;
       msg.data = relTempo;
-      msg.id    = SEQ_TEMPO_CHANGE;
+      msg.id   = SEQ_TEMPO_CHANGE;
       guiToSeq(msg);
 
       double tempo = cs->tempomap->tempo(playPos.key()) * relTempo * 0.01;
@@ -848,7 +816,6 @@ void Seq::setPos(int tick)
 
 void Seq::seek(int tick)
       {
-      cs->start();
       Segment* seg = cs->tick2segment(tick);
       if (seg) {
             foreach(Viewer* v, cs->getViewer())
@@ -861,24 +828,28 @@ void Seq::seek(int tick)
       msg.data = tick;
       msg.id   = SEQ_SEEK;
       guiToSeq(msg);
+      mscore->setPos(tick);
       }
 
 //---------------------------------------------------------
 //   startNote
 //---------------------------------------------------------
 
-void Seq::startNote(Part* part, int pitch, int velo)
+void Seq::startNote(Channel* a, int pitch, int velo)
       {
       if (state != STOP)
             return;
 
       bool active = false;
+      int port    = cs->midiPort(a->channel);
+      int channel = cs->midiChannel(a->channel);
+
       foreach(const Event* event, eventList) {
             NoteOn* n = (NoteOn*)event;
-            if (n->port() == part->midiPort() && n->channel() == part->midiChannel() && n->pitch() == pitch) {
+            if (n->channel() == a->channel && n->pitch() == pitch) {
                   MidiOutEvent ev;
-                  ev.port = n->port();
-                  ev.type = ME_NOTEON | n->channel();
+                  ev.port = port;
+                  ev.type = ME_NOTEON | channel;
                   ev.a    = n->pitch();
                   ev.b    = 0;
                   sendEvent(ev);
@@ -888,25 +859,25 @@ void Seq::startNote(Part* part, int pitch, int velo)
             }
 
       MidiOutEvent ev;
-      ev.port = part->midiPort();
-      ev.type = ME_NOTEON | part->midiChannel();
-      ev.a    = pitch + part->pitchOffset();
+      ev.port = port;
+      ev.type = ME_NOTEON | channel;
+      ev.a    = pitch;
       ev.b    = velo;
       sendEvent(ev);
 
       if (!active) {
             NoteOn* e = new NoteOn;
-            e->setChannel(part->midiChannel());
+            e->setChannel(a->channel);
             e->setPitch(pitch);
             e->setVelo(velo);
             eventList.append(e);
             }
       }
 
-void Seq::startNote(Part* part, int pitch, int velo, int duration)
+void Seq::startNote(Channel* a, int pitch, int velo, int duration)
       {
-      startNote(part, pitch, velo);
-      playTimer->start(duration);
+      startNote(a, pitch, velo);
+      QTimer::singleShot(duration, this, SLOT(stopNotes()));
       }
 
 //---------------------------------------------------------
@@ -918,9 +889,11 @@ void Seq::stopNotes()
       {
       foreach(const Event* event, eventList) {
             NoteOn* n = (NoteOn*)event;
+            int port    = cs->midiPort(n->channel());
+            int channel = cs->midiChannel(n->channel());
             MidiOutEvent ev;
-            ev.port = n->port();
-            ev.type = ME_NOTEON | n->channel();
+            ev.port = port;
+            ev.type = ME_NOTEON | channel;
             ev.a    = n->pitch();
             ev.b    = 0;
             sendEvent(ev);
@@ -933,11 +906,11 @@ void Seq::stopNotes()
 //   setController
 //---------------------------------------------------------
 
-void Seq::setController(int port, int channel, int ctrl, int data)
+void Seq::setController(int idx, int ctrl, int data)
       {
       MidiOutEvent event;
-      event.port = port;
-      event.type = ME_CONTROLLER | channel;
+      event.port = cs->midiPort(idx);
+      event.type = ME_CONTROLLER | cs->midiChannel(idx);
       event.a    = ctrl;
       event.b    = data;
       sendEvent(event);
@@ -1074,22 +1047,19 @@ void Seq::seekEnd()
 
 void Seq::guiToSeq(const SeqMsg& msg)
       {
-      if (!driver)
+      if (!driver || !running)
             return;
-      mutex.lock();
       toSeq.enqueue(msg);
-      wait.wait(&mutex);
-      mutex.unlock();
       }
 
 //---------------------------------------------------------
 //   getPatchInfo
 //---------------------------------------------------------
 
-const MidiPatch* Seq::getPatchInfo(int port, int ch, const MidiPatch* p)
+const MidiPatch* Seq::getPatchInfo(bool onlyDrums, const MidiPatch* p)
       {
       if (driver)
-            return driver->getPatchInfo(port, ch, p);
+            return driver->getPatchInfo(onlyDrums, p);
       return 0;
       }
 
@@ -1099,10 +1069,42 @@ const MidiPatch* Seq::getPatchInfo(int port, int ch, const MidiPatch* p)
 
 void Seq::midiInputReady()
       {
-      if (driver) {
-            cs->startCmd();
+      if (driver && cs) {
             driver->midiRead();
-            cs->endCmd();
             }
       }
 
+//---------------------------------------------------------
+//   SeqMsgFifo
+//---------------------------------------------------------
+
+SeqMsgFifo::SeqMsgFifo()
+      {
+      maxCount = SEQ_MSG_FIFO_SIZE;
+      clear();
+      }
+
+//---------------------------------------------------------
+//   enqueue
+//---------------------------------------------------------
+
+void SeqMsgFifo::enqueue(const SeqMsg& msg)
+      {
+      if (isFull()) {
+            printf("SeqMsgFifo: overflow\n");
+            return;
+            }
+      messages[widx] = msg;
+      push();
+      }
+
+//---------------------------------------------------------
+//   dequeue
+//---------------------------------------------------------
+
+SeqMsg SeqMsgFifo::dequeue()
+      {
+      SeqMsg msg = messages[ridx];
+      pop();
+      return msg;
+      }
