@@ -33,6 +33,7 @@
 #include "viewer.h"
 #include "navigate.h"
 #include "canvas.h"
+#include "articulation.h"
 
 //---------------------------------------------------------
 //   SlurSegment
@@ -109,7 +110,7 @@ void SlurSegment::updateGrips(int* n, QRectF* r) const
 //   startEdit
 //---------------------------------------------------------
 
-bool SlurSegment::startEdit(const QPointF&)
+bool SlurSegment::startEdit(Viewer*, const QPointF&)
       {
       return true;
       }
@@ -119,17 +120,18 @@ bool SlurSegment::startEdit(const QPointF&)
 //    return true if event is accepted
 //---------------------------------------------------------
 
-bool SlurSegment::edit(int curGrip, QKeyEvent* ev)
+bool SlurSegment::edit(Viewer*, int curGrip, int key, Qt::KeyboardModifiers modifiers, const QString& s)
       {
       if (slurTie()->type() != SLUR)
             return false;
-      Slur* sl = (Slur*) slurTie();
+      Slur* sl = static_cast<Slur*>(slurTie());
 
-      if (ev->key() == Qt::Key_X) {
+      if (key == Qt::Key_X) {
             sl->setSlurDirection(sl->isUp() ? DOWN : UP);
+            sl->layout(score()->layout());
             return true;
             }
-      if (!((ev->modifiers() & Qt::ShiftModifier)
+      if (!((modifiers & Qt::ShiftModifier)
          && ((_segmentType == SEGMENT_SINGLE)
               || (_segmentType == SEGMENT_BEGIN && curGrip == 0)
               || (_segmentType == SEGMENT_END && curGrip == 3)
@@ -141,9 +143,9 @@ bool SlurSegment::edit(int curGrip, QKeyEvent* ev)
       Element* e    = curGrip == 0 ? sl->startElement() : sl->endElement();
       Element* e1   = curGrip == 0 ? sl->endElement() : sl->startElement();
 
-      if (ev->key() == Qt::Key_Left)
+      if (key == Qt::Key_Left)
             cr = prevChordRest((ChordRest*)e);
-      else if (ev->key() == Qt::Key_Right)
+      else if (key == Qt::Key_Right)
             cr = nextChordRest((ChordRest*)e);
 
       if (cr == 0 || cr == (ChordRest*)e1)
@@ -478,32 +480,104 @@ QPointF SlurTie::slurPos(Element* e, System*& s)
       {
       ChordRest* cr;
       if (e->type() == NOTE)
-            cr = ((Note*)e)->chord();
+            cr = static_cast<Note*>(e)->chord();
       else
-            cr = (Chord*)e;
+            cr = static_cast<ChordRest*>(e);
       s = cr->measure()->system();
 
       //-----------------------------------------
       //    off
       //-----------------------------------------
 
-      qreal w  = cr->width() / 2.0;
+      qreal xo = cr->width() * .5;
       qreal yo = 0.0;
       if (cr->type() == CHORD) {
-            Chord* c = (Chord*)cr;
+            Chord* c = static_cast<Chord*>(cr);
             Stem* stem = c->stem();
+            Beam* beam = c->beam();
+
+            Chord* sc;
+            if (startElement()->type() == NOTE)
+                  sc = ((Note*)startElement())->chord();
+            else
+                  sc = (Chord*)startElement();
+
+            bool startIsGrace = false;
+            if (sc->type() == CHORD && sc->noteType() != NOTE_NORMAL)
+                  startIsGrace = true;
+            bool mainNoteOfGraceSlur = false;
+            if (startIsGrace && c == endElement() && c->noteType() == NOTE_NORMAL)
+                  mainNoteOfGraceSlur = true;
+
             if (up) {
-                  yo = c->upNote()->pos().y() - _spatium;
-                  if (c->isUp() && stem)
-                        yo = c->downNote()->pos().y() - stem->height() - _spatium;
+                  yo = c->upNote()->pos().y() - c->upNote()->headHeight();
+                  //
+                  // handle special case of tenuto and staccato;
+                  // should be generalized
+                  //
+                  QList<Articulation*>* al = c->getArticulations();
+                  if (al->size() == 1) {
+                        Articulation* a = al->at(0);
+                        if (a->subtype() == TenutoSym || a->subtype() == StaccatoSym) {
+                              yo = a->y() - _spatium * .5;
+                              }
+                        }
+                  if (c->isUp() && stem) {
+                        if (beam && !mainNoteOfGraceSlur)
+                              yo = c->downNote()->pos().y() - stem->height() - _spatium;
+                        else if (!startIsGrace) {
+                              // slurs on the stem side of stemmed notes start half
+                              // of a staff space from the end of the stem
+                              yo = c->downNote()->pos().y() - stem->height() + _spatium * .5;
+                              }
+                        }
                   }
             else {
-                  yo = c->downNote()->pos().y() + _spatium;
-                  if (!c->isUp() && stem)
-                        yo = c->upNote()->pos().y() + stem->height() + _spatium;
+                  yo = c->downNote()->pos().y() + c->downNote()->headHeight();
+                  //
+                  // handle special case of tenuto and staccato;
+                  // should be generalized
+                  //
+                  QList<Articulation*>* al = c->getArticulations();
+                  if (al->size() == 1) {
+                        Articulation* a = al->at(0);
+                        if (a->subtype() == TenutoSym || a->subtype() == StaccatoSym) {
+                              yo = a->y() + a->height() + _spatium * .5;
+                              }
+                        }
+                  if (!c->isUp() && stem) {
+                        if (beam && !mainNoteOfGraceSlur)
+                              yo = c->upNote()->pos().y() + stem->height() + _spatium;
+                        else if (!startIsGrace) {
+                              // slurs on the stem side of stemmed notes start half
+                              // of a staff space from the end of the stem
+                              yo = c->upNote()->pos().y() + stem->height() - _spatium * .5;
+                              }
+                        }
+                  }
+            if (up == c->isUp() && stem) {
+                  if (beam && !mainNoteOfGraceSlur) {
+                        // for beamed notes and slurs on stem side, slurs are aligned
+                        // to the stem rather than the middle of the notehead
+                        xo = stem->pos().x();
+                        if (cr == startElement())
+                              xo += stem->width() * .5;
+                        else
+                              xo -= stem->width() * .5;
+                        }
+                  else {
+                        if (cr == startElement()) {
+                              // don't collide with start stem
+                              xo = stem->pos().x() + _spatium * .5;
+                              }
+                        if (cr == endElement()) {
+                              // don't collide with end stem
+                              xo = stem->pos().x() - _spatium * .5;
+                              }
+                        }
                   }
             }
-      QPointF off(w, yo);
+      QPointF off(xo, yo);
 
       //-----------------------------------------
       //    p
@@ -568,6 +642,23 @@ void SlurTie::collectElements(QList<const Element*>& el) const
             el.append(seg);
       }
 
+//---------------------------------------------------------
+//   resetUserOffsets
+//---------------------------------------------------------
+
+void SlurTie::resetUserOffsets()
+      {
+      setUserOff(QPointF());
+      }
+
+void SlurSegment::resetUserOffsets()
+      {
+      setUserOff(QPointF());
+      for (int i = 0; i < 4; ++i)
+            ups[i].off = QPointF();
+      parent()->resetUserOffsets();
+      }
+
 //---------------------------------------------------------
 //   Slur
 //---------------------------------------------------------
@@ -620,26 +711,6 @@ void Slur::setEnd(int t, int track)
       _track2 = track;
       }
 
-#if 0
-//---------------------------------------------------------
-//   startsAt
-//---------------------------------------------------------
-
-bool Slur::startsAt(int tk, int tr)
-      {
-      return ((tick() == tk) && (track() == tr));
-      }
-
-//---------------------------------------------------------
-//   endsAt
-//---------------------------------------------------------
-
-bool Slur::endsAt(int t, int track)
-      {
-      return ((_tick2 == t) && (_track2 == track));
-      }
-#endif
-
 //---------------------------------------------------------
 //   write
 //---------------------------------------------------------
@@ -680,6 +751,50 @@ void Slur::read(QDomElement e)
             }
       }
 
+//---------------------------------------------------------
+//   chordsHaveTie
+//---------------------------------------------------------
+static bool chordsHaveTie (Chord* c1, Chord* c2)
+      {
+      NoteList* nl1 = c1->noteList();
+      NoteList* nl2 = c2->noteList();
+      for (iNote in1 = nl1->begin(); in1 != nl1->end(); ++in1) {
+            for (iNote in2 = nl2->begin(); in2 != nl2->end(); ++in2) {
+                  if (in1->second->tieFor() && in1->second->tieFor() == in2->second->tieBack())
+                        return true;
+                  }
+            }
+      return false;
+      }
+
+//---------------------------------------------------------
+//   directionMixture
+//---------------------------------------------------------
+
+static bool isDirectionMixture (Chord* c1, Chord* c2)
+      {
+      bool up = c1->isUp();
+      for (Segment* seg = c1->segment(); seg; seg = seg->next()) {
+            if (seg->subtype() == Segment::SegChordRest) {
+                  Element* e = seg->element(c1->track());
+                  if (!e)
+                        continue;
+                  Chord* c = 0;
+                  if (e->type() == NOTE)
+                        c = static_cast<Note*>(e)->chord();
+                  else if (e->type() == CHORD)
+                        c = static_cast<Chord*>(e);
+                  else
+                        continue;
+                  if (c && c->isUp() != up)
+                        return true;
+                  }
+            if (seg == c2->segment())
+                  break;
+            }
+      return false;
+      }
+
 //---------------------------------------------------------
 //   layout
 //---------------------------------------------------------
@@ -695,12 +810,39 @@ void Slur::layout(ScoreLayout* layout)
                   ChordRest* cr1 = (ChordRest*)startElement();
                   ChordRest* cr2 = (ChordRest*)endElement();
                   Measure* m1    = cr1->measure();
+
+                  Chord* c1;
+                  if (cr1->type() == NOTE)
+                        c1 = ((Note*)cr1)->chord();
+                  else
+                        c1 = (Chord*)cr1;
+                  Chord* c2;
+                  if (cr2->type() == NOTE)
+                        c2 = ((Note*)cr2)->chord();
+                  else
+                        c2 = (Chord*)cr2;
+
+                  up = !(cr1->isUp());
+
                   if ((cr2->tick() - cr1->tick()) > m1->tickLen()) {
                         // long slurs are always above
                         up = true;
                         }
-                  else
-                        up = !(cr1->isUp());
+                  if (cr1->type() == CHORD && cr2->type() == CHORD) {
+                        if (isDirectionMixture(c1, c2) && c1->noteType() == NOTE_NORMAL) {
+                              // slurs go above if start and end note have different stem directions,
+                              // but grace notes are exceptions
+                              up = true;
+                              }
+                        else if (m1->mstaff(cr1->staffIdx())->hasVoices && c1->noteType() == NOTE_NORMAL) {
+                              // in polyphonic passage, slurs go on the stem side
+                              up = cr1->isUp();
+                              }
+                        else if (chordsHaveTie(c1, c2)) {
+                              // could confuse slur with tie, put slur on stem side
+                              up = cr1->isUp();
+                              }
+                        }
                   }
                   break;
             }
@@ -857,7 +999,6 @@ void Tie::setStartNote(Note* note)
 void Tie::write(Xml& xml) const
       {
       xml.stag("Tie");
-      Element::writeProperties(xml);
       SlurTie::writeProperties(xml);
       xml.etag();
       }
@@ -902,14 +1043,20 @@ void Tie::layout(ScoreLayout* layout)
       System* s2  = m2->system();
 
       if (_slurDirection == AUTO)
-            up = !c1->isUp();
+            if (m1->mstaff(c1->staffIdx())->hasVoices) {
+                  // in polyphonic passage, ties go on the stem side
+                  up = c1->isUp();
+                  }
+            else
+                  up = !(c1->isUp());
       else
             up = _slurDirection == UP ? true : false;
       qreal w   = startNote()->headWidth();
+      qreal xo1 = w * 1.12;
       qreal h   = w * 0.3;
       qreal yo  = up ? -h : h;
 
-      QPointF off1(w, yo);
+      QPointF off1(xo1, yo);
       QPointF off2(0.0, yo);
 
       QPointF ppos(canvasPos());
@@ -929,6 +1076,7 @@ void Tie::layout(ScoreLayout* layout)
       unsigned nsegs   = sysIdx2 - sysIdx1 + 1;
       unsigned onsegs  = segments.size();
 
+
       if (nsegs != onsegs) {
             if (nsegs > onsegs) {
                   int n = nsegs - onsegs;
@@ -971,7 +1119,11 @@ void Tie::layout(ScoreLayout* layout)
             // case 3: middle segment
             else if (i != 0 && system != s2) {
                   // cannot happen
-                  abort();
+                  printf("sysIdx %d - %d\n", sysIdx1, sysIdx2);
+                  Measure* m1 = c1->measure();
+                  Measure* m2 = c2->measure();
+                  printf("Measure %d - %d, %d %d\n", m1->no(), m2->no(), m1->tick(), m2->tick());
+                  // abort();
                   }
             // case 4: end segment
             else {
