@@ -22,7 +22,6 @@
 #include "style.h"
 #include "xml.h"
 #include "score.h"
-#include "mtime.h"
 
 Style* style;
 //  20 points        font design size
@@ -31,6 +30,7 @@ Style* style;
 //  spatium = 20/4 points
 
 double _spatium;
+double _spatiumMag;
 QVector<TextStyle> defaultTextStyles;
 
 //---------------------------------------------------------
@@ -44,97 +44,100 @@ static const QString ff("Times New Roman");
 #define OS     OFFSET_SPATIUM
 
 const TextStyle defaultTextStyleArray[] = {
-      TextStyle(QString("Symbols1"), QString("MScore"), 20, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Symbols1"), QString("MScore"), 20, false, false, false,
          ALIGN_LEFT),
 
-      TextStyle(QString("Symbols3"), QString("MScore"), 14, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Symbols3"), QString("MScore"), 14, false, false, false,
          ALIGN_LEFT),
 
-      TextStyle(QString("Title"), ff, 24, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Title"), ff, 24, false, false, false,
          ALIGN_HCENTER | ALIGN_TOP, 0, 0, OA, 50, 0),
 
-      TextStyle(QString("Subtitle"), ff, 14, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Subtitle"), ff, 14, false, false, false,
          ALIGN_HCENTER | ALIGN_TOP, 0, MM(10), OA, 50, 0),
 
-      TextStyle(QString("Composer"), ff, 12, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Composer"), ff, 12, false, false, false,
          ALIGN_RIGHT | ALIGN_BOTTOM, 0, 0, OA, 100, 100),
 
-      TextStyle(QString("Poet"), ff, 12, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Poet"), ff, 12, false, false, false,
          ALIGN_LEFT | ALIGN_BOTTOM, 0, 0, OA, 0, 100),
 
-      TextStyle(QString("Lyrics"), ff, 11, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Lyrics"), ff, 11, false, false, false,
          ALIGN_HCENTER | ALIGN_TOP, 0, 7),
 
-      TextStyle(QString("Fingering"), ff,  8, false, false, false,
-         ALIGN_CENTER),
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Fingering"), ff,  8, false, false, false,
+         ALIGN_CENTER, 0.0, 0.0, OA, 0.0, 0.0, true),
 
-      TextStyle(QString("InstrumentsLong"),   ff, 12, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "InstrumentsLong"),   ff, 12, false, false, false,
          ALIGN_RIGHT | ALIGN_VCENTER),
 
-      TextStyle(QString("InstrumentsShort"),   ff, 12, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "InstrumentsShort"),   ff, 12, false, false, false,
          ALIGN_RIGHT | ALIGN_VCENTER),
 
-      TextStyle(QString("InstrumentsExcerpt"), ff, 18, false, false, false,
-         ALIGN_LEFT | ALIGN_BOTTOM, 0, 0, OA, 0, 100),
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "InstrumentsExcerpt"), ff, 18, false, false, false,
+         ALIGN_LEFT | ALIGN_BOTTOM, 0.0, 0.0, OA, 0, 100),
 
-      TextStyle(QString("Dynamics"), ff, 12, false, true, false,
-         ALIGN_LEFT | ALIGN_BASELINE, 0.0, 8.0, OS),
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Dynamics"), ff, 12, false, true, false,
+         ALIGN_LEFT | ALIGN_BASELINE, 0.0, 8.0, OS, 0, 0, true),
 
-      TextStyle(QString("Technik"), ff, 12, false, true, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Technik"), ff, 12, false, true, false,
          ALIGN_LEFT | ALIGN_BASELINE, 0.0, -2.0, OS),
 
-/*12*/
-      TextStyle(QString("Tempo"), ff, 12, true, false, false,
-         ALIGN_LEFT, 0, -5.0, OS, 0, 0, true),
+/*13*/
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Tempo"), ff, 12, true, false, false,
+         ALIGN_LEFT | ALIGN_BASELINE, 0, -4.0, OS, 0, 0, true, .0, .0, 0, Qt::black, false, true),
 
-      TextStyle(QString("Metronome"), ff, 12, true, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Metronome"), ff, 12, true, false, false,
          ALIGN_LEFT),
 
-      TextStyle(QString("Copyright"), ff, 8, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Copyright"), ff, 8, false, false, false,
          ALIGN_HCENTER | ALIGN_TOP, 0, MM(-15), OA, 50.0, 100.0),
 
-      TextStyle(QString("Measure Number"), ff, 8, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Measure Number"), ff, 8, false, false, false,
          ALIGN_CENTER | ALIGN_BOTTOM, 0.0, 0.0, OS),
 
-      TextStyle(QString("Page Number Odd"), ff, 12, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Page Number Odd"), ff, 12, false, false, false,
          ALIGN_RIGHT | ALIGN_BASELINE, MM(-10), MM(-10), OA, 100.0, 100.0),
 
-      TextStyle(QString("Page Number Even"), ff, 12, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Page Number Even"), ff, 12, false, false, false,
          ALIGN_LEFT | ALIGN_BASELINE, MM(10), MM(-10), OA, 0.0, 100.0),
 
-      TextStyle(QString("Translator"), ff, 11, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Translator"), ff, 11, false, false, false,
          ALIGN_HCENTER | ALIGN_TOP, 0, 6),
 
-      TextStyle(QString("Dynamics1"), QString("MScore1"), 20, false, false, false,
-         ALIGN_LEFT | ALIGN_BASELINE, 0.0, 8.0, OS),
-
-      TextStyle(QString("Tuplets"), ff,  8, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Tuplets"), ff,  8, false, false, false,
          ALIGN_CENTER),
 
-      TextStyle(QString("System"), ff,  10, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "System"), ff,  10, false, false, false,
          ALIGN_LEFT, 0, -4.0, OS, 0, 0, true),
 
-      TextStyle(QString("Staff"), ff,  10, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Staff"), ff,  10, false, false, false,
          ALIGN_LEFT, 0, -4.0, OS, 0, 0, true),
 
-      TextStyle(QString("Chordname"), ff,  12, false, false, false,
-         ALIGN_LEFT, 0, -5.0, OS, 0, 0, true),
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Chordname"), ff,  12, false, false, false,
+         ALIGN_LEFT | ALIGN_BASELINE, 0, -4.0, OS, 0, 0, true),
 
-      TextStyle(QString("Rehearsal Mark"), ff,  14, true, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Rehearsal Mark"), ff,  14, true, false, false,
          ALIGN_HCENTER | ALIGN_BASELINE, 0, -3.0, OS, 0, 0, true,
          0.3, 1.0, 20, Qt::black, false, true),
 
-      TextStyle(QString("Repeat Text"), ff,  12, false, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Repeat Text"), ff,  12, false, false, false,
          ALIGN_HCENTER | ALIGN_BASELINE, 0, -2.0, OS, 100, 0, true),
 
-      TextStyle(QString("Volta"), ff, 11, true, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Volta"), ff, 11, true, false, false,
          ALIGN_LEFT, 0, -5.0, OS, 0, 0, true),
 
-      TextStyle(QString("Frame"), ff, 11, true, false, false,
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Frame"), ff, 11, true, false, false,
          ALIGN_LEFT, 0, 0, OS, 0, 0, true),
 
-      TextStyle(QString("TextLine"), ff,  12, false, false, false,
-         ALIGN_HCENTER, 0, 0, OS, 0, 0, true),
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "TextLine"), ff,  12, false, false, false,
+         ALIGN_LEFT | ALIGN_VCENTER, 0, 0, OS, 0, 0, true),
+
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "Glissando"), ff, 8, false, true, false,
+         ALIGN_HCENTER | ALIGN_BASELINE, 0.0, 0.0, OS, 0, 0, true),
+
+      TextStyle(QT_TRANSLATE_NOOP("MuseScore", "String Number"), ff,  8, false, false, false,
+         ALIGN_CENTER, 0, -5.0, OS, 100, 0, true, 0.20, 0.0, 0, Qt::black, true, false),
       };
 
 #undef MM
@@ -154,7 +157,7 @@ Style defaultStyle = {
       Spatium(2),     // lyricsDistance
       Spatium(2),     // lyricsMinBottomDistance
       Spatium(7.0),   // dist. between system and vertical box
-      Spatium(7.0),   // dist. between vertical box and next system
+      Spatium(1.0),   // dist. between vertical box and next system
 
       Spatium(4.0),   // minMeasureWidth  12.0
       Spatium(0.16),  // barWidth;
@@ -163,7 +166,7 @@ Style defaultStyle = {
       Spatium(0.30),  // doubleBarDistance;
       Spatium(0.30),  // endBarDistance
       Spatium(0.35),  // system bracket width
-      Spatium(0.3),   // system bracket distance
+      Spatium(0.25),  // system bracket distance
 
       Spatium(0.5),   // clefLeftMargin;
       Spatium(0.5),   // keysig left margin
@@ -171,17 +174,17 @@ Style defaultStyle = {
       Spatium(1.75),  // clef/key/sig right margin
 
       Spatium(0.13),  // stemWidth = 1.3 * staffLineWidth
+      Spatium(1.0),   // beginRepeatLeftMargin
 
-      Spatium(.5),    // minNoteDistance
+      Spatium(0.4),   // minNoteDistance
       Spatium(1.5),   // barNoteDistance
       Spatium(1.0),   // noteBarDistance
 
-      1.3,              // measureSpacing
+      1.2,              // measureSpacing
 
       Spatium(0.08),    // staff line width
       Spatium(0.08),    // ledgerLineWidth;
       Spatium(1.6),     // akkoladeWidth;
-      Spatium(5.5),     // akkoladeDistance;
       Spatium(0.13),    // prefixDistance
 
       Spatium(0.22),    // prefixNoteDistance
@@ -192,14 +195,16 @@ Style defaultStyle = {
       0.05,             // beamMinSlope
       0.2,              // beamMaxSlope
       division,         //  maxBeamTicks : 1/4 or 1/2 groups
-      Spatium(0.5),     // dotNoteDistance
+      Spatium(0.35),    // dotNoteDistance
       Spatium(0.25),    // dotRestDistance
       Spatium(0.5),     // dotDotDistance
+
       Spatium(0.5),     // propertyDistanceHead
-      Spatium(0.5),     // propertyDistanceStem; note property to note stem
-      Spatium(0.25),    // propertyDistance; note property to note property
+      Spatium(0.5),     // propertyDistanceStem; note articulation to note stem
+      Spatium(0.25),    // propertyDistance; note articulation to note articulation
+
       0.7,              // pageFillLimit
-      0.5,              // lastSystemFillLimit
+      0.3,              // lastSystemFillLimit
       Spatium(1.2),     // hairpinHeight
       Spatium(0.5),     // hairpinContHeight
       Spatium(0.13),    // hairpinWidth
@@ -218,12 +223,25 @@ Style defaultStyle = {
       0.8,              // smallClefMag
       true,             // genClef
       true,             // genKeySig
+      true,             // genTimesig
+      true,             // genCourtesyTimesig
+
       false,            // use german note names
+      false,            // use symbols in chord names
+      false,            // display in concert pitch
+      false,            // create multi measure rests
+      2,                // minimum number of empty measures for multi measure rest
+      Spatium(4),       // minimum width of multi measure rest
+
+      false,            // hide empty staves
+
+      { UP, DOWN, UP, DOWN }, // stem direction in multi voice context
+
       // play style
-      70,               // gateTime
+      85,               // gateTime
       100,              // tenutoGateTime
-      40,               // staccatoGateTime
-      90                // slurGateTime
+      50,               // staccatoGateTime
+      100               // slurGateTime
       };
 
 //---------------------------------------------------------
@@ -425,6 +443,8 @@ void Style::load(QDomElement e, int version)
                   clefKeyRightMargin = Spatium(d);
             else if (tag == "stemWidth")
                   stemWidth = Spatium(d);
+            else if (tag == "beginRepeatLeftMargin")
+                  beginRepeatLeftMargin = Spatium(d);
             else if (tag == "minNoteDistance")
                   minNoteDistance = Spatium(d);
             else if (tag == "spacing16")        // obsolete
@@ -447,8 +467,9 @@ void Style::load(QDomElement e, int version)
                   ledgerLineWidth = Spatium(d);
             else if (tag == "akkoladeWidth")
                   akkoladeWidth = Spatium(d);
-            else if (tag == "akkoladeDistance")
-                  akkoladeDistance = Spatium(d);
+            else if (tag == "akkoladeDistance") {
+                  accoladeDistance = Spatium(d);
+                  }
             else if (tag == "prefixDistance")
                   prefixDistance = Spatium(d);
             else if (tag == "prefixNoteDistance")
@@ -520,8 +541,24 @@ void Style::load(QDomElement e, int version)
                   genClef = i;
             else if (tag == "genKeysig")
                   genKeysig = i;
+            else if (tag == "genTimesig")
+                  genTimesig = i;
+            else if (tag == "genCourtesyTimesig")
+                  genCourtesyTimesig = i;
             else if (tag == "useGermanNoteNames")
                   useGermanNoteNames = i;
+            else if (tag == "chordNamesUseSymbols")
+                  chordNamesUseSymbols = i;
+            else if (tag == "displayInConcertPitch")
+                  concertPitch = i;
+            else if (tag == "createMultiMeasureRests")
+                  createMultiMeasureRests = i;
+            else if (tag == "minEmptyMeasures")
+                  minEmptyMeasures = i;
+            else if (tag == "minMMRestWidth")
+                  minMMRestWidth = Spatium(d);
+            else if (tag == "hideEmptyStaves")
+                  hideEmptyStaves = i;
             else if (tag == "gateTime")
                   gateTime  = i;
             else if (tag == "tenutoGateTime")
@@ -530,6 +567,11 @@ void Style::load(QDomElement e, int version)
                   staccatoGateTime = i;
             else if (tag == "slurGateTime")
                   slurGateTime = i;
+            else if (tag == "stemDir") {
+                  int voice = e.attribute("voice", "1").toInt() - 1;
+                  if (voice >= 0 && voice < VOICES)
+                        stemDir[voice] = val == "up" ? UP : DOWN;
+                  }
             else
                   domError(e);
             }
@@ -546,7 +588,8 @@ void Style::save(Xml& xml)
       xml.tag("staffUpperBorder",       staffUpperBorder.val());
       xml.tag("staffLowerBorder",       staffLowerBorder.val());
       xml.tag("staffDistance",          staffDistance.val());
-      xml.tag("akkoladeDistance",       akkoladeDistance.val());
+      xml.tag("akkoladeDistance",       accoladeDistance.val());
+
       xml.tag("systemDistance",         systemDistance.val());
       xml.tag("lyricsDistance",         lyricsDistance.val());
       xml.tag("lyricsMinBottomDistance", lyricsMinBottomDistance.val());
@@ -567,6 +610,7 @@ void Style::save(Xml& xml)
       xml.tag("timesigLeftMargin",      timesigLeftMargin.val());
       xml.tag("clefKeyRightMargin",     clefKeyRightMargin.val());
       xml.tag("stemWidth",              stemWidth.val());
+      xml.tag("beginRepeatLeftMargin",  beginRepeatLeftMargin.val());
       xml.tag("minNoteDistance",        minNoteDistance.val());
 
       xml.tag("measureSpacing",         measureSpacing);
@@ -611,8 +655,33 @@ void Style::save(Xml& xml)
       xml.tag("smallClefMag",           smallClefMag);
       xml.tag("genClef",                genClef);
       xml.tag("genKeysig",              genKeysig);
+      if (!genTimesig)
+            xml.tag("genTimesig", genTimesig);
+      if (!genCourtesyTimesig)
+            xml.tag("genCourtesyTimesig", genCourtesyTimesig);
       if (useGermanNoteNames)
             xml.tag("useGermanNoteNames", useGermanNoteNames);
+      if (chordNamesUseSymbols)
+            xml.tag("chordNamesUseSymbols", chordNamesUseSymbols);
+      if (concertPitch)
+            xml.tag("displayInConcertPitch", concertPitch);
+      if (createMultiMeasureRests)
+            xml.tag("createMultiMeasureRests", createMultiMeasureRests);
+
+      xml.tag("minEmptyMeasures", minEmptyMeasures);
+      xml.tag("minMMRestWidth", minMMRestWidth.val());
+
+      if (hideEmptyStaves)
+            xml.tag("hideEmptyStaves", hideEmptyStaves);
+
+      if (stemDir[0] != UP)
+            xml.tag("stemDir voice=\"1\"", "down");
+      if (stemDir[1] != DOWN)
+            xml.tag("stemDir voice=\"2\"", "up");
+      if (stemDir[2] != UP)
+            xml.tag("stemDir voice=\"3\"", "down");
+      if (stemDir[3] != DOWN)
+            xml.tag("stemDir voice=\"4\"", "up");
 
       xml.tag("gateTime",               gateTime);
       xml.tag("tenutoGateTime",         tenutoGateTime);
