@@ -3,7 +3,7 @@
 //  Linux Music Score Editor
 //  $Id: system.cpp,v 1.41 2006/04/12 14:58:10 wschweer Exp $
 //
-//  Copyright (C) 2002-2007 Werner Schweer and others
+//  Copyright (C) 2002-2008 Werner Schweer and others
 //
 //  This program is free software; you can redistribute it and/or modify
 //  it under the terms of the GNU General Public License version 2.
@@ -53,6 +53,7 @@ SysStaff::SysStaff()
       {
       idx             = 0;
       instrumentName  = 0;
+      _show           = true;
       }
 
 //---------------------------------------------------------
@@ -61,8 +62,10 @@ SysStaff::SysStaff()
 
 SysStaff::~SysStaff()
       {
-      foreach(Bracket* b, brackets)
-            delete b;
+      foreach(Bracket* b, brackets) {
+            if (b)
+                  delete b;
+            }
       brackets.clear();
       if (instrumentName) {
             delete instrumentName;
@@ -131,19 +134,27 @@ SysStaff* System::removeStaff(int idx)
 
 //---------------------------------------------------------
 //   layout
+//    If first MeasureBase is a HBOX, then xo1 is the
+//    width of this box.
 //---------------------------------------------------------
 
 /**
  Layout the System.
 */
 
-void System::layout(ScoreLayout* layout)
+void System::layout(ScoreLayout* layout, double xo1)
       {
       if (isVbox())                 // ignore vbox
             return;
       static const Spatium instrumentNameOffset(1.0);
+
+      Style* style = score()->style();
+
       int nstaves  = _staves.size();
 
+      if (nstaves != score()->nstaves())
+            printf("System::layout: nstaves %d != %d\n", nstaves, score()->nstaves());
+
       //---------------------------------------------------
       //  find x position of staves
       //    create brackets
@@ -151,16 +162,21 @@ void System::layout(ScoreLayout* layout)
 
       double xoff2 = 0.0;         // x offset for instrument name
 
-      int bracketLevels = score()->staff(0)->bracketLevels();
+      int bracketLevels = 0;
+      for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
+            int n = score()->staff(staffIdx)->bracketLevels();
+            if (n > bracketLevels)
+                  bracketLevels = n;
+            }
       double bracketWidth[bracketLevels];
       for (int i = 0; i < bracketLevels; ++i)
             bracketWidth[i] = 0.0;
 
       for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
-            Staff* s = score()->staff(staffIdx);
-            if (!s->show())
-                  continue;
+            Staff* s     = score()->staff(staffIdx);
             SysStaff* ss = _staves[staffIdx];
+            if (!ss->show())
+                  continue;
 
             if (bracketLevels < ss->brackets.size()) {
                   for (int i = bracketLevels; i < ss->brackets.size(); ++i) {
@@ -208,25 +224,26 @@ void System::layout(ScoreLayout* layout)
       //  layout  SysStaff and StaffLines
       //---------------------------------------------------
 
+      // xoff2 += xo1;
       _leftMargin = xoff2;
 
       for (int i = 0; i < bracketLevels; ++i)
-            _leftMargin += bracketWidth[i];
+            _leftMargin += bracketWidth[i] + style->bracketDistance.point();
 
       for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
             SysStaff* s  = _staves[staffIdx];
             Staff* staff = score()->staff(staffIdx);
-            if (!staff->show()) {
+            if (!s->show()) {
                   s->setbbox(QRectF());
                   continue;
                   }
             double staffMag = staff->mag();
-            s->setbbox(QRectF(_leftMargin, 0.0, 0.0, 4 * _spatium * staffMag));
+            s->setbbox(QRectF(_leftMargin + xo1, 0.0, 0.0, 4 * _spatium * staffMag));
             }
 
       if (nstaves > 1 && barLine == 0) {
             barLine = new Line(score(), true);
-            barLine->setLineWidth(score()->style()->barWidth);
+            barLine->setLineWidth(style->barWidth);
             barLine->setParent(this);
             }
       else if (nstaves <= 1 && barLine) {
@@ -234,7 +251,7 @@ void System::layout(ScoreLayout* layout)
             barLine = 0;
             }
       if (barLine) {
-            barLine->setPos(_leftMargin, score()->style()->barWidth.point() * .25);
+            barLine->setPos(_leftMargin + xo1, style->barWidth.point() * .25);
             }
 
       //---------------------------------------------------
@@ -242,19 +259,17 @@ void System::layout(ScoreLayout* layout)
       //---------------------------------------------------
 
       for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
-            Staff* s = score()->staff(staffIdx);
-            if (!s->show())
+            SysStaff* ss = _staves[staffIdx];
+            if (!ss->show())
                   continue;
 
-            SysStaff* ss = _staves[staffIdx];
-            double xo = 0.0;
+            double xo = -xo1;
             for (int i = 0; i < bracketLevels; ++i) {
-                  xo += bracketWidth[i] + _spatium * .25;
-                  Bracket*   b = ss->brackets[i];
+                  xo += bracketWidth[i] + style->bracketDistance.point();
+                  Bracket* b = ss->brackets[i];
                   if (b == 0)
                         continue;
 
-                  qreal sy = ss->bbox().top();
                   if (b->span() >= (nstaves - staffIdx)) {
                         //
                         // this may happen if a system was removed in
@@ -262,9 +277,20 @@ void System::layout(ScoreLayout* layout)
                         //
                         b->setSpan(nstaves - staffIdx);
                         }
-                  qreal ey = _staves[staffIdx + b->span() - 1]->bbox().bottom();
-                  b->setPos(_leftMargin - xo, sy);
-                  b->setHeight(ey - sy);
+                  if (!_staves[staffIdx + b->span() - 1]->show()) {
+                        //
+                        // if the bracket ends on an invisible staff
+                        // find last visible staff in bracket
+                        //
+                        for (int j = staffIdx + b->span() - 2; j >= staffIdx; --j) {
+                              if (_staves[j]->show()) {
+                                    b->setSpan(j - staffIdx + 1);
+                                    break;
+                                    }
+                              }
+                        }
+                  // right align bracket
+                  b->setXpos(_leftMargin - xo + bracketWidth[i] - b->width());
                   }
             }
 
@@ -278,7 +304,7 @@ void System::layout(ScoreLayout* layout)
             int nstaves = p->nstaves();
             if (s->instrumentName && !s->instrumentName->isEmpty()) {
                   double d  = instrumentNameOffset.point() + s->instrumentName->bbox().width();
-                  s->instrumentName->setXpos(xoff2 - d);
+                  s->instrumentName->setXpos(xoff2 - d + xo1);
                   }
             idx += nstaves;
             }
@@ -295,25 +321,30 @@ void System::layout2(ScoreLayout* layout)
       if (isVbox())                 // ignore vbox
             return;
 
-      int staves = _staves.size();
+      int nstaves = _staves.size();
 
       qreal y = 0.0;
       int lastStaffIdx = 0;   // last visible staff
-      for (int staffIdx = 0; staffIdx < staves; ++staffIdx) {
-            Staff* staff    = score()->staff(staffIdx);
-            if ((staffIdx + 1) == staves)
+      for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
+            Staff* staff = score()->staff(staffIdx);
+            if ((staffIdx + 1) == nstaves) {
                   setDistance(staffIdx, score()->style()->systemDistance);
-            else if ((staff->part()->staves()->size() + 1) < staff->rstaff())
+                  }
+            else if (staff->rstaff() < (staff->part()->staves()->size()-1)) {
                   setDistance(staffIdx, score()->style()->accoladeDistance);
-            else
+                  }
+            else {
                   setDistance(staffIdx, score()->style()->staffDistance);
+                  }
             double dist = 0.0;
-            foreach(MeasureBase* m, ml)
+            foreach(MeasureBase* m, ml) {
                   dist = std::max(dist, m->distance(staffIdx));
+                  dist = std::max(dist, m->userDistance(staffIdx).point());
+                  }
             if (dist > distance(staffIdx))
                   setDistance(staffIdx, dist);
             SysStaff* s = _staves[staffIdx];
-            if (!staff->show()) {
+            if (!s->show()) {
                   s->setbbox(QRectF());
                   continue;
                   }
@@ -329,8 +360,8 @@ void System::layout2(ScoreLayout* layout)
                   m->setHeight(systemHeight);
             }
 
-      double staffY[staves];
-      for (int i = 0; i < staves; ++i)
+      double staffY[nstaves];
+      for (int i = 0; i < nstaves; ++i)
             staffY[i] = staff(i)->bbox().y();
 
       if (barLine) {
@@ -340,28 +371,18 @@ void System::layout2(ScoreLayout* layout)
             }
 
       //---------------------------------------------------
-      //  layout brackets
+      //  layout brackets vertical position
       //---------------------------------------------------
 
-      int bracketLevels = score()->staff(0)->bracketLevels();
-      double bracketWidth[bracketLevels];
-      for (int i = 0; i < bracketLevels; ++i)
-            bracketWidth[i] = 0.0;
-
-      for (int staffIdx = 0; staffIdx < staves; ++staffIdx) {
-            Staff* s = score()->staff(staffIdx);
-            if (!s->show())
-                  continue;
+      for (int staffIdx = 0; staffIdx < nstaves; ++staffIdx) {
             SysStaff* ss = _staves[staffIdx];
+            if (!ss->show())
+                  continue;
 
-            double xo = 0.0;
-            for (int i = 0; i < bracketLevels; ++i) {
-                  xo += bracketWidth[i] + _spatium * .25;
-                  Bracket*   b = ss->brackets[i];
+            foreach(Bracket* b, ss->brackets) {
                   if (b == 0)
                         continue;
-
-                  int restStaves = staves - staffIdx;
+                  int restStaves = nstaves - staffIdx;
                   if (b->span() >= restStaves) {
                         //
                         // this may happen if a system was removed in
@@ -371,7 +392,7 @@ void System::layout2(ScoreLayout* layout)
                         }
                   qreal sy = ss->bbox().top();
                   qreal ey = _staves[staffIdx + b->span() - 1]->bbox().bottom();
-                  b->setPos(b->ipos().x(), sy);
+                  b->setYpos(sy);
                   b->setHeight(ey - sy);
                   b->layout(layout);
                   }
@@ -381,20 +402,20 @@ void System::layout2(ScoreLayout* layout)
       //  layout instrument names
       //---------------------------------------------------
 
-      int idx = 0;
+      int staffIdx = 0;
       foreach(Part* p, *score()->parts()) {
-            SysStaff* s = staff(idx);
+            SysStaff* s = staff(staffIdx);
             int nstaves = p->nstaves();
             if (s->instrumentName && !s->instrumentName->isEmpty()) {
                   //
                   // override Text->layout()
                   //
                   double y1 = s->bbox().top();
-                  double y2 = staff(idx + nstaves - 1)->bbox().bottom();
+                  double y2 = staff(staffIdx + nstaves - 1)->bbox().bottom();
                   double y  = y1 + (y2 - y1) * .5 - s->instrumentName->bbox().height() * .5;
                   s->instrumentName->setYpos(y);
                   }
-            idx += nstaves;
+            staffIdx += nstaves;
             }
       }
 
@@ -452,14 +473,14 @@ void System::setInstrumentName(int staffIdx)
             return;
       SysStaff* staff = _staves[staffIdx];
       if (staff->instrumentName == 0)
-            staff->instrumentName = new Text(cs);
+            staff->instrumentName = new TextC(s->part()->longNameBase(), cs);
       if (_firstSystem) {
             staff->instrumentName->setSubtype(TEXT_INSTRUMENT_LONG);
-            staff->instrumentName->setDoc(s->longName());
+            staff->instrumentName->setTextBase(s->part()->longNameBase());
             }
       else {
             staff->instrumentName->setSubtype(TEXT_INSTRUMENT_SHORT);
-            staff->instrumentName->setDoc(s->shortName());
+            staff->instrumentName->setTextBase(s->part()->shortNameBase());
             }
       staff->instrumentName->setParent(this);
       staff->instrumentName->setTrack(staffIdx * VOICES);
@@ -502,12 +523,14 @@ void System::add(Element* el)
       {
       el->setParent(this);
       if (el->type() == TEXT && (el->subtype() == TEXT_INSTRUMENT_LONG || el->subtype() == TEXT_INSTRUMENT_SHORT)) {
-            _staves[el->staffIdx()]->instrumentName = (Text*)el;
+            _staves[el->staffIdx()]->instrumentName = static_cast<TextC*>(el);
             }
       else if (el->type() == BRACKET) {
             SysStaff* ss = _staves[el->staffIdx()];
-            Bracket* b   = (Bracket*)el;
+            Bracket* b   = static_cast<Bracket*>(el);
             int level = b->level();
+            while (level >= ss->brackets.size())
+                  ss->brackets.append(0);
             ss->brackets[level] = b;
             b->staff()->setBracket(level,   b->subtype());
             b->staff()->setBracketSpan(level, b->span());
@@ -533,7 +556,6 @@ void System::remove(Element* el)
                   if (staff->brackets[i] == el) {
                         staff->brackets[i] = 0;
                         el->staff()->setBracket(i, NO_BRACKET);
-                        // TODO: remove empty bracket levels
 //TODO                        score()->layout();
                         return;
                         }
@@ -586,6 +608,32 @@ int System::snapNote(int tick, const QPointF p, int staff) const
       return ((Measure*)ml.back())->snap(tick, p-pos());          // TODO: MeasureBase
       }
 
+//---------------------------------------------------------
+//   firstMeasure
+//---------------------------------------------------------
+
+Measure* System::firstMeasure() const {
+      for (MeasureBase* mb = ml.front(); mb; mb = mb->next()) {
+            if (mb->type() != MEASURE)
+                  continue;
+            return static_cast<Measure*>(mb);
+            }
+      return 0;
+      }
+
+//---------------------------------------------------------
+//   lastMeasure
+//---------------------------------------------------------
+
+Measure* System::lastMeasure() const {
+      for (MeasureBase* mb = ml.back(); mb; mb = mb->prev()) {
+            if (mb->type() != MEASURE)
+                  continue;
+            return static_cast<Measure*>(mb);
+            }
+      return 0;
+      }
+
 //---------------------------------------------------------
 //   prevMeasure
 //---------------------------------------------------------
@@ -633,7 +681,7 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
             int sysIdx2  = systems->indexOf(s2);
 
             double lw = l->bbox().right();            // lyrics width
-            QPointF p1(lw, l->basePosition());
+            QPointF p1(lw, l->baseLine());
 
             int segIdx = 0;
             for (int i = sysIdx1; i <= sysIdx2; ++i, ++segIdx) {
@@ -647,7 +695,7 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
                   if (sysIdx1 == sysIdx2) {
                         // single segment
                         line->setPos(p1);
-                        double len = seg->canvasPos().x() - l->canvasPos().x() - lw + 2. * _spatium;
+                        double len = seg->canvasPos().x() - l->canvasPos().x() - lw + 2 * _spatium;
                         line->setLen(Spatium(len / _spatium));
                         }
                   else if (i == sysIdx1) {
@@ -674,6 +722,11 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
       //
       int verse   = l->no();
       Segment* ns = s;
+
+      // TODO: then next two values should be style parameters
+      const double maxl = 0.8 * _spatium;   // lyrics hyphen length
+      const Spatium hlw(0.1);               // hyphen line width
+
       while ((ns = ns->next1())) {
             LyricsList* nll = ns->lyricsList(staffIdx);
             if (!nll)
@@ -688,7 +741,7 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
             Line* line;
             if (sl->isEmpty()) {
                   line = new Line(l->score(), false);
-                  line->setLineWidth(Spatium(0.1));
+                  line->setLineWidth(hlw);
                   l->add(line);
                   }
             else {
@@ -709,16 +762,11 @@ void System::layoutLyrics(ScoreLayout* layout, Lyrics* l, Segment* s, int staffI
                   x2 = system->canvasPos().x() + system->bbox().width();
                   }
 
-            len = x2 - x1 - 2 * _spatium - w;
-            double xo = _spatium;
-            if (len < 0.0) {                // set minimum len
-                  len       = _spatium * .5;
-                  double gap = x2 - x1 - w - len;
-                  if (gap < 0.0)
-                        xo = 0.0;
-                  else
-                        xo = gap * .5;
-                  }
+            double gap = x2 - x1 - w;
+            len        = gap;
+            if (len > maxl)
+                  len = maxl;
+            double xo = (gap - len) * .5;
 
             line->setPos(QPointF(x + xo, y));
             Spatium sp;
@@ -741,9 +789,8 @@ void System::collectElements(QList<const Element*>& el) const
             return;
       if (barLine)
             el.append(barLine);
-      int staffIdx = 0;
       foreach(SysStaff* st, _staves) {
-            if (!score()->staff(staffIdx++)->show())
+            if (!st->show())
                   continue;
             foreach(Bracket* b, st->brackets) {
                   if (b)
