@@ -32,6 +32,8 @@
 #include "symbol.h"
 #include "layout.h"
 #include "textline.h"
+#include "preferences.h"
+#include "system.h"
 
 TextPalette* palette;
 
@@ -42,13 +44,14 @@ TextPalette* palette;
 TextBase::TextBase()
       {
       _doc          = new QTextDocument(0);
-//      _doc->setUseDesignMetrics(true);
+      _doc->setUseDesignMetrics(true);
 
       _frameWidth   = 0.0;
       _paddingWidth = 0.0;
-      _frameColor   = QColor(Qt::black);
+      _frameColor   = preferences.defaultColor;
       _frameRound   = 25;
       _circle       = false;
+      _layoutWidth  = -1;
 
       QTextOption to = _doc->defaultTextOption();
       to.setUseDesignMetrics(true);
@@ -68,6 +71,11 @@ TextBase::TextBase(const TextBase& t)
       _frameColor   = t._frameColor;
       _frameRound   = t._frameRound;
       _circle       = t._circle;
+      frame         = t.frame;
+      _bbox         = t._bbox;
+      _layoutWidth  = t._layoutWidth;
+      _doc->documentLayout()->setPaintDevice(pdev);
+      layout(_layoutWidth);
       }
 
 //---------------------------------------------------------
@@ -90,9 +98,14 @@ void TextBase::setText(const QString& s, Align align)
       _doc->clear();
       QTextCursor cursor(_doc);
       cursor.movePosition(QTextCursor::Start);
-      if (align & ALIGN_HCENTER) {
+      if (align & (ALIGN_HCENTER | ALIGN_RIGHT)) {
+            Qt::Alignment a;
+            if (align & ALIGN_HCENTER)
+                  a = Qt::AlignHCenter;
+            else if (align & ALIGN_RIGHT)
+                  a = Qt::AlignRight;
             QTextBlockFormat bf = cursor.blockFormat();
-            bf.setAlignment(Qt::AlignHCenter);
+            bf.setAlignment(a);
             cursor.setBlockFormat(bf);
             }
       QTextCharFormat tf = cursor.charFormat();
@@ -140,7 +153,7 @@ void TextBase::writeProperties(Xml& xml) const
             xml.tag("frameWidth", _frameWidth);
       if (_paddingWidth != 0.0)
             xml.tag("paddingWidth", _paddingWidth);
-      if (_frameColor != QColor(Qt::black))
+      if (_frameColor != preferences.defaultColor)
             xml.tag("frameColor", _frameColor);
       if (_frameRound != 25)
             xml.tag("frameRound", _frameRound);
@@ -189,11 +202,21 @@ bool TextBase::readProperties(QDomElement e)
 //   layout
 //---------------------------------------------------------
 
-void TextBase::layout(ScoreLayout*)
+void TextBase::layout(double w)
       {
+      _layoutWidth = w;
+      if (!_doc->isModified())
+            return;
       _doc->documentLayout()->setPaintDevice(pdev);
-      _doc->setTextWidth(-1.0);
-//      _doc->setTextWidth(-1.0);
+      if (w <= 0.0)
+            w = _doc->idealWidth();
+      else {
+            QTextOption to = _doc->defaultTextOption();
+            to.setUseDesignMetrics(true);
+            to.setWrapMode(QTextOption::WrapAtWordBoundaryOrAnywhere);
+            _doc->setDefaultTextOption(to);
+            }
+      _doc->setTextWidth(w);   // to make alignment work
 
       if (_frameWidth > 0.0) {
             frame = QRectF();
@@ -220,12 +243,9 @@ void TextBase::layout(ScoreLayout*)
             _bbox = frame.adjusted(-w, -w, w, w);
             }
       else {
-            // double tw = _doc->idealWidth();
-            // _bbox = QRectF(QPointF(), QPointF(tw, _doc->size().height()));
             _bbox = _doc->documentLayout()->frameBoundingRect(_doc->rootFrame());
-            // _bbox = QRectF(QPointF(), _doc->documentLayout()->documentSize());
-            // _bbox = _doc->documentLayout()->blockBoundingRect(_doc->begin());
             }
+      _doc->setModified(false);
       }
 
 //---------------------------------------------------------
@@ -234,9 +254,6 @@ void TextBase::layout(ScoreLayout*)
 
 void TextBase::draw(QPainter& p, QTextCursor* cursor) const
       {
-      p.setRenderHint(QPainter::Antialiasing, true);
-      p.setRenderHint(QPainter::TextAntialiasing, true);
-
       QAbstractTextDocumentLayout::PaintContext c;
       c.cursorPosition = -1;
       if (cursor) {
@@ -248,6 +265,7 @@ void TextBase::draw(QPainter& p, QTextCursor* cursor) const
             }
       QColor color = p.pen().color();
       c.palette.setColor(QPalette::Text, color);
+
       _doc->documentLayout()->setProperty("cursorWidth", QVariant(int(lrint(2.0*DPI/PDPI))));
       _doc->documentLayout()->draw(&p, c);
 
@@ -258,10 +276,13 @@ void TextBase::draw(QPainter& p, QTextCursor* cursor) const
             if (_circle)
                   p.drawArc(frame, 0, 5760);
             else {
+                  double mag = 1.0; // DPI/PDPI;
+
                   int r2 = _frameRound * lrint((frame.width() / frame.height()));
                   if (r2 > 99)
                         r2 = 99;
-                  p.drawRoundRect(frame, _frameRound, r2);
+                  QRectF fr(frame.x(), frame.y(), frame.width() * mag, frame.height() * mag);
+                  p.drawRoundRect(fr, _frameRound, r2);
                   }
             }
       }
@@ -289,6 +310,15 @@ TextB::TextB(const TextB& e)
       cursorPos = e.cursorPos;
       }
 
+//---------------------------------------------------------
+//   styleChanged
+//---------------------------------------------------------
+
+void TextB::styleChanged()
+      {
+      setSubtype(subtype());
+      }
+
 //---------------------------------------------------------
 //   TextC
 //---------------------------------------------------------
@@ -304,8 +334,8 @@ TextC::TextC(TextBase** txtb, Score* s)
 TextC::TextC(const TextC& e)
    : TextB(e)
       {
-      _tbb  = e._tbb;
-      _otb  = 0;
+      _tbb   = e._tbb;
+      _otb   = 0;
       cursor = 0;
       baseChanged();
       }
@@ -350,6 +380,8 @@ void TextC::setStyle(const TextStyle* s)
       _ryoff         = s->ryoff;
       _offsetType    = s->offsetType;
       _sizeIsSpatiumDependent = s->sizeIsSpatiumDependent;
+
+      setSystemFlag(s->systemFlag);
       if (s->systemFlag)
             setTrack(-1);
       }
@@ -362,7 +394,7 @@ Text::Text(Score* s)
    : TextB(s)
       {
       _tb = new TextBase;
-      setSubtype(TEXT_STAFF);
+      setSubtype(0);
       }
 
 Text::Text(const Text& e)
@@ -408,9 +440,9 @@ void TextB::setSubtype(int val)
 //   setAbove
 //---------------------------------------------------------
 
-void TextB::setAbove(bool val)
+void TextB::setAbove(bool /*val*/)
       {
-      setYoff(val ? -2.0 : 7.0);
+      // setYoff(val ? -2.0 : 7.0);
       }
 
 //---------------------------------------------------------
@@ -445,14 +477,16 @@ TextStyle* TextB::style() const
             case TEXT_VOLTA:            st = TEXT_STYLE_VOLTA; break;
             case TEXT_FRAME:            st = TEXT_STYLE_FRAME; break;
             case TEXT_TEXTLINE:         st = TEXT_STYLE_TEXTLINE; break;
+            case TEXT_STRING_NUMBER:    st = TEXT_STYLE_STRING_NUMBER; break;
+            case TEXT_UNKNOWN:          break;
             default:
                   printf("unknown text subtype %d <%s>\n",
                      subtype(), qPrintable(doc()->toPlainText()));
                   break;
             }
-      if (st != -1)
-            return score()->textStyle(st);
-      return 0;
+      if (st == -1)
+            st = TEXT_STYLE_STAFF;
+      return score()->textStyle(st);
       }
 
 //---------------------------------------------------------
@@ -486,6 +520,7 @@ const QString TextB::subtypeName() const
             case TEXT_VOLTA:            return "Volta";
             case TEXT_FRAME:            return "Frame";
             case TEXT_TEXTLINE:         return "TextLine";
+            case TEXT_STRING_NUMBER:    return "StringNumber";
             default:
                   printf("unknown text subtype %d\n", subtype());
                   break;
@@ -548,6 +583,8 @@ void TextB::setSubtype(const QString& s)
             st = TEXT_FRAME;
       else if (s == "TextLine")
             st = TEXT_TEXTLINE;
+      else if (s == "StringNumber")
+            st = TEXT_STRING_NUMBER;
       else
             printf("setSubtype: unknown type <%s>\n", qPrintable(s));
       setSubtype(st);
@@ -560,6 +597,10 @@ void TextB::setSubtype(const QString& s)
 void TextB::resetMode()
       {
       editMode = 0;
+      if (cursor) {
+            delete cursor;
+            cursor = 0;
+            }
       }
 
 //---------------------------------------------------------
@@ -577,33 +618,33 @@ bool TextB::isEmpty() const
 
 void TextB::layout(ScoreLayout* layout)
       {
-      textBase()->layout(layout);
+      if (parent() && parent()->type() == HBOX) {
+            textBase()->layout(parent()->width());
+            }
+      else
+            textBase()->layout(-1.0);
       setbbox(textBase()->bbox());
 
       if (parent() == 0)
             return;
 
-      QPointF o(QPointF(_xoff, _yoff));
-      if (_offsetType == OFFSET_SPATIUM)
-            o *= _spatium;
-      else
-            o *= DPI;
-      o += QPointF(_rxoff * parent()->width() * 0.01, _ryoff * parent()->height() * 0.01);
+      Element::layout(layout);      // process alignment
 
-      double th = height();
-      double tw = width();
-      QPointF p;
-      if (_align & ALIGN_BOTTOM)
-            p.setY(-th);
-      else if (_align & ALIGN_VCENTER)
-            p.setY(-(th * .5));
-      else if (_align & ALIGN_BASELINE)
-            p.setY(-basePosition());
-      if (_align & ALIGN_RIGHT)
-            p.setX(-tw);
-      else if (_align & ALIGN_HCENTER)
-            p.setX(-(tw * .5));
-      setPos(p + o);
+      if (_align & ALIGN_VCENTER && subtype() == TEXT_TEXTLINE) {
+            // special case: vertically centered text with TextLine needs to
+            // take into account the line width
+            TextLineSegment* tls = (TextLineSegment*)parent();
+            TextLine* tl = (TextLine*)(tls->line());
+            qreal textlineLineWidth = tl->lineWidth().point();
+            setYpos(pos().y() - textlineLineWidth * .5);
+            }
+
+      if (parent()->type() == MEASURE) {
+            Measure* m = static_cast<Measure*>(parent());
+            double y = track() != -1 ? m->system()->staff(track() / VOICES)->y() : 0.0;
+            double x = (tick() != -1) ? m->tick2pos(tick()) : 0.0;
+            setPos(ipos() + QPointF(x, y));
+            }
       }
 
 //---------------------------------------------------------
@@ -612,25 +653,7 @@ void TextB::layout(ScoreLayout* layout)
 
 void TextB::draw(QPainter& p) const
       {
-      p.save();
       textBase()->draw(p, cursor);
-      if (editMode) {
-            QRectF f;
-            for (QTextBlock tb = doc()->begin(); tb.isValid(); tb = tb.next()) {
-                  QTextLayout* tl = tb.layout();
-                  int n = tl->lineCount();
-                  for (int i = 0; i < n; ++i) {
-                        const QTextLine l = tl->lineAt(i);
-                        f |= l.naturalTextRect().translated(tl->position());
-                        }
-                  }
-            qreal w = 6.0 / p.matrix().m11();   // 6 pixel border
-            f.adjust(-w, -w, w, w);
-            p.setPen(QPen(QBrush(Qt::blue), w / 3.0));
-            p.setBrush(QBrush(Qt::NoBrush));
-            p.drawRect(f);
-            }
-      p.restore();
       }
 
 //---------------------------------------------------------
@@ -649,13 +672,13 @@ void TextB::setStyle(const TextStyle* s)
       _ryoff         = s->ryoff;
       _offsetType    = s->offsetType;
       _sizeIsSpatiumDependent = s->sizeIsSpatiumDependent;
-      if (s->systemFlag)
-            setSystemFlag(true);
+      setSystemFlag(s->systemFlag);
 
       textBase()->setFrameWidth(s->frameWidth);
       textBase()->setPaddingWidth(s->paddingWidth);
       textBase()->setFrameColor(s->frameColor);
       textBase()->setFrameRound(s->frameRound);
+      textBase()->setCircle(s->circle);
       }
 
 //---------------------------------------------------------
@@ -740,6 +763,8 @@ void TextB::writeProperties(Xml& xml) const
 
       if (!_sizeIsSpatiumDependent && _sizeIsSpatiumDependent != st->sizeIsSpatiumDependent)
             xml.tag("spatiumSizeDependent", _sizeIsSpatiumDependent);
+      if (subtype() == TEXT_MEASURE_NUMBER)
+            return;
       textBase()->writeProperties(xml);
       }
 
@@ -801,8 +826,10 @@ bool TextB::readProperties(QDomElement e)
 //   startEdit
 //---------------------------------------------------------
 
-bool TextB::startEdit(const QPointF& p)
+bool TextB::startEdit(Viewer* view, const QPointF& p)
       {
+      if ((type() == TEXT) && (subtype() == TEXT_MEASURE_NUMBER))
+            return false;
       cursor = new QTextCursor(doc());
       cursor->setPosition(cursorPos);
       editMode = true;
@@ -826,7 +853,8 @@ bool TextB::startEdit(const QPointF& p)
             palette->setCharFormat(cursor->charFormat());
             palette->setBlockFormat(cursor->blockFormat());
             }
-
+      qreal w = 8.0 / view->matrix().m11();
+      score()->addRefresh(abbox().adjusted(-w, -w, w, w));
       return true;
       }
 
@@ -835,12 +863,15 @@ bool TextB::startEdit(const QPointF& p)
 //    return true if event is accepted
 //---------------------------------------------------------
 
-bool TextB::edit(int, QKeyEvent* ev)
+bool TextB::edit(Viewer* view, int /*grip*/, int key, Qt::KeyboardModifiers modifiers, const QString& s)
       {
-//      score()->setLayoutAll(false);
-score()->setLayoutAll(true);
-      score()->addRefresh(abbox().adjusted(-6, -6, 12, 12));
-      int key = ev->key();
+      if (debugMode)
+            printf("TextB::edit\n");
+      bool lo = (subtype() == TEXT_INSTRUMENT_SHORT) || (subtype() == TEXT_INSTRUMENT_LONG);
+      score()->setLayoutAll(lo);
+      qreal w = 8.0 / view->matrix().m11();
+      score()->addRefresh(abbox().adjusted(-w, -w, w, w));
+
       if (key == Qt::Key_F2) {
             if (palette == 0)
                   palette = new TextPalette(score()->canvas());
@@ -854,7 +885,7 @@ score()->setLayoutAll(true);
                   }
             return true;
             }
-      if (ev->modifiers() & Qt::ControlModifier) {
+      if (modifiers == Qt::ControlModifier) {
             switch (key) {
                   case Qt::Key_B:   // toggle bold face
                         {
@@ -908,7 +939,7 @@ score()->setLayoutAll(true);
             if (key != Qt::Key_Space && key != Qt::Key_Minus)
                   return true;
             }
-      QTextCursor::MoveMode mm = (ev->modifiers() & Qt::ShiftModifier)
+      QTextCursor::MoveMode mm = (modifiers & Qt::ShiftModifier)
          ? QTextCursor::KeepAnchor : QTextCursor::MoveAnchor;
       switch (key) {
             case Qt::Key_Return:
@@ -924,11 +955,13 @@ score()->setLayoutAll(true);
                   break;
 
             case Qt::Key_Left:
-                  cursor->movePosition(QTextCursor::Left, mm);
+                  if (!cursor->movePosition(QTextCursor::Left, mm))
+                        return false;
                   break;
 
             case Qt::Key_Right:
-                  cursor->movePosition(QTextCursor::Right, mm);
+                  if (!cursor->movePosition(QTextCursor::Right, mm))
+                        return false;
                   break;
 
             case Qt::Key_Up:
@@ -956,17 +989,54 @@ score()->setLayoutAll(true);
                   break;
 
             default:
-                  cursor->insertText(ev->text());
+                  cursor->insertText(s);
                   break;
             }
+      if (key == Qt::Key_Return || key == Qt::Key_Space || key == Qt::Key_Tab) {
+            replaceSpecialChars();
+            }
       if (palette) {
             palette->setCharFormat(cursor->charFormat());
             palette->setBlockFormat(cursor->blockFormat());
             }
-      score()->addRefresh(abbox().adjusted(-6, -6, 12, 12));      // HACK
+      layout(0);
+      score()->addRefresh(abbox().adjusted(-w, -w, w, w));
       return true;
       }
 
+//---------------------------------------------------------
+//   replaceSpecialChars
+//---------------------------------------------------------
+
+bool TextB::replaceSpecialChars() {
+      QTextCursor startCur = *cursor;
+      foreach (const char* s, charReplaceMap.keys()) {
+            SymCode sym = *charReplaceMap.value(s);
+            switch (sym.type) {
+                  case SYMBOL_COPYRIGHT:
+                        if (!preferences.replaceCopyrightSymbol || subtype() != TEXT_COPYRIGHT)
+                              continue;
+                        break;
+                  case SYMBOL_FRACTION:
+                        if (!preferences.replaceFractions)
+                              continue;
+                        break;
+                  default:
+                        ;
+                  }
+            QTextCursor cur = doc()->find(s, cursor->position() - 1 - strlen(s),
+                  QTextDocument::FindWholeWords);
+            if (cur.isNull())
+                  continue;
+            // do not go beyond the cursor
+            if (cur.selectionEnd() > cursor->selectionEnd())
+                  continue;
+            addSymbol(sym, &cur);
+            return true;
+            }
+      return false;
+      }
+
 //---------------------------------------------------------
 //   moveCursorToEnd
 //---------------------------------------------------------
@@ -977,6 +1047,16 @@ void TextB::moveCursorToEnd()
             cursor->movePosition(QTextCursor::End);
       }
 
+//---------------------------------------------------------
+//   moveCursor
+//---------------------------------------------------------
+
+void TextB::moveCursor(int col)
+      {
+      if (cursor)
+            cursor->setPosition(col);
+      }
+
 //---------------------------------------------------------
 //   endEdit
 //---------------------------------------------------------
@@ -992,11 +1072,6 @@ void TextB::endEdit()
 
       if (subtype() == TEXT_COPYRIGHT)
             score()->undoChangeCopyright(doc()->toHtml("UTF-8"));
-      if (subtype() == TEXT_TEXTLINE) {
-            TextLineSegment* tls = (TextLineSegment*)parent();
-            TextLine* tl = (TextLine*)(tls->line());
-            tl->setText(getText());
-            }
       }
 
 //---------------------------------------------------------
@@ -1017,11 +1092,11 @@ QPainterPath TextB::shape() const
       }
 
 //---------------------------------------------------------
-//   basePosition
+//   baseLine
 //    returns ascent of first text line in first block
 //---------------------------------------------------------
 
-qreal TextB::basePosition() const
+qreal TextB::baseLine() const
       {
       for (QTextBlock tb = doc()->begin(); tb.isValid(); tb = tb.next()) {
             const QTextLayout* tl = tb.layout();
@@ -1055,20 +1130,22 @@ double TextB::lineHeight() const
 //   addSymbol
 //---------------------------------------------------------
 
-void TextB::addSymbol(const SymCode& s)
+void TextB::addSymbol(const SymCode& s, QTextCursor* cur)
       {
+      if (cur == 0)
+            cur = cursor;
 // printf("Text: addSymbol(%x)\n", s.code);
-      QTextCharFormat oFormat = cursor->charFormat();
-      if (s.style >= 0) {
-            QTextCharFormat oFormat = cursor->charFormat();
+      QTextCharFormat oFormat = cur->charFormat();
+      if (s.fontId >= 0) {
+            QTextCharFormat oFormat = cur->charFormat();
             QTextCharFormat nFormat(oFormat);
-            nFormat.setFontFamily(score()->textStyle(s.style)->font().family());
-            cursor->setCharFormat(nFormat);
-            cursor->insertText(s.code);
-            cursor->setCharFormat(oFormat);
+            nFormat.setFontFamily(fontId2font(s.fontId).family());
+            cur->setCharFormat(nFormat);
+            cur->insertText(s.code);
+            cur->setCharFormat(oFormat);
             }
       else
-            cursor->insertText(s.code);
+            cur->insertText(s.code);
       score()->setLayoutAll(true);
       score()->end();
       }
@@ -1122,20 +1199,50 @@ bool TextB::mousePress(const QPointF& p, QMouseEvent* ev)
       if (!setCursor(p))
             return false;
 
-      if (ev->button() == Qt::MidButton) {
-            QString txt = QApplication::clipboard()->text(QClipboard::Selection);
-            cursor->insertText(txt);
-            }
+      if (ev->button() == Qt::MidButton)
+            paste();
       return true;
       }
 
+//---------------------------------------------------------
+//   paste
+//---------------------------------------------------------
+
+void TextB::paste()
+      {
+#ifdef __MINGW32__
+      QString txt = QApplication::clipboard()->text(QClipboard::Clipboard);
+#else
+      QString txt = QApplication::clipboard()->text(QClipboard::Selection);
+#endif
+      if (debugMode)
+            printf("TextB::paste() <%s>\n", qPrintable(txt));
+      cursor->insertText(txt);
+      layout(0);
+      bool lo = (subtype() == TEXT_INSTRUMENT_SHORT) || (subtype() == TEXT_INSTRUMENT_LONG);
+      score()->setLayoutAll(lo);
+      score()->setUpdateAll();
+      score()->end();
+      }
+
 //---------------------------------------------------------
 //   dragAnchor
 //---------------------------------------------------------
 
 QLineF TextB::dragAnchor() const
       {
-      QPointF p1(parent()->abbox().topLeft());
+      QPointF p1;
+
+      if (parent()->type() == MEASURE) {
+            Measure* m     = static_cast<Measure*>(parent());
+            System* system = m->system();
+            double yp      = system->staff(staffIdx())->y() + system->y();
+            double xp      = m->tick2pos(tick()) + m->canvasPos().x();
+            p1 = QPointF(xp, yp);
+            }
+      else {
+            p1 = QPointF(parent()->abbox().topLeft());
+            }
       double tw = width();
       double th = height();
       double x  = 0.0;
@@ -1145,26 +1252,11 @@ QLineF TextB::dragAnchor() const
       else if (_align & ALIGN_VCENTER)
             y = (th * .5);
       else if (_align & ALIGN_BASELINE)
-            y = basePosition();
+            y = baseLine();
       if (_align & ALIGN_RIGHT)
             x = tw;
       else if (_align & ALIGN_HCENTER)
             x = (tw * .5);
-#if 0
-      if (anchor() == ANCHOR_SEGMENT) {
-            Measure* m   = (Measure*) parent();
-            if (m->type() != MEASURE)
-                  abort();
-            Segment* seg = m->tick2segment(tick());
-            if (seg)                            // DEBUG
-                  p1.rx() += seg->x();
-            return QLineF(p1, QPointF(x, y) + canvasPos());
-            }
-      else if (anchor() == ANCHOR_PARENT) {
-            }
-#endif
-            return QLineF(p1, abbox().topLeft());
-//            }
-//      return QLineF(p1, QPointF(x, y) + canvasPos());
+      return QLineF(p1, abbox().topLeft() + QPointF(x, y));
       }
 
