@@ -51,6 +51,10 @@
 #include "barline.h"
 #include "volta.h"
 #include "tuplet.h"
+#include "harmony.h"
+#include "pitchspelling.h"
+#include "part.h"
+#include "beam.h"
 
 extern Measure* tick2measure(int tick);
 
@@ -67,9 +71,9 @@ static const char* undoName[] = {
       "InsertMeasure",     "RemoveMeasure",
       "InsertStaves",      "RemoveStaves",
       "SortStaves",        "ToggleInvisible",
-      "ChangeColor",       "ChangePitch",
+      "ChangeColor",       "ChangePitch", "ChangeTpc",
       "ChangeSubtype",     "AddAccidental",
-      "SetStemDirection",  "FlipSlurDirection",
+      "SetStemDirection",  "FlipSlurDirection", "FlipBeamDirection",
       "ChangeKeySig",
       "ChangeClef",
       "ChangeSig",
@@ -87,7 +91,12 @@ static const char* undoName[] = {
       "SigInsertTime",
       "FixTicks",
       "ChangeBeamMode",
-      "ChangeCopyright"
+      "ChangeCopyright",
+      "TransposeHarmony",
+      "ExchangeVoice",
+      "ChangeConcertPitch",
+      "ChangeInstrumentShort", "ChangeInstrumentLong",
+      "ChangeChordRestLen", "MoveElement"
       };
 
 static bool UNDO = false;
@@ -117,7 +126,6 @@ Undo::Undo(const InputState& is, const Selection* s)
    : selection(*s)
       {
       inputState = is;
-      pitchSpellNeeded = false;
       }
 
 //---------------------------------------------------------
@@ -131,34 +139,33 @@ Undo::Undo(const InputState& is, const Selection* s)
 void Score::doUndo()
       {
       if (debugMode)
-            printf("doUndo\n");
+            printf("doUndo %d redo: %d\n", undoList.size(), redoList.size());
       Selection oSel(*sel);
       InputState oIs(_is);
       sel->deselectAll(this);
-      if (!undoList.isEmpty()) {
-            Undo* u = undoList.back();
-            int n = u->size();
-            for (int i = n-1; i >= 0; --i) {
-                  UndoOp* op = &(*u)[i];
-                  processUndoOp(op, true);
-                  if (op->type == UndoOp::ChangeAccidental) {
-                        // HACK:
-                        // selection is not valid anymore because changeAccidental()
-                        // changes the selected Accidental element
-                        // u->selection.clear();
-                        oSel.clear();
-                        }
-                  }
-            if (u->pitchSpellNeeded)
-                  spell();
-            redoList.push_back(u); // put item on redo list
-            undoList.pop_back();
-            endUndoRedo(u);
-            u->inputState = oIs;
-            u->selection  = oSel;
-            if (debugMode)
-                  printf("  end doUndo\n");
+      if (undoList.isEmpty()) {
+            printf("doUndo: undoList is empty!\n");
+            getAction("undo")->setEnabled(false);
+            return;
             }
+      Undo* u = undoList.back();
+      int n = u->size();
+      for (int i = n-1; i >= 0; --i) {
+            UndoOp* op = &(*u)[i];
+            processUndoOp(op, true);
+            if (op->type == UndoOp::ChangeAccidental) {
+                  // HACK:
+                  // selection is not valid anymore because changeAccidental()
+                  // changes the selected Accidental element
+                  // u->selection.clear();
+                  oSel.clear();
+                  }
+            }
+      redoList.append(u);     // put item on redo list
+      undoList.removeLast();
+      endUndoRedo(u);
+      u->inputState = oIs;
+      u->selection  = oSel;
       }
 
 //---------------------------------------------------------
@@ -188,15 +195,11 @@ void Score::doRedo()
                   u->selection.clear();
                   }
             }
-      if (u->pitchSpellNeeded)
-            spell();
-      undoList.push_back(u); // put item on undo list
-      redoList.pop_back();
+      undoList.append(u); // put item on undo list
+      redoList.removeLast();
       endUndoRedo(u);
       u->inputState = oIs;
       u->selection  = oSel;
-      if (debugMode)
-            printf("  end doRedo\n");
       }
 
 //---------------------------------------------------------
@@ -291,7 +294,7 @@ void Score::processUndoOp(UndoOp* i, bool undo)
                         removeMeasure(i->measure);
                   break;
             case UndoOp::SortStaves:
-                  sortStaves(i->si);
+                  sortStaves(i->di);
                   break;
             case UndoOp::ToggleInvisible:
                   i->element1->setVisible(!i->element1->visible());
@@ -307,8 +310,22 @@ void Score::processUndoOp(UndoOp* i, bool undo)
                   {
                   Note* note = (Note*)(i->element1);
                   int pitch  = note->pitch();
+                  int tpc    = note->tpc();
+                  int userAcc = note->userAccidental();
                   note->changePitch(i->val1);
+                  note->setTpc(i->val2);
+                  note->setUserAccidental(i->val3);
                   i->val1 = pitch;
+                  i->val2 = tpc;
+                  i->val3 = userAcc;
+                  }
+                  break;
+            case UndoOp::ChangeTpc:
+                  {
+                  Note* note = (Note*)(i->element1);
+                  int tpc  = note->tpc();
+                  note->setTpc(i->val1);
+                  i->val1 = tpc;
                   }
                   break;
             case UndoOp::ChangeAccidental:
@@ -339,6 +356,12 @@ void Score::processUndoOp(UndoOp* i, bool undo)
                   slur->setSlurDirection(slur->isUp() ? DOWN : UP);
                   }
                   break;
+            case UndoOp::FlipBeamDirection:
+                  {
+                  Beam* beam = static_cast<Beam*>(i->element1);
+                  beam->setBeamDirection(beam->isUp() ? DOWN : UP);
+                  }
+                  break;
             case UndoOp::ChangeSubtype:
                   {
                   int st = i->element1->subtype();
@@ -375,19 +398,19 @@ void Score::processUndoOp(UndoOp* i, bool undo)
                   KeyList* kl = i->staff->keymap();
                   if (undo) {
                         // remove new value if there is any
-                        if (i->val3 != -1000) {
+                        if (i->val3 != NO_KEY) {
                               iKeyEvent ik = kl->find(i->val1);
                               kl->erase(ik);
                               }
-                        if (i->val2 != -1000)
+                        if (i->val2 != NO_KEY)
                               (*kl)[i->val1] = i->val2;
                         }
                   else {
-                        if (i->val2 != -1000) {
+                        if (i->val2 != NO_KEY) {
                               iKeyEvent ik = kl->find(i->val1);
                               kl->erase(ik);
                               }
-                        if (i->val3 != -1000)
+                        if (i->val3 != NO_KEY)
                               (*kl)[i->val1] = i->val3;
                         }
                   }
@@ -395,7 +418,7 @@ void Score::processUndoOp(UndoOp* i, bool undo)
 
             case UndoOp::ChangeClef:
                   {
-                  ClefList* kl = i->staff->clef();
+                  ClefList* kl = i->staff->clefList();
                   if (undo) {
                         // remove new value if there is any
                         if (i->val3 != NO_CLEF) {
@@ -482,12 +505,12 @@ void Score::processUndoOp(UndoOp* i, bool undo)
             case UndoOp::ChangeMeasureLen:
                   {
                   Measure* m = (Measure*)i->measure;
-                  int ol     = m->tickLen();
+                  int ol     = i->val2;
                   int nl     = i->val1;
                   m->setTickLen(nl);
 
                   //
-                  // move EndBarLin and TimeSigAnnounce
+                  // move EndBarLine and TimeSigAnnounce
                   // to end of measure:
                   //
                   int staves = nstaves();
@@ -503,6 +526,7 @@ void Score::processUndoOp(UndoOp* i, bool undo)
                               }
                         }
                   i->val1 = ol;
+                  i->val2 = nl;
                   }
                   break;
 
@@ -534,8 +558,8 @@ void Score::processUndoOp(UndoOp* i, bool undo)
                   {
                   Volta* volta = (Volta*)i->element1;
                   QList<int> l = volta->endings();
-                  volta->setEndings(i->si);
-                  i->si = l;
+                  volta->setEndings(i->di);
+                  i->di = l;
                   }
                   break;
             case UndoOp::ChangeVoltaText:
@@ -594,6 +618,74 @@ void Score::processUndoOp(UndoOp* i, bool undo)
                   i->s = s;
                   }
                   break;
+            case UndoOp::TransposeHarmony:
+                  {
+                  Harmony* h = static_cast<Harmony*>(i->element1);
+                  int semitones = i->val1;
+                  if (undo)
+                        semitones = -semitones;
+                  int baseTpc = h->baseTpc();
+                  int rootTpc = h->rootTpc();
+                  h->setBaseTpc(transposeTpc(baseTpc, semitones));
+                  h->setRootTpc(transposeTpc(rootTpc, semitones));
+                  h->buildText();
+                  }
+                  break;
+            case UndoOp::ExchangeVoice:
+                  {
+                  Measure* m = static_cast<Measure*>(i->element1);
+                  if (undo)
+                        m->exchangeVoice(i->val2, i->val1, i->val3, i->val4);
+                  else
+                        m->exchangeVoice(i->val1, i->val2, i->val3, i->val4);
+                  }
+                  break;
+            case UndoOp::ChangeConcertPitch:
+                  {
+                  int oval = int(_style->concertPitch);
+                  _style->concertPitch = bool(i->val1);
+                  QAction* action = getAction("concert-pitch");
+                  action->setChecked(_style->concertPitch);
+                  i->val1 = oval;
+                  }
+                  break;
+            case UndoOp::ChangeInstrumentShort:
+                  {
+                  QString s = i->part->shortNameHtml();
+                  i->part->setShortNameHtml(i->s);
+                  i->s = s;
+                  }
+                  break;
+            case UndoOp::ChangeInstrumentLong:
+                  {
+                  QString s = i->part->longNameHtml();
+                  i->part->setLongNameHtml(i->s);
+                  i->s = s;
+                  }
+                  break;
+            case UndoOp::ChangeChordRestLen:
+                  {
+                  ChordRest* cr = static_cast<ChordRest*>(i->element1);
+                  int oldLen = cr->tickLen();
+                  cr->setLen(i->val1);
+                  i->val1 = oldLen;
+                  }
+                  break;
+            case UndoOp::MoveElement:
+                  {
+                  QPointF po = i->element1->userOff();
+                  i->element1->setUserOff(i->pt);
+                  i->pt = po;
+                  }
+                  break;
+            case UndoOp::ChangeBracketSpan:
+                  {
+                  int column = i->val1;
+                  int oSpan  = i->staff->bracketSpan(column);
+                  i->staff->setBracketSpan(column, i->val2);
+                  i->val2 = oSpan;
+                  }
+                  break;
             }
       UNDO = FALSE;
       }
@@ -608,24 +700,27 @@ void Score::processUndoOp(UndoOp* i, bool undo)
 
 void Score::endUndoRedo(Undo* undo)
       {
-      getAction("undo")->setEnabled(!undoList.empty());
-      getAction("redo")->setEnabled(!redoList.empty());
       setDirty(true);
 
       _is = undo->inputState;
+      emit posChanged(_is.pos());
 
       if (!noteEntryMode()) {
             // no input state
             canvas()->setState(Canvas::NORMAL);
-            mscore->setState(STATE_NORMAL);
+            setState(STATE_NORMAL);
             }
       else {
             // input state
             canvas()->setState(Canvas::NOTE_ENTRY);
-            mscore->setState(STATE_NOTE_ENTRY);
+            setState(STATE_NOTE_ENTRY);
             }
+
       *sel = undo->selection;
       sel->update();
+
+      getAction("undo")->setEnabled(!undoEmpty());
+      getAction("redo")->setEnabled(!redoEmpty());
       }
 
 //---------------------------------------------------------
@@ -642,6 +737,8 @@ void Score::checkUndoOp()
             fprintf(stderr, "undoOp: cmd not started\n");
             if (debugMode)
                   abort();
+            undoList.push_back(new Undo(_is, sel));
+            cmdActive = true;
             }
       if (UNDO) {
             fprintf(stderr, "create undo op in undo/redo operation\n");
@@ -719,15 +816,6 @@ void Score::undoFixTicks()
       processUndoOp(&undoList.back()->back(), false);
       }
 
-//---------------------------------------------------------
-//   undoSetPitchSpellNeeded
-//---------------------------------------------------------
-
-void Score::undoSetPitchSpellNeeded()
-      {
-      undoList.back()->pitchSpellNeeded = true;
-      }
-
 //---------------------------------------------------------
 //   undoRemoveElement
 //---------------------------------------------------------
@@ -743,7 +831,7 @@ void Score::undoRemoveElement(Element* element)
             foreach(Slur* slur, cr->slurBack()) {
                   undoRemoveElement(slur);
                   }
-            if (cr->tuplet() && cr->tuplet()->elements()->empty())
+            if (cr->tuplet() && cr->tuplet()->elements().empty())
                   undoRemoveElement(cr->tuplet());
             }
       checkUndoOp();
@@ -758,13 +846,14 @@ void Score::undoRemoveElement(Element* element)
 //   undoChangeMeasureLen
 //---------------------------------------------------------
 
-void Score::undoChangeMeasureLen(Measure* m, int tick)
+void Score::undoChangeMeasureLen(Measure* m, int oldTicks, int newTicks)
       {
       checkUndoOp();
       UndoOp i;
       i.type     = UndoOp::ChangeMeasureLen;
       i.measure  = m;
-      i.val1     = tick;
+      i.val1     = newTicks;
+      i.val2     = oldTicks;
       undoList.back()->push_back(i);
       processUndoOp(&undoList.back()->back(), false);
       }
@@ -818,13 +907,30 @@ void Score::undoChangeNoteHead(Note* note, int group)
 //   undoChangePitch
 //---------------------------------------------------------
 
-void Score::undoChangePitch(Note* note, int pitch)
+void Score::undoChangePitch(Note* note, int pitch, int tpc, int userAccidental)
       {
       checkUndoOp();
       UndoOp i;
       i.type     = UndoOp::ChangePitch;
       i.element1 = note;
       i.val1     = pitch;
+      i.val2     = tpc;
+      i.val3     = userAccidental;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
+//---------------------------------------------------------
+//   undoChangeTpc
+//---------------------------------------------------------
+
+void Score::undoChangeTpc(Note* note, int tpc)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type     = UndoOp::ChangeTpc;
+      i.element1 = note;
+      i.val1     = tpc;
       undoList.back()->push_back(i);
       processUndoOp(&undoList.back()->back(), false);
       }
@@ -844,6 +950,21 @@ void Score::undoChangeBeamMode(ChordRest* cr, int mode)
       processUndoOp(&undoList.back()->back(), false);
       }
 
+//---------------------------------------------------------
+//   undoChangeChordRestLen
+//---------------------------------------------------------
+
+void Score::undoChangeChordRestLen(ChordRest* cr, int len)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type     = UndoOp::ChangeChordRestLen;
+      i.element1 = cr;
+      i.val1     = len;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
 //---------------------------------------------------------
 //   undoChangeEndBarLineType
 //---------------------------------------------------------
@@ -888,6 +1009,114 @@ void Score::undoChangeCopyright(const QString& s)
       processUndoOp(&undoList.back()->back(), false);
       }
 
+//---------------------------------------------------------
+//   undoTransposeHarmony
+//---------------------------------------------------------
+
+void Score::undoTransposeHarmony(Harmony* h, int semitones)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type  = UndoOp::TransposeHarmony;
+      i.element1 = h;
+      i.val1     = semitones;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
+//---------------------------------------------------------
+//   undoExchangeVoice
+//---------------------------------------------------------
+
+void Score::undoExchangeVoice(Measure* measure, int val1, int val2, int staff1, int staff2)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type  = UndoOp::ExchangeVoice;
+      i.element1 = measure;
+      i.val1     = val1;
+      i.val2     = val2;
+      i.val3     = staff1;
+      i.val4     = staff2;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
+//---------------------------------------------------------
+//   undoRemovePart
+//---------------------------------------------------------
+
+void Score::undoRemovePart(Part* part, int idx)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type = UndoOp::RemovePart;
+      i.part = part;
+      i.val1  = idx;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
+//---------------------------------------------------------
+//   undoInsertPart
+//---------------------------------------------------------
+
+void Score::undoInsertPart(Part* part, int idx)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type = UndoOp::InsertPart;
+      i.part = part;
+      i.val1  = idx;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
+//---------------------------------------------------------
+//   undoRemoveStaff
+//---------------------------------------------------------
+
+void Score::undoRemoveStaff(Staff* staff, int idx)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type = UndoOp::RemoveStaff;
+      i.staff = staff;
+      i.val1  = idx;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
+//---------------------------------------------------------
+//   undoInsertStaff
+//---------------------------------------------------------
+
+void Score::undoInsertStaff(Staff* staff, int idx)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type = UndoOp::InsertStaff;
+      i.staff = staff;
+      i.val1  = idx;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
+//---------------------------------------------------------
+//   undoMove
+//---------------------------------------------------------
+
+void Score::undoMove(Element* e, const QPointF& pt)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type     = UndoOp::MoveElement;
+      i.element1 = e;
+      i.pt       = pt;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
 //---------------------------------------------------------
 //   undoOp
 //---------------------------------------------------------
@@ -916,6 +1145,7 @@ void Score::undoOp(UndoOp::UndoType type, Segment* seg, int staff)
       undoList.back()->push_back(i);
       }
 
+#if 0
 void Score::undoOp(UndoOp::UndoType type, Part* part, int idx)
       {
       checkUndoOp();
@@ -935,6 +1165,7 @@ void Score::undoOp(UndoOp::UndoType type, Staff* staff, int idx)
       i.val1   = idx;
       undoList.back()->push_back(i);
       }
+#endif
 
 void Score::undoOp(UndoOp::UndoType type, Measure* m, MStaff* s, int staff)
       {
@@ -967,12 +1198,12 @@ void Score::undoOp(UndoOp::UndoType type, Measure* m, int a, int b)
       undoList.back()->push_back(i);
       }
 
-void Score::undoOp(QList<int> si)
+void Score::undoOp(QList<int>& di)
       {
       checkUndoOp();
       UndoOp i;
       i.type = UndoOp::SortStaves;
-      i.si   = si;
+      i.di   = di;
       undoList.back()->push_back(i);
       }
 
@@ -1080,7 +1311,7 @@ void Score::undoChangeVoltaEnding(Volta* volta, const QList<int>& l)
       UndoOp i;
       i.type = UndoOp::ChangeVoltaEnding;
       i.element1 = volta;
-      i.si   = l;
+      i.di   = l;
       undoList.back()->push_back(i);
       processUndoOp(&undoList.back()->back(), false);
       }
@@ -1115,6 +1346,22 @@ void Score::undoChangeChordRestSize(ChordRest* cr, bool small)
       processUndoOp(&undoList.back()->back(), false);
       }
 
+//---------------------------------------------------------
+//   undoChangeBracketSpan
+//---------------------------------------------------------
+
+void Score::undoChangeBracketSpan(Staff* staff, int column, int span)
+      {
+      checkUndoOp();
+      UndoOp i;
+      i.type  = UndoOp::ChangeBracketSpan;
+      i.staff = staff;
+      i.val1  = column;
+      i.val2  = span;
+      undoList.back()->push_back(i);
+      processUndoOp(&undoList.back()->back(), false);
+      }
+
 //---------------------------------------------------------
 //   addElement
 //---------------------------------------------------------
@@ -1209,49 +1456,67 @@ void Score::removeElement(Element* element)
       // special for MEASURE, HBOX, VBOX
       // their parent is not static
 
-      if (element->type() == MEASURE || element->type() == HBOX || element->type() == VBOX) {
+      if (element->type() == MEASURE
+         || (element->type() == HBOX && parent->type() != VBOX)
+         || element->type() == VBOX) {
             _layout->remove(element);
             return;
             }
       parent->remove(element);
-      if (element->type() == CLEF) {
-            Clef* clef   = (Clef*)element;
-            int tick     = clef->tick();
-            int staffIdx = clef->staffIdx();
 
-            //-----------------------------------------------
-            //   move notes
-            //-----------------------------------------------
+      switch(element->type()) {
+            case CHORD:
+            case REST:
+                  {
+                  ChordRest* cr = static_cast<ChordRest*>(element);
+                  cr->setBeam(0);
+                  }
+                  break;
+            case CLEF:
+                  {
+                  Clef* clef   = static_cast<Clef*>(element);
+                  int tick     = clef->tick();
+                  int staffIdx = clef->staffIdx();
 
-            bool endFound = false;
-            for (MeasureBase* mb = _layout->first(); mb; mb = mb->next()) {
-                  if (mb->type() != MEASURE)
-                        continue;
-                  Measure* measure = (Measure*)mb;
-                  for (Segment* segment = measure->first(); segment; segment = segment->next()) {
-                        int startTrack = staffIdx * VOICES;
-                        int endTrack   = startTrack + VOICES;
-                        for (int track = startTrack; track < endTrack; ++track) {
-                              Element* ie = segment->element(track);
-                              if (ie && ie->type() == CLEF && ie->tick() > tick) {
-                                    endFound = true;
-                                    break;
+                  //-----------------------------------------------
+                  //   move notes
+                  //-----------------------------------------------
+
+                  bool endFound = false;
+                  for (MeasureBase* mb = _layout->first(); mb; mb = mb->next()) {
+                        if (mb->type() != MEASURE)
+                              continue;
+                        Measure* measure = static_cast<Measure*>(mb);
+                        for (Segment* segment = measure->first(); segment; segment = segment->next()) {
+                              int startTrack = staffIdx * VOICES;
+                              int endTrack   = startTrack + VOICES;
+                              for (int track = startTrack; track < endTrack; ++track) {
+                                    Element* ie = segment->element(track);
+                                    if (ie && ie->type() == CLEF && ie->tick() > tick) {
+                                          endFound = true;
+                                          break;
+                                          }
                                     }
+                              if (endFound)
+                                    break;
                               }
                         if (endFound)
                               break;
                         }
-                  if (endFound)
-                        break;
                   }
-            }
-      else if (element->type() == KEYSIG) {
-            layoutAll = true;
-            }
-      else if (element->type() == SLUR) {
-            Slur* s = (Slur*)element;
-            ((ChordRest*)s->startElement())->removeSlurFor(s);
-            ((ChordRest*)s->endElement())->removeSlurBack(s);
+                  break;
+            case KEYSIG:
+                  layoutAll = true;
+                  break;
+            case SLUR:
+                  {
+                  Slur* s = static_cast<Slur*>(element);
+                  static_cast<ChordRest*>(s->startElement())->removeSlurFor(s);
+                  static_cast<ChordRest*>(s->endElement())->removeSlurBack(s);
+                  }
+                  break;
+            default:
+                  break;
             }
       }
 
