@@ -23,6 +23,17 @@
 
 #include "globals.h"
 
+//---------------------------------------------------------
+//   cycles
+//---------------------------------------------------------
+
+static inline unsigned long long cycles()
+      {
+      unsigned long long rv;
+      __asm__ __volatile__("rdtsc" : "=A" (rv));
+      return rv;
+      }
+
 class Measure;
 class Segment;
 class System;
@@ -32,12 +43,17 @@ extern QRectF drawHandle(QPainter& p, const QPointF& pos, bool active);
 extern QRectF handleRect(const QPointF& pos);
 
 extern int getStaff(System* system, const QPointF& p);
-extern int headType(int tickLen, DurationType* type, int* dots);
 extern int pitchKeyAdjust(int note, int key);
 extern int line2pitch(int line, int clef, int key);
 extern int y2pitch(double y, int clef);
 extern int quantizeLen(int, int, int);
 extern double curTime();
+extern void selectNoteMessage();
+extern void selectNoteRestMessage();
+extern void selectNoteSlurMessage();
+extern void selectStavesMessage();
+extern QString pitch2string(int v);
+
 
 #endif
 
